package epe.agents.acquaintances;

	import java.util.Vector;
	
	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.gui.links.Link;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.messages.contract.CancellingCooperationMessage;


public class AcquaintancesManager extends Object {
  
	  protected Agent owner;
	  protected GUI gui;
	  protected Acquaintances acquaintances;
	  protected Vector<Object> updatedListeners = new Vector<Object>();
	  
	  // Keep only a few neutral acquaintances if the policy is enbales
	  // When removing acquaintances, ensure the connectivity is not lost
	  protected int maxNumberOfNeutralAcquaintances = 2;
	  protected String connectivityAgent = "AgentIn";
	  protected boolean enableAcquaintancesPolicy = false;

  public AcquaintancesManager(Agent owner) {
    this.owner = owner;
    gui = owner.getGUI();
    acquaintances = new Acquaintances(gui);
    gui.println("AcquaintancesManager created");
  }
  
    public void addAcquaintance(Acquaintance acquaintance) {
    // The validity of the add operation is also tested in the specificAgent sub classes
    if (!acquaintances.isPresentAcquaintance(acquaintance)
        | acquaintance.getAgentInterface().getType() == "OUT") {
      Agent agent = acquaintance.getAgent();
      String type = acquaintance.getInterfaceType();
      int position = acquaintance.getInterfacePosition();
      
      
      // ###KEEP_DEBUG : acqs showing during addAcquaintance (initial)
//      if ((acquaintance.getAgentInterface().getType() == "IN")
//        |(acquaintance.getAgentInterface().getType() == "OUT"))
//        showAllAcquaintances();
      //end DEBUG
      acquaintances.addAcquaintance(acquaintance);
      // ###KEEP_DEBUG : acqs showing during addAcquaintance (final)
//      if ((acquaintance.getAgentInterface().getType() == "IN")
//        |(acquaintance.getAgentInterface().getType() == "OUT"))
//        showAllAcquaintances();
      //end DEBUG
      
      // Limit the number of acquaintances
      if (enableAcquaintancesPolicy) 
        applyAcquaintancesPolicy();
            
      if (type.equals("IN")) {
        int numberOfINs = owner.getNumberOfINs();
        owner.getBackground().getLinks().addLink(new Link(agent, owner, position, numberOfINs));
        owner.notifyReorganizationListenersOfAddLink("IN",
        																						 agent.getAgentName(), 
                                                     owner.getAgentName());
        // ###KEEP_DEBUG : verify the visual representation of the links
//        gui.println("["+ owner.getAgentName()
//                  +"] : DEBUG : backgroundLink created: "+
//                  agent.getAgentName()+"/"+
//                  owner.getAgentName()+"/"+
//                  position+"/"+numberOfINs, GUI.red);
      }/*
      else if (type.equals("OUT")) {
        owner.getBackground().getLinks().addLink(new Link(owner, agent));
        owner.notifyReorganizationListenersOfAddLink(owner.getAgentName(), 
                                                     agent.getAgentName());
      }*/
      
      notifyUpdatedListeners(acquaintance);
      owner.getBackground().linksChanged();
    }
    else
      gui.println("["+ owner.getAgentName()
                  +"] : WARNING : acqaintance to be added already present."
                  , GUI.red);
  }

  public String addAcquaintanceWithReturn(Acquaintance acquaintance) {
    System.out.println("ERROR : g?n?ric addAcquaintance should not be called.");
    return "ERROR : generic addAcquaintance should not be called";
  }
    
  public void addAcquaintance(Agent agent, String type, int position) {
    Acquaintance acq = new Acquaintance(agent);
    acq.setBelievedCompetence(new BelievedCompetence());
    acq.setTrustValue(new TrustValue(new AgentInterface(type)));
    acq.setAgentInterface(new AgentInterface(type, position));
    addAcquaintance(acq);
  }
  
  public String addAcquaintanceWithReturn(Agent agent, 
  																				String type, 
  																				int myPosition, 
  																				int positionOnAcq) {
    String ownNeededDataTypeOnIN = "";
    String acqNeededDataTypeOnOUT = "";
    String ownNeededDataTypeOnOUT = "";
    String acqNeededDataTypeOnIN = "";
    
    // Check for data types
    if (type.equals("IN")) {
      ownNeededDataTypeOnIN = owner.getReorganizationManager().dataTypeNeeded("IN", myPosition);
      acqNeededDataTypeOnOUT = agent.getReorganizationManager().dataTypeNeeded("OUT", 1);
    } else  { // "OUT"
      ownNeededDataTypeOnOUT = owner.getReorganizationManager().dataTypeNeeded("OUT", 1);
      acqNeededDataTypeOnIN = agent.getReorganizationManager()
      														 .dataTypeNeeded("IN", positionOnAcq);
    }
    
    if (type.equals("IN"))
      if (!(owner.getReorganizationManager().isDataRelevant(type, 
                           myPosition, 
                           acqNeededDataTypeOnOUT))) {
      return owner.getAgentName() + " : ERROR : Acquaintance [ " 
                 + agent.getAgentName()
                 + " ] not added to Acquaintances : data type missmatch : "
                 + ownNeededDataTypeOnIN + "/"
                 + acqNeededDataTypeOnOUT;
      }
      else if (type.equals("OUT"))
        if (!(owner.getReorganizationManager().isDataRelevant(type, 
                           myPosition, 
                           acqNeededDataTypeOnIN))) {
          return owner.getAgentName() + " : ERROR : Acquaintance [ " 
                 + agent.getAgentName()
                 + " ] not added to Acquaintances : data type missmatch : "
                 + ownNeededDataTypeOnOUT + "/"
                 + acqNeededDataTypeOnIN;
        }
         
      else {
        SystemPrinter.printSystem("ERROR : " + owner.getAgentName());
        owner.errorHappened();
        gui.println(owner.getAgentName() +" : ERROR : Unkown case, should not happen." 
                 , GUI.red);
        return "ERROR";
      }
    
    // build Acquaintance
    Acquaintance acq = new Acquaintance(agent);
    acq.setBelievedCompetence(new BelievedCompetence());
    acq.setTrustValue(new TrustValue(new AgentInterface(type)));
    acq.setAgentInterface(new AgentInterface(type, myPosition));
    if (type.equals("IN"))
      acq.setDataType(AgentInterface
      	 .lessGenericDataType(ownNeededDataTypeOnIN, acqNeededDataTypeOnOUT));
    else // "OUT"
      acq.setDataType(AgentInterface.lessGenericDataType(ownNeededDataTypeOnOUT, 
      																									 acqNeededDataTypeOnIN));

    return addAcquaintanceWithReturn(acq);
  }
  
  public void addAcquaintance(Agent agent, String type, int position, String dataType) {
    Acquaintance acq = new Acquaintance(agent);
    acq.setBelievedCompetence(new BelievedCompetence());
    acq.setTrustValue(new TrustValue(new AgentInterface(type)));
    acq.setAgentInterface(new AgentInterface(type, position, dataType));
    addAcquaintance(acq);
  }
  
  public void addAcquaintance(Agent agent, String type, int position, boolean fixedTrustValue) {
    Acquaintance acq = new Acquaintance(agent);
    acq.setBelievedCompetence(new BelievedCompetence());
    acq.setTrustValue(new TrustValue(new AgentInterface(type)));
    acq.setAgentInterface(new AgentInterface(type, position));
    acq.setFixedTrustValue(fixedTrustValue);
    addAcquaintance(acq);
  }
  
  public void addAcquaintance(Agent agent, String type, 
                              int position, 
                              String dataType, 
                              boolean fixedTrustValue) {
    Acquaintance acq = new Acquaintance(agent);
    acq.setBelievedCompetence(new BelievedCompetence());
    acq.setTrustValue(new TrustValue(new AgentInterface(type)));
    acq.setAgentInterface(new AgentInterface(type, position, dataType));
    acq.setFixedTrustValue(fixedTrustValue);
    addAcquaintance(acq);
  }
  
  public void removeAcquaintance(Acquaintance acquaintance) {
    acquaintances.removeAcquaintance(acquaintance);
    gui.println("Acquaintance [ " 
                  + acquaintance.getAgent().getAgentName()
                  + " ] removed from Acquaintances [ "
                  + acquaintance.getAgentInterface().getType()
                  + acquaintance.getAgentInterface().getPosition()
                  + " ].", GUI.blue);
    notifyUpdatedListeners(acquaintance);
  }
   
  public void removeAllAcquaintances() {
    while (getAcquaintances().size()!=0) {
      Acquaintance acq = getAcquaintances().getAcquaintance(0);
      cancelCooperation(acq);
      removeAcquaintance(acq);
    }
    
    gui.println("All Acquaintances removed.", GUI.blue);
  }
  
  public synchronized void cancelCooperation(Acquaintance acq) {
    synchronized(owner) {
      // ###KEEP_DEBUG : acqs showing during cooperation cancelling (initial)
//        showAllAcquaintances();
      //end DEBUG
      String type = acq.getInterfaceType();  
      if (type.equals("IN")) {
        owner.getBackground().getLinks().removeLink(new Link(acq.getAgent(), 
                                                             owner, 
                                                             acq.getInterfacePosition()));
        owner.notifyReorganizationListenersOfRemoveLink("IN", acq.getAgent().getAgentName(), 
                                                       owner.getAgentName());
      }
      else if (type.equals("OUT")) {
        // not needed anymore cause only the receiverAgent draws the links,
        /*owner.getBackground().getLinks().removeLink(new Link(owner, 
                                                             acq.getAgent(),
                                                             acq.getInterfacePosition()));*/
        owner.notifyReorganizationListenersOfRemoveLink("OUT", owner.getAgentName(),
                                                        acq.getAgent().getAgentName());
      }
      owner.getBackground().linksChanged();
      // Insure there will not be two NEUTRAL instances of the same agent
      if (isANeutralAcquaintance(acq.getAgent())) {
        removeAcquaintance(acq);
        gui.println("Acquaintance removed cause already NEUTRAL", GUI.blue);
      }
      else {
        acq.setAgentInterfaceToNeutral();
        gui.println("Acquaintance's interface set to NEUTRAL", GUI.blue);
      }
      // Decrease NEUTRAL trustValue of that agent because cooperative work failled 
      // => ensure some other more trustworthy agent is chosen next time
      // the agent has to choose between two relevant potential agents.
      decreaseNeutralAgentTrustValue(acq.getAgent());
      
      // ###DEBUG : test reinitialization of inputs and waiting
      owner.cooperationWithInAcquaintanceMayHaveBeenCancelled(acq);
      
      owner.checkCooperativeState();
      notifyUpdatedListeners(acq);
      // ###KEEP_DEBUG : acqs showing during cooperation cancelling (final)
//        showAllAcquaintances();
      //end DEBUG
    }
  }
  
  public void applyAcquaintancesPolicy() {
    if (acquaintances.numberOfNeutralAcquaintances() > maxNumberOfNeutralAcquaintances) {
        removeOldestNeutralAcquaintance();
    }
  }
    
  public void removeOldestNeutralAcquaintance() {
    boolean deleted = false;
    Vector<Object> pos = acquaintances.findAcquaintance("NEUTRAL");
    int i = 0;
    if (pos.size()>0)
      while (!deleted & i<pos.size()) {
        int rang = ((Integer)pos.elementAt(i)).intValue();
        Acquaintance acq = acquaintances.getAcquaintance(rang);
        if (!(acq.getAgent().getAgentName().equals(connectivityAgent))) {
          removeAcquaintance(acq);
          deleted = true;
        } else {
         i++ ;
        }
          
      }
  }
  
  public Acquaintances getAcquaintances() {
    return acquaintances;
  }

  public void addAcquaintancesUpdatedListener(AcquaintancesUpdatedListener listener) {
    updatedListeners.addElement(listener);
  }

  public void removeAcquaintancesUpdatedListener(AcquaintancesUpdatedListener listener) {
    updatedListeners.removeElement(listener);
  }

  public void notifyUpdatedListeners(Acquaintance acq) {
    for (int i=0; i < updatedListeners.size(); i++)
      ((AcquaintancesUpdatedListener)(updatedListeners.elementAt(i))).acquaintancesUpdated(acq);
  }
  
  public void decreaseTrustValue(AgentInterface interfaceType, String decreaseType) {
    decreaseTrustValueWithFeedback(interfaceType, decreaseType);
  }
  
  public String decreaseTrustValueWithFeedback(AgentInterface interfaceType, 
  																						 String decreaseType) {
    String feedback = "";
    boolean acquaintancesModified = false;
    Vector<Object> positions = acquaintances.findAcquaintance(interfaceType);
    Acquaintance acq;
    int i=0;
    while (i < positions.size()) {
      if (acquaintancesModified) {
        // If an acquaintances has been changed, the loop must take notice
        positions = acquaintances.findAcquaintance(interfaceType);
        acquaintancesModified = false;
      }
      else {
        int rang = ((Integer)positions.elementAt(i)).intValue();
        acq = acquaintances.getAcquaintance(rang);
        acq.decreaseTrustValue(decreaseType);
        gui.println("decreasing trustValue", GUI.orange);
        if (!(acq.trustingForCooperation())) {
          Agent adressee = acq.getAgent();
          AgentInterface type = new AgentInterface(acq.getInterfaceType());  
          CancellingCooperationMessage message = new CancellingCooperationMessage(owner, type);
          owner.fireMessage(message, adressee, acq.getInterfaceType(), owner); 
          gui.println("AcquaintancesManager : acquaintance ["
                    + acq.getAgent().getAgentName() + ":"
                    + acq.getInterfaceType() + ":"
                    + acq.getInterfacePosition() + ":"
                    + acq.getDataType() + "]"
                    + " is no longer trustworthy for cooperation : "
                    + " setting to NEUTRAL", GUI.blue);
          cancelCooperation(acq);
          // For the UNIQUE/MULTIPLE Link management
          owner.getReorganizationManager()
          		 .getLinkMultiplicityManager()
          		 .manageLinkMultiplicityUpdate(interfaceType);
          owner.detectInOutNCSs();
          acquaintancesModified = true;
          feedback = "cooperationCancelled";
        } else {
          i++;
        }
        
        if (!(acq.trustingAsAcquaintance())) {
          removeAcquaintance(acq);
          gui.println("AcquaintancesManager : acquaintance ["
                    + acq.getAgent().getAgentName() + ":"
                    + acq.getInterfaceType() + ":"
                    + acq.getInterfacePosition() + ":"
                    + acq.getDataType() + "]"
                    + " is no longer trustworthy as an acquaintance : "
                    + " eliminating from acquaintances", GUI.blue);
          acquaintancesModified = true;
        }
      }
    }
    return feedback;
  }
  
  public void decreaseNeutralAgentTrustValue(Agent neutralAgent) {
		Vector position = acquaintances.findAcquaintance(neutralAgent, "NEUTRAL");
		if (position.size() != 1) {
			gui.println("DEBUG : more than 1 NEUTRAL or no NEUTRAL found", GUI.red);
			gui.errorNotification();
		}
	  else {
			int rang = ((Integer)position.elementAt(0)).intValue();
			Acquaintance acq = acquaintances.getAcquaintance(rang);
			acq.decreaseTrustValue("neutral");
			gui.println("decreasing NEUTRAL trustValue", GUI.orange);
    
			if (!(acq.trustingAsAcquaintance())
					&& (!isAnInOutAcquaintance(acq.getAgent(), "IN"))
					&& (!isAnInOutAcquaintance(acq.getAgent(), "OUT"))) {
			  removeAcquaintance(acq);
			  gui.println("AcquaintancesManager : acquaintance ["
						+ acq.getAgent().getAgentName() + ":"
						+ acq.getInterfaceType() + ":"
						+ acq.getInterfacePosition() + ":"
						+ acq.getDataType() + "]"
						+ " is no longer trustworthy as an acquaintance : "
						+ " ELIMINATING from acquaintances", GUI.blue);
			}
	  }
  }
  
  public void decreaseAllTrustValues(String decreaseType) {
    decreaseTrustValue(new AgentInterface("IN"), decreaseType);
    decreaseTrustValue(new AgentInterface("OUT"), decreaseType);
  }
  
  public void cancelOneOutAcquaintance() {
    Vector<Object> positions = acquaintances.findAcquaintance(new AgentInterface("OUT"));
    Acquaintance acq;
    int rang = ((Integer)positions.elementAt(0)).intValue();
    acq = acquaintances.getAcquaintance(rang);
	  Agent adressee = acq.getAgent();
	  AgentInterface type = new AgentInterface(acq.getInterfaceType());  
	  CancellingCooperationMessage message = new CancellingCooperationMessage(owner, type);
	  owner.fireMessage(message, adressee, acq.getInterfaceType(), owner); 
	  gui.println("AcquaintancesManager : acquaintance ["
	            + acq.getAgent().getAgentName() + ":"
	            + acq.getInterfaceType() + ":"
	            + acq.getInterfacePosition() + ":"
	            + acq.getDataType() + "]"
	            + " belongs to a multipleOUT and has been chosen to be cancelled : "
	            + " setting to NEUTRAL", GUI.blue);
	  cancelCooperation(acq);
	  // For the UNIQUE/MULTIPLE Link management
	  owner.getReorganizationManager()
	  		 .getLinkMultiplicityManager()
	  		 .manageLinkMultiplicityUpdate(new AgentInterface("OUT"));
  }
  
  public void cancelOutAcquaintances() {
    while (possessingOUT())
    	cancelOneOutAcquaintance();
  }
  
  public void increaseTrustValue(AgentInterface interfaceType, String increaseType) {
    Vector<Object> positions = acquaintances.findAcquaintance(interfaceType);
    Acquaintance acq;
    for (int i=0; i<positions.size(); i++) {
      int rang = ((Integer)positions.elementAt(i)).intValue();
      acq = acquaintances.getAcquaintance(rang);
      acq.increaseTrustValue(increaseType);
    }
  }
  
  public void increaseAllTrustValues(String increaseType) {
    increaseTrustValue(new AgentInterface("IN"), increaseType);
    increaseTrustValue(new AgentInterface("OUT"), increaseType);
  }
  
  public boolean isAnAcquaintance(Agent agent) {
    return (acquaintances.findAcquaintance(agent).size()!=0);
  }
  
  public boolean isAnInOutAcquaintance(Agent agent, String type) {
    return (acquaintances.findAcquaintance(agent, type).size()!=0);
  }
  
  public boolean isANeutralAcquaintance(Agent agent) {
    return (acquaintances.findAcquaintance(agent, "NEUTRAL").size()!=0);
  }
  
  public void showAllAcquaintances() {
    Vector<Object> positions = acquaintances.getAllAcquaintances();
    if (positions.size()==0)
      gui.println(owner.getAgentName() +" has no Acquaintances.", GUI.blue);
    else {
      Acquaintance acq;
      gui.println("Agent" + owner.getAgentName() +"'s Acquaintances " +
      						"[Interface(position): " +
      						"Agent's name : " +
      						"DataType / Multiplicity]", GUI.blue);
      for (int i=0; i<positions.size(); i++) {
        int rang = ((Integer)positions.elementAt(i)).intValue();
        acq = acquaintances.getAcquaintance(rang);
        gui.println("	[" + acq.getInterfaceType() +
        						acq.getInterfacePosition() +
        						": " + acq.getAgent().getAgentName() +  
                  	" : " + acq.getDataType() + "/" +
        						acq.getLinkMultiplicity() +
                  	"]", GUI.blue);
      }
    }
  }
  
  public boolean possessingOUT() {
    Vector<Object> position = acquaintances.findAcquaintance(new AgentInterface("OUT"));
    return (position.size() >= 1);
  }
  
  public boolean possessingMultipleOUT() {
    Vector<Object> position = acquaintances.findAcquaintance(new AgentInterface("OUT", 1));
    return (position.size() > 1);
  }
  
  public boolean possessingINs() {
    Vector<Object> position = acquaintances.findAcquaintance(new AgentInterface("IN"));
    return (position.size() > 1);
  }

  public boolean possessingIN(int pos) {
    Vector<Object> position = acquaintances.findAcquaintance(new AgentInterface("IN", pos));
    return (position.size() >= 1);
  }
  
  // Overriden by specificAgentsAcqManagers
  public Agent getAgentIN1() {
    return null;
  }
  // Overriden by specificAgentsAcqManagers
  public Agent getAgentIN2() {
    return null;
  }
  
  public Agent getAgentIN(int pos) {
    if (pos == 1)
      return getAgentIN1();
    else
      return getAgentIN2();
  }
  
}