package epe.agents.specificAgents;

	import epe.gui.SystemPrinter;
	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.specificManagers.AgentAdditionAcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.data.NumberMessage;
	import epe.communication.messages.data.ReadyToReceiveMessage;
	import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
	import epe.reorganization.specificManagers.AgentAdditionReorganizationManager;
	import epe.synchronization.SynchronizationManager;

// Fonctionnement : 2 entr?es mono, une sortie multiple renvoyant la somme des 2 entr?es.
public class AgentAddition extends Agent {
  
	  private boolean receivedIN1 = false;
	  private boolean receivedIN2 = false;
	  private int IN1;
	  private int IN2;

  public AgentAddition(String suffixe) {
    super("Add" + suffixe);
    acquaintancesManager = new AgentAdditionAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentAdditionReorganizationManager(this);
    SystemPrinter.printSystem(getAgentName() +" created");
  }

  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.NORMAL_AGENT,
                                         GUI.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedIN1 = false;
    receivedIN2 = false;
  }
  
  protected void acting() {
    boolean sendReadyToReceive = false;
    
    setActed(false);
    // Firing ready messages to IN1 and IN2
    if (sendReadyToReceiveTimerRanOut && !receivedIN1) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",1), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    if (sendReadyToReceiveTimerRanOut && !receivedIN2) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",2), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    
    if (sendReadyToReceive)
        readyToReceiveFired();
    
    // Firing IN1+IN2 to Waiters
    if (receivedIN1 && receivedIN2
        && (synchronizationManager.waitersAreReady())) {
      gui.println("Agent Addition : Waiters are ready, Firing NumberMessage.");
      synchronizationManager.sendToOutDone();
      NumberMessage message = new NumberMessage(IN1 + IN2, this);
      AgentInterface agentInterface = new AgentInterface("OUT", 1);
      fireMessage(message, agentInterface, this);
      reorganizationManager.setLastOutputValue(IN1 + IN2);
      //reorganizationManager.updatePotentialOuput();
      receivedIN1 = false;
      receivedIN2 = false;
      setActed(true);
      agentHasComputedItsFunction();
      cancelLongTimeNotActed();
    }
    
    if ((!hasActed())&&(debug)) {
      if (!receivedIN1)
        gui.println(getAgentName() + " : DEBUG : waiting for IN1");
      if (!receivedIN2)
        gui.println(getAgentName() + " : DEBUG : waiting for IN2");
      if (!synchronizationManager.waitersAreReady()) {
        gui.println(getAgentName() + " : DEBUG : waiters are not ready.");
        synchronizationManager.tellWhoIsNotReady();
      }
    }
  }
  
   public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);
    
    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE)) 
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        Agent agentIN1 = ((AgentAdditionAcquaintancesManager)acquaintancesManager)
                            .getAgentIN1();
        Agent agentIN2 = ((AgentAdditionAcquaintancesManager)acquaintancesManager)
                            .getAgentIN2();
        if (sender.equals(agentIN1) | sender.equals(agentIN2)) {
          if (sender.equals(agentIN1) && !(receivedIN1)) { 
            receivedIN1 = true;
            IN1 = ((NumberMessage)message).getValue();
            reorganizationManager.setLastInputValue(1, IN1);
            //reorganizationManager.updatePotentialInput();
            // => should not always update GoalReached
            gui.println(getAgentName() + " : Getting IN1");
          }
          else if (sender.equals(agentIN2) && !(receivedIN2)) {
            receivedIN2 = true;
            IN2 = ((NumberMessage)message).getValue();
            reorganizationManager.setLastInputValue(2, IN2);
            //reorganizationManager.updatePotentialInput();
            // => should not always update GoalReached
            gui.println(getAgentName() + " : Getting IN2");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                        + sender.getAgentName() + " ]."
                        + "already received his IN value.", GUI.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : Unknown sender of the NumberMessage : [ " 
                      + sender.getAgentName() + " ].", GUI.red);
        }
      } else// not in cooperative state and receiving DataMessage
          super.receivedDataMessageWhileNotInCooperativeState(message);
    else 
      super.receiveMessage(message);    
    
    setActed(true);
    startAgent();
  }
  
  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  	receivedIN2 = false;
  	receivedIN1 = false;
  	
//    AgentAdditionAcquaintancesManager acqMng = 
//      (AgentAdditionAcquaintancesManager)acquaintancesManager;
//    
//    if (! acqMng.possessingIN2()) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    }
//    if (!acqMng.possessingIN1()) {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
  }
  
  public void retryCalculus() {
  	// Retry on both IN 
  	receivedIN2 = false;
  	receivedIN1 = false;
  	
    // Retry only on one IN
//    if (blockingSituationCommutator) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    } else {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
//    switchBockingSituationCommutator();
  }
  
}