package epe.agents.specificAgents;

import epe.gui.SystemPrinter;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.specificManagers.AgentEqualsAcquaintancesManager;
import epe.agents.agentInterface.AgentInterface;
import epe.communication.Message;
import epe.communication.messages.data.BooleanMessage;
import epe.communication.messages.data.NumberMessage;
import epe.communication.messages.data.ReadyToReceiveMessage;
import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
import epe.reorganization.specificManagers.AgentEqualsReorganizationManager;
import epe.synchronization.SynchronizationManager;

 /*
 * Fonctionnement : 2 entr?es mono, une sortie multiple renvoyant le boolean (IN1 == IN2).
 */
public class AgentEquals extends Agent {
  
  private boolean receivedIN1 = false;
  private boolean receivedIN2 = false;
  private int IN1;
  private int IN2;
  
  /** Creates new AgentEquals */
  public AgentEquals(String suffixe) {
    super("Equals" + suffixe);
    acquaintancesManager = new AgentEqualsAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentEqualsReorganizationManager(this);
    SystemPrinter.printSystem(getAgentName() +" created");
  }

  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.NORMAL_AGENT,
                                         java.awt.Color.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedIN1 = false;
    receivedIN2 = false;
  }
  
  protected void acting() {
    boolean sendReadyToReceive = false;
    
    setActed(false);
    // Firing ready messages to IN1 and IN2
    if (sendReadyToReceiveTimerRanOut && !receivedIN1) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",1), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    if (sendReadyToReceiveTimerRanOut && !receivedIN2) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",2), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    
    if (sendReadyToReceive)
        readyToReceiveFired();
    
    // Firing test (IN1 == IN2) to Waiters
    if (receivedIN1 && receivedIN2
        && (synchronizationManager.waitersAreReady())) {
      gui.println("Agent Equals : Waiters are ready, Firing BooleanMessage.");
      synchronizationManager.sendToOutDone();
      BooleanMessage message = new BooleanMessage((IN1 == IN2), this);
      AgentInterface agentInterface = new AgentInterface("OUT", 1);
      fireMessage(message, agentInterface, this);
      receivedIN1 = false;
      receivedIN2 = false;
      setActed(true);
      agentHasComputedItsFunction();
      cancelLongTimeNotActed();
    }
    
    if ((!hasActed())&& (debug)) {
      if (!receivedIN1)
        gui.println(getAgentName() + " : DEBUG : WAITING for IN1");
      if (!receivedIN2)
        gui.println(getAgentName() + " : DEBUG : WAITING for IN2");
      if (!synchronizationManager.waitersAreReady()) {
        gui.println("Agent Addition : DEBUG : waiters are not ready.");
        synchronizationManager.tellWhoIsNotReady();
      }
    }
  }
  
  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);

    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE) || typeId.equals(Message.BOOLEAN_MESSAGE)) 
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        Agent agentIN1 = ((AgentEqualsAcquaintancesManager)acquaintancesManager).getAgentIN1();
        Agent agentIN2 = ((AgentEqualsAcquaintancesManager)acquaintancesManager).getAgentIN2();
        if (sender.equals(agentIN1) | sender.equals(agentIN2)) {
          if (sender.equals(agentIN1) && !(receivedIN1)) { 
            receivedIN1 = true;
            // Generic AgentEquals instanciated to INTEGER
            if (typeId.equals(Message.NUMBER_MESSAGE))
              IN1 = ((NumberMessage)message).getValue();
            else
              // Generic AgentEquals instanciated to BOOLEAN
              if (((BooleanMessage)message).getTruthValue())
                IN1 = 1;
              else
                IN1 = 0;
            gui.println(getAgentName() + " : Getting IN1");
          }
          else if (sender.equals(agentIN2) && !(receivedIN2)) {
            receivedIN2 = true;
            // Generic AgentEquals instanciated to INTEGER
            if (typeId.equals(Message.NUMBER_MESSAGE))
              IN2 = ((NumberMessage)message).getValue();
            else
              // Generic AgentEquals instanciated to BOOLEAN
              if (((BooleanMessage)message).getTruthValue())
                IN2 = 1;
              else
                IN2 = 0;
            gui.println(getAgentName() + " : Getting IN2");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                        + sender.getAgentName() + " ]."
                        + " Already received his IN value.", java.awt.Color.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : Unknown sender of the NumberMessage : [ " 
                      + sender.getAgentName() + " ].", java.awt.Color.red);
        }
      } else // not in cooperative state and receiving DataMessage
          super.receivedDataMessageWhileNotInCooperativeState(message);
    else
      super.receiveMessage(message);
    setActed(true);
    startAgent();
  }
  
  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  	receivedIN2 = false;
  	receivedIN1 = false;
    
//    AgentEqualsAcquaintancesManager acqMng = (AgentEqualsAcquaintancesManager)acquaintancesManager;
//    
//    if (! acqMng.possessingIN2()) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    }
//    if (!acqMng.possessingIN1()) {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
    /*
    if (acqMng.possessingIN1() 
       && ! acqMng.possessingIN2()) {
      receivedIN2 = false;
      sendReadyIN2 = false;
    }
    else if (!acqMng.possessingIN1() 
             && acqMng.possessingIN2()) {
      receivedIN1 = false;
      sendReadyIN1 = false;
    }
    /*
    else {
      SystemPrinter.printSystem("ERROR : " + getAgentName());
      gui.println(getAgentName() 
                  + " : ERROR : received cancelledCooperation notification" 
                  + " but still possessing IN1 and IN2", java.awt.Color.red);
    }*/
  }
  
  public void retryCalculus() {
   	// Retry on both IN 
  	receivedIN2 = false;
  	receivedIN1 = false;
  	
    // Retry only on one IN
//    if (blockingSituationCommutator) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    } else {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
//    switchBockingSituationCommutator();
  }  
}