package epe.agents.specificAgents;

import epe.gui.SystemPrinter;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.specificManagers.AgentFirstTimeAcquaintancesManager;
import epe.agents.agentInterface.AgentInterface;
import epe.communication.Message;
import epe.communication.messages.data.NumberMessage;
import epe.communication.messages.data.ReadyToReceiveMessage;
import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
import epe.reorganization.specificManagers
					.AgentFirstTimeReorganizationManager;
import epe.synchronization.SynchronizationManager;

/**
 * Fonctionnement : 2 entr?es mono, une sortie multiple renvoyant l'entr?e IN1
 *                  la premi?re fois et IN2 toutes les autres fois.
 */
public class AgentFirstTime extends Agent {

  private boolean receivedIN1 = false;
  private boolean receivedIN2 = false;
  private boolean firstTime = true;
  private Message IN1Message;
  private Message IN2Message;

  /** Creates new AgentFirstTime */
  public AgentFirstTime(String suffixe) {
    super("First" + suffixe);
    acquaintancesManager = new AgentFirstTimeAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentFirstTimeReorganizationManager(this);
    SystemPrinter.printSystem(getAgentName() +" created");
  }

  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.NORMAL_AGENT,
                                         java.awt.Color.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    firstTime = true;
    receivedIN1 = false;
    receivedIN2 = false;
  }

  protected void acting() {
    boolean sendReadyToReceive = false;
    
    setActed(false);
    // Firing ready messages to IN1 and IN2
    if (sendReadyToReceiveTimerRanOut && !receivedIN1) {
      fireMessage(new ReadyToReceiveMessage(this),
      new AgentInterface("IN",1), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    if (sendReadyToReceiveTimerRanOut && !receivedIN2) {
      fireMessage(new ReadyToReceiveMessage(this),
      new AgentInterface("IN",2), this);
      setActed(true);
      sendReadyToReceive = true;
    }
    
    if (sendReadyToReceive)
        readyToReceiveFired();
    
    // Firing IN1 to Waiters if it's the first time they ask.
    if (receivedIN1 && synchronizationManager.waitersAreReady() && firstTime) {
      gui.println("Agent FirstTime : Waiters are ready, Firing [" 
                + IN1Message.getId() + "] [FirstTime].");
      synchronizationManager.sendToOutDone();
      IN1Message.setSender(this);
      AgentInterface agentInterface = new AgentInterface("OUT", 1);
      fireMessage(IN1Message, agentInterface, this);
      if (IN1Message.getId().equals(Message.NUMBER_MESSAGE)) {
        reorganizationManager
          .setLastOutputValue(((NumberMessage)IN1Message).getValue());
      	//reorganizationManager.updatePotentialOuput();
      }
      receivedIN1 = false;
      firstTime = false;
      setActed(true);
      agentHasComputedItsFunction();
      cancelLongTimeNotActed();
    }
    // Firing IN2 to Waiters if it's NOT the first time they ask.
    if (receivedIN2 && synchronizationManager.waitersAreReady() && (!firstTime)) {
      gui.println("Agent FirstTime : Waiters are ready, Firing [" 
                + IN2Message.getId() + "] [OtherTimes].");
      synchronizationManager.sendToOutDone();
      IN2Message.setSender(this);
      AgentInterface agentInterface = new AgentInterface("OUT", 1);
      fireMessage(IN2Message, agentInterface, this);
      if (IN2Message.getId().equals(Message.NUMBER_MESSAGE))
        reorganizationManager.setLastOutputValue(
        ((NumberMessage)IN2Message).getValue());
      receivedIN2 = false;
      setActed(true);
      agentHasComputedItsFunction();
      cancelLongTimeNotActed();
    }

    if ((!hasActed())&&(debug)) {
      if (!receivedIN1)
        gui.println(getAgentName() + " : DEBUG : waiting for IN1");
      if (!receivedIN2)
        gui.println(getAgentName() + " : DEBUG : waiting for IN2");
      if (!synchronizationManager.waitersAreReady()) {
        gui.println(getAgentName() + " : DEBUG : waiters are not ready.");
        synchronizationManager.tellWhoIsNotReady();
      }
    }
  }

  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);

    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE) || typeId.equals(Message.BOOLEAN_MESSAGE))
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        Agent agentIN1 = ((AgentFirstTimeAcquaintancesManager)acquaintancesManager).getAgentIN1();
        Agent agentIN2 = ((AgentFirstTimeAcquaintancesManager)acquaintancesManager).getAgentIN2();
        if (sender.equals(agentIN1) | sender.equals(agentIN2)) {
          if (sender.equals(agentIN1) && !(receivedIN1)) { 
            receivedIN1 = true;
            IN1Message = message;
            if (IN1Message.getId().equals(Message.NUMBER_MESSAGE)) {
              reorganizationManager
                .setLastInputValue(1 ,((NumberMessage)IN1Message).getValue());
              reorganizationManager.updatePotentialInput();
            }
            gui.println(getAgentName() + " : Getting IN1");
          }
          else if (sender.equals(agentIN2) && !(receivedIN2)) {
            receivedIN2 = true;
            IN2Message = message;
            if (IN2Message.getId().equals(Message.NUMBER_MESSAGE))
              reorganizationManager
                .setLastInputValue(2 ,((NumberMessage)IN2Message).getValue());
            gui.println(getAgentName() + " : Getting IN2");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                        + sender.getAgentName() + " ]."
                        + "already received his IN value.", java.awt.Color.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : Unknown sender of the DataMessage["
                      + typeId+ "] : [ " 
                      + sender.getAgentName() + " ].", java.awt.Color.red);
          gui.println("Acquaintances : ");
          acquaintancesManager.showAllAcquaintances();
        }
      } else // not in cooperative state and receiving DataMessage
        super.receivedDataMessageWhileNotInCooperativeState(message);
    else
      super.receiveMessage(message);
    
    setActed(true);
    startAgent();
  }

  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  	receivedIN2 = false;
  	receivedIN1 = false;
    
//    AgentFirstTimeAcquaintancesManager acqMng = (AgentFirstTimeAcquaintancesManager)acquaintancesManager;
//    
//    
//    if (!acqMng.possessingIN1()) {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//      firstTime = true;
//    }
//    if (! acqMng.possessingIN2()) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//      //firstTime = true; //Maybe he doesn't need to resend IN1 if IN1acq is still OK
//    }
    
    /*
    if (acqMng.possessingIN1() 
       && ! acqMng.possessingIN2()) {
      receivedIN2 = false;
      sendReadyIN2 = false;
    }
    else if (!acqMng.possessingIN1() 
             && acqMng.possessingIN2()) {
      receivedIN1 = false;
      sendReadyIN1 = false;
    }
    /*
    else {
      SystemPrinter.printSystem("ERROR : " + getAgentName());
      gui.println(getAgentName() 
                  + " : ERROR : received cancelledCooperation notification" 
                  + " but still possessing IN1 and IN2", java.awt.Color.red);
    }*/
  }
  
  public void retryCalculus() {
  	// Retry on both IN 
  	receivedIN2 = false;
  	receivedIN1 = false;
  	
    // Retry only on one IN
//    if (blockingSituationCommutator) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    } else {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
//    switchBockingSituationCommutator();
  }  
}