package epe.agents.specificAgents.environment;

import java.awt.event.ActionListener;

import javax.swing.JTextField;
import javax.swing.Timer;

import epe.gui.CommandPanel;
import epe.gui.SystemPrinter;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.specificManagers.environment.AgentEnvironmentAcquaintancesManager;
import epe.agents.agentInterface.AgentInterface;
import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedListener;
import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedListeners;
import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedNotifier;
import epe.communication.Message;
import epe.communication.messages.data.NumberMessage;
import epe.communication.messages.data.ReadyToReceiveMessage;
import epe.communication.messages.ncs.NCSInadequateDataMessage;
import epe.communication.messages.ncs.NCSMissingDataMessage;
import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
import epe.reorganization.specificManagers.environment.AgentEnvironmentReorganizationManager;
import epe.synchronization.SynchronizationManager;
import epe.synchronization.waiters.Waiter;
import epe.synchronization.waiters.Waiters;


/** 
 * How it works : 1 IN, 1 OUT : the agentEnvironment gets a data from the inpoutTextField
 *                and gives it to the agentIn. When the system has finished computing,  
 *                it send the result, via the agentOut, to the agentEnvironment who judges 
 *                the adequacy and sends back a NCSinadequeteData if needed.
 */
public class AgentEnvironment extends Agent 
                              implements ActionListener,
                                         EnvironmentStateChangedNotifier {
  
  protected boolean readyToSend = false;
  protected int numberToSend;
  protected JTextField inputTextField;
  protected CommandPanel inputOutput;
  private String newline = "\n";
  
  private boolean receivedIN = false;
  private boolean sendReadyIN = false;
  private int IN;
  private boolean receivedGoodResult = false;
      
  // The waitingForDataTimer : if it runs out, the system is not adequate because it isn't fast enough.
  protected final static int ONE_SECOND = 1000;
  protected Timer waitingForDataTimer;
  protected int waitingForDataTimerDelay = 20*ONE_SECOND;//0
  protected boolean waitingForDataTimerActivated = false;
  protected boolean waitingForDataTimerRanOut = false;
  // Tells if the Environment should regularly prompt the system
  protected boolean waitingForDataTimerEnabled = false;
  // Tells if the Environment should send NCSinadequateData to the system
  protected boolean feedbackEnabled = false; //true
  // Tells if the environment should stop prompting the system for values 
  // if it receives an adequate data
  protected boolean stopIfAdequateData = false;
  
  // Concerning the EnvironmmentSateChangedNotifier
  protected EnvironmentStateChangedListeners environmentStateChangedListeners;
  
  /** Creates new AgentEnvironment */
  public AgentEnvironment(String suffixe, CommandPanel commandPanel) {
    super("Env" + suffixe);
    acquaintancesManager = new AgentEnvironmentAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentEnvironmentReorganizationManager(this);
    inputOutput = commandPanel;
    waitingForDataTimer = new Timer(waitingForDataTimerDelay, this);
    waitingForDataTimer.setRepeats(false);
    setRepeatReadyToReceive(true);
    environmentStateChangedListeners = new EnvironmentStateChangedListeners();
    SystemPrinter.printSystem(getAgentName() +" created");
  }

  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.ENVIRONMENT_ENVIRONMENT,
                                         java.awt.Color.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    readyToSend = false;
    receivedGoodResult = false;
    receivedIN = false;
    sendReadyIN = false;
    waitingForDataTimerRanOut = false;
    waitingForDataTimerActivated = false;
  }
  
  protected void acting() {
    if (!receivedGoodResult || !stopIfAdequateData) {
    setActed(false);
    
    if (!readyToSend) {
      // Get the value in the TextField
      String text = inputOutput.getText();
      if (text.length() == 0) {
        SystemPrinter.printSystem("ERROR : " + getAgentName());
        errorHappened();
        gui.println("Agent Environment : ERROR : No IN value", java.awt.Color.red);
      }
      else {
        gui.println(getAgentName() + " trying to get the value to calculate.");
        try {
          numberToSend = new Integer(text).intValue();
          readyToSend = true;
          gui.println(getAgentName() + " : Getting : " + numberToSend);
        } catch (NumberFormatException e) {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : Incorrect IN value", java.awt.Color.red);
        }
        setActed(true);
        cancelLongTimeNotActed();
      }
    }
    
    // Sending value to compute to OUT (InAgent)
    if (readyToSend && synchronizationManager.waitersAreReady()) {
      gui.println("Agent Environment : the waiters are ready, Firing NumberMessage.");
      NumberMessage outMessage = new NumberMessage(numberToSend, this);
      
      Waiters readyWaiters = synchronizationManager.getReadyWaiters();
      synchronizationManager.sendToReadyWaitersDone(readyWaiters);
      for (int i=0; i < readyWaiters.size(); i++) {
      Agent destinataire = ((Waiter)readyWaiters.elementAt(i)).getAgent();
        fireMessage(outMessage, destinataire, "OUT" , this);
      }
      setActed(true);
      cancelLongTimeNotActed();
    }
    
    // Reveived IN value => judging it's adequacy
    if (receivedIN) {
      gui.println(getAgentName() + " : received value from the system : ready to judge");
      reorganizationManager.setLastInputValue(1, IN);
      // notify stateChangedListeners
      notifyEnvironmentStateChangedListeners(IN);
      waitingForDataTimerActivated = false;
      if (IN == function(numberToSend)) {
        SystemPrinter.printSystem("OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS OUAIS ");
        gui.println(getAgentName() + " : OUTPUT = " + IN);
        println("OUTPUT = " + IN);
        receivedGoodResult = true;
        gui.adequacyNotification();
      }
      else {
        SystemPrinter.printSystem("MMMMMMMMMMMMMMMMMM...........");
        gui.println(getAgentName() + " : WRONG DATA = " + IN);
        println("WRONG DATA = " + IN);
        
        if (feedbackEnabled) {
          SystemPrinter.printSystem("Environment : sending feedback");
          NCSInadequateDataMessage message = new NCSInadequateDataMessage(this);
          AgentInterface agentInterface = new AgentInterface("IN", 1);
          fireMessage(message, agentInterface, this);
        }
      }
      receivedIN = false;
      sendReadyIN = false;
      setActed(true);
      cancelLongTimeNotActed();
    }
    
    // Firing ready messages to IN (i.e : AgentOUT)
    if ((!sendReadyIN || sendReadyToReceiveTimerRanOut) && !receivedIN && !receivedGoodResult) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",1), this);
      sendReadyIN = true;
      setActed(true);
      readyToReceiveFired();
      if (!waitingForDataTimerActivated) {
        waitingForDataTimerActivated = true;
        fireWaitingForDataTimer();
      }
    }
    
    // Firing missingData if the waitingForDataTimer ran out
    if (waitingForDataTimerRanOut && !receivedIN) {
      gui.println(getAgentName() + " : waitingForDataTimerRanOut", java.awt.Color.orange);
      if (feedbackEnabled) {
        //SystemPrinter.printSystem("Environment : sending NCSmissingData");
        NCSMissingDataMessage message = new NCSMissingDataMessage(this);
        AgentInterface agentInterface = new AgentInterface("IN", 1);
        fireMessage(message, agentInterface, this);
      }
      fireWaitingForDataTimer();
      setActed(true);
      cancelLongTimeNotActed();
      gui.println(getAgentName() + " : fired NCSmissingData", java.awt.Color.orange);
    }
    
    if ((!hasActed())&&(debug)) {
      if (!receivedIN)
        gui.println(getAgentName() + " : DEBUG : waiting for IN");
      if (!synchronizationManager.waitersAreReady()) {
        gui.println(getAgentName() + " : DEBUG : waiters are not ready.");
        synchronizationManager.tellWhoIsNotReady();
      }
    }
    }
    else // reveivedGoodResult == false
      stopAgent();
  }
  
  protected int function(int x) {
    return factorial(x);
  }
  
  protected int inValue() {
    return numberToSend;
  }
  
  protected int constant() {
    return 10;
  }

  protected int factorial(int x) {
    if (x==0)
      return 1;
    else if (x==1)
      return 1;
    else
      return x*factorial(x-1);
  }

  public synchronized void print(String text) {
    inputOutput.print(text);
  }

  public synchronized void println( String text) {
    this.print( text + newline );
  }
   
  // sendReadyToReceiveTime and waitingForDataTimer call this method when they get to zero
  public void actionPerformed(java.awt.event.ActionEvent evt) {
    setReceivedEvent(true);
    if ((evt.getSource()).equals(sendReadyToReceiveTimer)) {
      sendReadyToReceiveTimerRanOut = true;
      startAgent();
    }
    else if ((evt.getSource()).equals(waitingForDataTimer)) {
      //SystemPrinter.printSystem(getAgentName() + " : waitingForDataTimer ran out.");
      waitingForDataTimerRanOut = true;
      startAgent();
    }
  }
  
  protected void fireWaitingForDataTimer() {
    if (waitingForDataTimerEnabled) {
      waitingForDataTimerRanOut = false;
      waitingForDataTimer.restart();
    }
  }
  
  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);
    
    if (typeId.equals(Message.NUMBER_MESSAGE)) {
      super.receivedDataMessage(message);
      Agent sender = message.getSender();
      Agent agentIN = ((AgentEnvironmentAcquaintancesManager)acquaintancesManager).getAgentIN();
      if (sender.equals(agentIN)) {
        if (sender.equals(agentIN) && !(receivedIN)) { 
          receivedIN = true;
          IN = ((NumberMessage)message).getValue();
          gui.println(getAgentName() + " : Getting IN");
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                      + sender.getAgentName() + " ]."
                      + "already received his IN value.", java.awt.Color.red);
        }
      }
      else {
        SystemPrinter.printSystem("ERROR : " + getAgentName());
        errorHappened();
        gui.println(getAgentName() + " : ERROR : Unknown sender of the NumberMessage :[ " 
                    + sender.getAgentName() + " ].", java.awt.Color.red);
      }
    }
    else if (typeId.equals(Message.READY_TO_RECEIVE_MESSAGE))
      synchronizationManager.processReadyMessage(message);
    //else 
      //super.receiveMessage(message);    
    setActed(true);
    startAgent();
  }
  
  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  }

  public void retryCalculus() {
  }
  
  
  // Relative to the EnvironmentStateChangedNotifier interface
  public void addEnvironmentStateChangedListener(EnvironmentStateChangedListener listener) {
    environmentStateChangedListeners.addListener(listener);
  }
  
  public void removeEnvironmentStateChangedListener(EnvironmentStateChangedListener listener) {
    environmentStateChangedListeners.removeListener(listener);
  }
  
  public void notifyEnvironmentStateChangedListeners(int data) {
    environmentStateChangedListeners.notifyListeners(data);
  }
  
}