package epe.agents.specificAgents.environment;

	import java.awt.event.ActionListener;
	
	import javax.swing.JTextArea;
	import javax.swing.Timer;
	
	import epe.gui.CommandPanel;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.specificManagers.environment
						.AgentEnvironmentReceiverAcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
	import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedListener;
	import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedListeners;
	import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedNotifier;
	import epe.communication.Message;
	import epe.communication.messages.data.NumberMessage;
	import epe.communication.messages.data.ReadyToReceiveMessage;
	import epe.communication.messages.ncs.NCSInformedInadequateDataMessage;
	import epe.communication.messages.ncs.NCSInadequateDataMessage;
	import epe.communication.messages.ncs.NCSMissingDataMessage;
	import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
	import epe.reorganization.specificManagers.environment
						.AgentEnvironmentReceiverReorganizationManager;
	import epe.synchronization.SynchronizationManager;

/** 
 * How it works : 
 * 	1 IN, 1 OUT : When the system has finished computing,it send the result, via the agentOut, 
 * 	to the agentEnvironmentReceiver who judges the adequacy 
 * 	and sends back a NCSinadequeteData if needed.
 */
public class AgentEnvironmentReceiver extends Agent 
       implements ActionListener, EnvironmentStateChangedNotifier {
  
	  protected JTextArea outputTextArea;
	  protected CommandPanel inputOutput;
	  protected String newline = "\n";
	  
	  protected boolean receivedIN = false;
	  protected boolean sendReadyIN = false;
	  protected int IN;
	  protected boolean receivedGoodResult = false;
	  protected boolean receivedData = false;
	  protected boolean receivedConfirmation = false;
	  protected int firstReceivedData = 0;
	  protected int numberOfSameDataReceived = 1;
	  protected int minNumberOfDataToSupposeCompleteOrganization = 20;
	      
	  // The waitingForDataTimer : if it runs out, the system is not adequate 
	  // because it isn't fast enough.
	  protected final static int ONE_SECOND = 1000;
	  protected Timer waitingForDataTimer;
	  protected int waitingForDataTimerDelay = 20*ONE_SECOND;//0
	  protected boolean waitingForDataTimerActivated = false;
	  protected boolean waitingForDataTimerRanOut = false;
	  // Tells if the Environment should regularly prompt the system
	  protected boolean waitingForDataTimerEnabled = true;
	  // Tells if the Environment should send NCSinadequateData to the system
	  protected boolean feedbackEnabled = false; //true
	  protected boolean stepByStepFeedbackEnabled = true; //true
	  // Tells if the user has to manually send the feedback
	  protected boolean manualStepByStep = true;
	  // Counts the number of times the environment sends a feedback to the system
	  protected int feedbackCounter = 0;
	  // Tells if the environment should stop prompting the system for values 
	  // if it receives an adequate data
	  protected boolean stopIfAdequateData = true;
	  // if it receives any data
	  protected boolean stopIfData = false;
	  // Simple feedback or Higher/lower Feedback
	  protected boolean simpleFeedback = false;
	  
	  // Concerning the EnvironmmentSateChangedNotifier
	  protected EnvironmentStateChangedListeners environmentStateChangedListeners;
  
  public AgentEnvironmentReceiver(String suffixe, CommandPanel commandPanel) {
    super("Env" + suffixe);
    acquaintancesManager = new AgentEnvironmentReceiverAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentEnvironmentReceiverReorganizationManager(this);
    inputOutput = commandPanel;
    waitingForDataTimer = new Timer(waitingForDataTimerDelay, this);
    waitingForDataTimer.setRepeats(false);
    setRepeatReadyToReceive(true);
    environmentStateChangedListeners = new EnvironmentStateChangedListeners();
    SystemPrinter.printSystem(getAgentName() +" created");
  }

  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.ENVIRONMENT_ENVIRONMENT,
                                         java.awt.Color.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedGoodResult = false;
    receivedData = false;
    receivedIN = false;
    sendReadyIN = false;
    waitingForDataTimerRanOut = false;
    waitingForDataTimerActivated = false;
    feedbackCounter = 0;
  }
  
  protected void acting() {
    if ((!receivedGoodResult || !stopIfAdequateData) 
     && (!receivedData || !stopIfData)) {
      setActed(false);

      // Reveived IN value => judging it's adequacy
      if (receivedIN) {
        // Wait to receive the data twice to be sure it is the one calculated by the system
        // (and not some on the fly catched value)
        reorganizationManager.setLastInputValue(1, IN);
        if (IN == firstReceivedData) {
          receivedConfirmation = true;
          firstReceivedData = IN;
          numberOfSameDataReceived++;
        }
        else {
          println("FIRST DATA = " + IN);
          firstReceivedData = IN;
          numberOfSameDataReceived = 1;
        }
        
        if (receivedConfirmation) {
          receivedConfirmation = false;
          
          gui.println(getAgentName() + " : received value from the system : ready to judge");
          // notify stateChangedListeners
          notifyEnvironmentStateChangedListeners(IN);
          waitingForDataTimerActivated = false;
          if (IN == function()) {
            SystemPrinter.printSystem("SUCCESS : ENVIRONMENT RECEIVED ADEQUATE DATA " +
            													"(System Output = " + IN + ")");
            gui.println(getAgentName() + " : OUTPUT = " + IN);
            println("OUTPUT = " + IN);
            receivedGoodResult = true;
            gui.adequacyNotification();
          }
          else {
            SystemPrinter.printSystem("Environment received inadequate data " +
            													"(System Output = " + IN + ")");
            gui.println(getAgentName() + " : WRONG DATA = " + IN);
            println("WRONG DATA = " + IN);
            receivedData = true;
            
            if (stepByStepFeedbackEnabled && !manualStepByStep)
              if (numberOfSameDataReceived > minNumberOfDataToSupposeCompleteOrganization) {
                feedback();
                numberOfSameDataReceived = 0;
              }
            
            if (feedbackEnabled) {
              if (simpleFeedback)
                // Simple Feedback version
                sendSimpleFeedback();
              else
                // Higher/Lowwer Feedback version
                sendHigherLowerFeedback();
            }
          }
        }
        receivedIN = false;
        sendReadyIN = false;
        setActed(true);
        cancelLongTimeNotActed();
      }

      // Firing ready messages to IN (i.e : AgentOUT)
      if ((!sendReadyIN && sendReadyToReceiveTimerRanOut) 
      		&& !receivedIN && !receivedGoodResult) {
        fireMessage(new ReadyToReceiveMessage(this), 
                    new AgentInterface("IN",1), this);
        sendReadyIN = true;
        setActed(true);
        readyToReceiveFired();
        if (!waitingForDataTimerActivated) {
          waitingForDataTimerActivated = true;
          fireWaitingForDataTimer();
        }
      }

      // Firing missingData if the waitingForDataTimer ran out
      if (waitingForDataTimerRanOut && !receivedIN) {
        gui.println(getAgentName() + " : waitingForDataTimerRanOut", java.awt.Color.orange);
        if (feedbackEnabled) {
          //SystemPrinter.printSystem("Environment : sending NCSmissingData");
          NCSMissingDataMessage message = new NCSMissingDataMessage(this);
          AgentInterface agentInterface = new AgentInterface("IN", 1);
          fireMessage(message, agentInterface, this);
        }
        fireWaitingForDataTimer();
        setActed(true);
        cancelLongTimeNotActed();
        gui.println(getAgentName() + " : fired NCSmissingData", java.awt.Color.orange);
      }

      if ((!hasActed())&&(debug)) {
        if (!receivedIN)
          gui.println(getAgentName() + " : DEBUG : waiting for IN");
//        if (!synchronizationManager.waitersAreReady()) {
//          gui.println(getAgentName() + " : DEBUG : waiters are not ready.");
//          synchronizationManager.tellWhoIsNotReady();
//        }
      }
    }
    else // receivedGoodResult == true
      stopAgent();
  }
  
  public void feedback() {
    SystemPrinter.printSystem("Environment : Next Step called");
    
    if (stepByStepFeedbackEnabled) {
      if (simpleFeedback) 
        // Simple Feedback version
        sendSimpleFeedback();
      else
        // Higher/Lower Feedback version
        sendHigherLowerFeedback();
    }
  }
  
  // Simple Feedback version
  protected void sendSimpleFeedback() {
    SystemPrinter.printSystem("Environment : sending Simple Feedback");
    println("----Feedback");
    NCSInadequateDataMessage message = new NCSInadequateDataMessage(this);
    AgentInterface agentInterface = new AgentInterface("IN", 1);
    fireMessage(message, agentInterface, this);
    feedbackCounter++;
    SystemPrinter.printSystem("FeedbackCounter = " + feedbackCounter);
  }

  // Higher/Lowwer Feedback version
  protected void sendHigherLowerFeedback() {
    SystemPrinter.printSystem("Environment : sending Higher/Lower Feedback");
    println("----Feedback");
    gui.println("Environment : sending Higher/Lower Feedback", 
                java.awt.Color.orange);
    if (IN < function()) {
      NCSInformedInadequateDataMessage message 
      	= new NCSInformedInadequateDataMessage("HIGHER", this);
      AgentInterface agentInterface = new AgentInterface("IN", 1);
      fireMessage(message, agentInterface, this);
    } else {
      NCSInformedInadequateDataMessage message
      	= new NCSInformedInadequateDataMessage("LOWER", this);
      AgentInterface agentInterface = new AgentInterface("IN", 1);
      fireMessage(message, agentInterface, this);
    }
    feedbackCounter++;
    SystemPrinter.printSystem("FeedbackCounter = " + feedbackCounter);
  }
  
  protected int function() {
    return constant();
  }
  
  protected int constant() {
    return 100000;
  }

  public synchronized void print(String text) {
    inputOutput.print(text);
  }

  public synchronized void println( String text) {
    this.print( text + newline );
  }
   
  // sendReadyToReceiveTime and waitingForDataTimer call this method when they get to zero
  public void actionPerformed(java.awt.event.ActionEvent evt) {
    setReceivedEvent(true);
    if ((evt.getSource()).equals(sendReadyToReceiveTimer)) {
      sendReadyToReceiveTimerRanOut = true;
      startAgent();
    }
    else if ((evt.getSource()).equals(waitingForDataTimer)) {
      //SystemPrinter.printSystem(getAgentName() + " : waitingForDataTimer ran out.");
      waitingForDataTimerRanOut = true;
      startAgent();
    }
    else super.actionPerformed(evt);
  }
  
  protected void fireWaitingForDataTimer() {
    if (waitingForDataTimerEnabled) {
      waitingForDataTimerRanOut = false;
      waitingForDataTimer.restart();
    }
  }
  
  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);
    
    if (typeId.equals(Message.NUMBER_MESSAGE)) {
      super.receivedDataMessage(message);
      Agent sender = message.getSender();
      Agent agentIN = ((AgentEnvironmentReceiverAcquaintancesManager)acquaintancesManager)
                          .getAgentIN();
      if (sender.equals(agentIN)) {
        if (sender.equals(agentIN) && !(receivedIN)) { 
          receivedIN = true;
          IN = ((NumberMessage)message).getValue();
          reorganizationManager.setLastInputValue(1, IN);
          gui.println(getAgentName() + " : Getting IN");
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                      + sender.getAgentName() + " ]."
                      + "already received his IN value.", java.awt.Color.red);
        }
      }
      else {
        SystemPrinter.printSystem("ERROR : " + getAgentName());
        errorHappened();
        gui.println(getAgentName() + " : ERROR : Unknown sender of the NumberMessage :[ " 
                    + sender.getAgentName() + " ].", java.awt.Color.red);
      }
    }
    else if (typeId.equals(Message.READY_TO_RECEIVE_MESSAGE))
      synchronizationManager.processReadyMessage(message);
    //else 
      //super.receiveMessage(message);    
    setActed(true);
    startAgent();
  }
  
  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  }

  public void retryCalculus() {
  }
  
  
  // Relative to the EnvironmentStateChangedNotifier interface
  public void addEnvironmentStateChangedListener(EnvironmentStateChangedListener listener) {
    environmentStateChangedListeners.addListener(listener);
  }
  
  public void removeEnvironmentStateChangedListener(EnvironmentStateChangedListener listener) {
    environmentStateChangedListeners.removeListener(listener);
  }
  
  public void notifyEnvironmentStateChangedListeners(int data) {
    environmentStateChangedListeners.notifyListeners(data);
  }
  
}