package epe.agents.specificAgents.interfaceAgents;

import epe.gui.SystemPrinter;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.specificManagers.interfaceAgents.AgentInAcquaintancesManager;
import epe.agents.agentInterface.AgentInterface;
import epe.communication.Message;
import epe.communication.messages.data.NumberMessage;
import epe.communication.messages.data.ReadyToReceiveMessage;
import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
import epe.reorganization.specificManagers.interfaceAgents.AgentInReorganizationManager;
import epe.synchronization.SynchronizationManager;

/** 
 * How it works : 1 IN(mono), 1 OUT(multi) : the IN is linked to the agentEnvironment,
 *                the OUT is linked to some agents in the system.
 */
public class AgentIn extends Agent {

  private boolean receivedIN = false;
  private int valueToCompute;
    
  /** Creates new AgentIn */
  public AgentIn(String suffixe) {
    super("In" + suffixe);
    acquaintancesManager = new AgentInAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentInReorganizationManager(this);
    SystemPrinter.printSystem(getAgentName() +" created");
  }
  
  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.INPUT_AGENT,
                                         java.awt.Color.lightGray);
  }
  
  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedIN = false;
  }
  
  protected void acting() {
    setActed(false);
    
    // Firing ready messages to IN
    if (sendReadyToReceiveTimerRanOut && !receivedIN) {
      fireMessage(new ReadyToReceiveMessage(this), 
                  new AgentInterface("IN",1), this);
      setActed(true);
      readyToReceiveFired();
    }
    
    // Firing IN to Waiters
    if (receivedIN && (synchronizationManager.waitersAreReady())) {
      gui.println(getAgentName() + " : Waiters are ready, Firing NumberMessage.");
      synchronizationManager.sendToOutDone();
      NumberMessage message = new NumberMessage(valueToCompute, this);
      AgentInterface agentInterface = new AgentInterface("OUT", 1);
      fireMessage(message, agentInterface, this);
      receivedIN = false;
      setActed(true);
      agentHasComputedItsFunction();
      cancelLongTimeNotActed();
    }
    
    if ((!hasActed())&&(debug)) {
      if (!receivedIN)
        gui.println(getAgentName() + " : DEBUG : waiting for IN");
      if (!synchronizationManager.waitersAreReady()) {
        gui.println(getAgentName() + " : DEBUG : waiters are not ready.");
        synchronizationManager.tellWhoIsNotReady();
      }
    }
  }
  
   public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);
    
    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE)) 
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        Agent agentIN = ((AgentInAcquaintancesManager)acquaintancesManager).getAgentIN();
        if (sender.equals(agentIN)) {
          if (sender.equals(agentIN) && !(receivedIN)) { 
            valueToCompute = ((NumberMessage)message).getValue();
            receivedIN = true;
            gui.println(getAgentName() + " : Getting IN (value for the system to compute)");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println(getAgentName() + " : ERROR : not ready to receive from [ " 
                        + sender.getAgentName() + " ]."
                        + "already received his IN value.", java.awt.Color.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println(getAgentName() + " : ERROR : Unknown sender of the NumberMessage : [ " 
                      + sender.getAgentName() + " ].", java.awt.Color.red);
        }
      } else // not in cooperative state and receiving DataMessage
        super.receivedDataMessageWhileNotInCooperativeState(message);
    else 
      super.receiveMessage(message);    
 
    setActed(true);
    startAgent();
  }
  
  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  }
  
  public void retryCalculus() {
  }
  
}