package epe.agents.specificAgents.interfaceAgents;

import epe.gui.CommandPanel;
import epe.gui.SystemPrinter;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.specificManagers.interfaceAgents.AgentOutAcquaintancesManager;
import epe.agents.agentInterface.AgentInterface;
import epe.communication.Message;
import epe.communication.messages.data.BooleanMessage;
import epe.communication.messages.data.NumberMessage;
import epe.communication.messages.data.ReadyToReceiveMessage;
import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
import epe.reorganization.specificManagers.interfaceAgents.AgentOutReorganizationManager;
import epe.synchronization.SynchronizationManager;

/**
 * How it works : 2 IN (mono, int and boolean), 1 OUT(mono) : if (IN2==true), sends IN1 to
 *                the agentEnvironment
 */
public class AgentOut extends Agent {

  private boolean receivedIN1 = false;
  private boolean receivedIN2 = false;
  private boolean sendGoodResult = false;
  private int IN1;  // Input <Value> (Integer)
  private boolean IN2;  // Input <Control> (Boolean)
  private CommandPanel output;
  private String newline = "\n";
  
  /** Creates new AgentOut */
  public AgentOut(String suffixe, CommandPanel commandPanel) {
    super("Out" + suffixe);
    acquaintancesManager = new AgentOutAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentOutReorganizationManager(this);
    output = commandPanel;
    SystemPrinter.printSystem(getAgentName() +" created");
  }
  
  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.OUTPUT_AGENT,
                                         java.awt.Color.lightGray);
  }

  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedIN1 = false;
    receivedIN2 = false;
    sendGoodResult = false;
  }
  
  protected void acting() {
    boolean sendReadyToReceive = false;
    
    if (!sendGoodResult) {
      setActed(false);
      // Firing ready messages to IN1 and IN2
      if (sendReadyToReceiveTimerRanOut && !receivedIN1) {
        fireMessage(new ReadyToReceiveMessage(this),
                    new AgentInterface("IN",1), this);
        setActed(true);
        sendReadyToReceive = true;
      }
      
      if (sendReadyToReceiveTimerRanOut && !receivedIN2) {
        fireMessage(new ReadyToReceiveMessage(this),
                    new AgentInterface("IN",2), this);
        setActed(true);
        sendReadyToReceive = true;
      }
    
      if (sendReadyToReceive)
        readyToReceiveFired();
      
      // Printing to OUTPUT or sending to agentEnvironment
      if (receivedIN1 && receivedIN2) {
        if (IN2 && (synchronizationManager.waitersAreReady())) {
          gui.println(getAgentName() + " : OUTPUT = " + IN1);
          gui.println(getAgentName() + " : Waiters are ready, Firing NumberMessage.");
          synchronizationManager.sendToOutDone();
          NumberMessage message = new NumberMessage(IN1, this);
          AgentInterface agentInterface = new AgentInterface("OUT", 1);
          fireMessage(message, agentInterface, this);
          sendGoodResult = true;
        }
        else {
          gui.println("Agent Out : not authorized OUTPUT = " + IN1);
          println("BURNING = " + IN1);
        }
        receivedIN1 = false;
        receivedIN2 = false;
        setActed(true);
        agentHasComputedItsFunction();
        cancelLongTimeNotActed();
      }
      
      if (!hasActed()) {
        if (debug) {
          if (!receivedIN1)
            gui.println(getAgentName() + " : DEBUG : waiting for IN1");
          if (!receivedIN2)
            gui.println(getAgentName() + " : DEBUG : waiting for IN2");
        }
        //pause(mainDelay + 10000);
        startAgent();
        //setActed(true);
      }
    }
    else
      stopAgent();
  }

  public synchronized void print(String text) {
    output.print(text);
  }

  public synchronized void println( String text) {
    this.print( text + newline );
  }
  
  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);

    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE)) {
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        if (sender.equals(((AgentOutAcquaintancesManager)acquaintancesManager).getAgentIN1())) {
          if (!(receivedIN1)) {
            receivedIN1 = true;
            IN1 = ((NumberMessage)message).getValue();
            reorganizationManager.setLastInputValue(1, IN1);
            reorganizationManager.updatePotentialInput();
            gui.println("Agent Out : Getting IN1");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println("Agent Out : ERROR : not ready to receive from [ "
                        + sender.getAgentName() + " ] IN1.", java.awt.Color.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println("Agent Out : ERROR : Unknown sender of the NumberMessage : [ "
                      + sender.getAgentName() + " ].", java.awt.Color.red);
        }
      }else // not in cooperative state and receiving DataMessage
        super.receivedDataMessageWhileNotInCooperativeState(message);
    } else if (typeId.equals(Message.BOOLEAN_MESSAGE)) {
        if (inCooperativeState) {
          super.receivedDataMessage(message);
          Agent sender = message.getSender();
          if (sender.equals(((AgentOutAcquaintancesManager)acquaintancesManager).getAgentIN2())) {
            if (!(receivedIN2)) {
              receivedIN2 = true;
              IN2 = ((BooleanMessage)message).getTruthValue();
              gui.println("Agent Out : Getting IN2");
            }
            else {
              SystemPrinter.printSystem("ERROR : " + getAgentName());
              errorHappened();
              gui.println("Agent Out : ERROR : not ready to receive from [ "
                          + sender.getAgentName() + " ] IN2.", java.awt.Color.red);
            }
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println("Agent Out : ERROR : Unknown sender of the BooleanMessage : [ "
                        + sender.getAgentName() + " ].", java.awt.Color.red);
            gui.println("Acquaintances : ");
            acquaintancesManager.showAllAcquaintances();
          }
        } else // not in cooperative state and receiving DataMessage
          super.receivedDataMessageWhileNotInCooperativeState(message);
    } else if (typeId.equals(Message.NCS_INADEQUATE_DATA_MESSAGE)) {
        sendGoodResult = false;
        reorganizationManager.processMessage(message);
    }
    else
      super.receiveMessage(message);
 
    setActed(true);
    startAgent();
  }

  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  	receivedIN2 = false;
  	receivedIN1 = false;
    
    
//    AgentOutAcquaintancesManager acqMng = (AgentOutAcquaintancesManager)acquaintancesManager;
//    
//    if (! acqMng.possessingIN2()) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    }
//    if (!acqMng.possessingIN1()) {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
    /*
    if (acqMng.possessingIN1() 
       && ! acqMng.possessingIN2()) {
      receivedIN2 = false;
      sendReadyIN2 = false;
    }
    else if (!acqMng.possessingIN1() 
             && acqMng.possessingIN2()) {
      receivedIN1 = false;
      sendReadyIN1 = false;
    }
    /*
    else {
      SystemPrinter.printSystem("ERROR : " + getAgentName());
      gui.println(getAgentName() 
                  + " : ERROR : received cancelledCooperation notification" 
                  + " but still possessing IN1 and IN2", java.awt.Color.red);
    }*/
  }
  
  public void retryCalculus() {
  	// Retry on both IN 
  	receivedIN2 = false;
  	receivedIN1 = false;
  	
    // Retry only on one IN
//    if (blockingSituationCommutator) {
//      receivedIN2 = false;
//      sendReadyIN2 = false;
//    } else {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
//    switchBockingSituationCommutator();
  }
  
}