package epe.agents.specificAgents.interfaceAgents;

	import epe.gui.CommandPanel;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.specificManagers.interfaceAgents
						.AgentOutWithoutControlAcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.data.NumberMessage;
	import epe.communication.messages.data.ReadyToReceiveMessage;
	import epe.observationTools.graphLayoutTool.GraphLayoutToolInterface;
	import epe.reorganization.specificManagers.interfaceAgents
						.AgentOutWithoutControlReorganizationManager;
	import epe.synchronization.SynchronizationManager;

/**
 * How it works : 1 IN (mono, int), 1 OUT(mono) : sends IN1 
 *                to the agentEnvironment
 */
public class AgentOutWithoutControl extends Agent {

	  private boolean receivedIN1 = false;
	  private boolean sendGoodResult = false;
	  private int IN1;  // Input <Value> (Integer)
	  private CommandPanel output;
	  private String newline = "\n";
	  
	  // Let the system work step by step
	  protected boolean stepByStep = false;
	  protected boolean stepByStepControlEnable = true;
  
  /** Creates new AgentOut */
  public AgentOutWithoutControl(String suffixe, CommandPanel commandPanel) {
    super("Out" + suffixe);
    acquaintancesManager = new AgentOutWithoutControlAcquaintancesManager(this);
    synchronizationManager = new SynchronizationManager(acquaintancesManager, this);
    reorganizationManager = new AgentOutWithoutControlReorganizationManager(this);
    output = commandPanel;
    SystemPrinter.printSystem(getAgentName() + " created");
  }
  
  protected void graphLayoutToolAgentInitialisation() {
    getProgrammingEnvironment().getGraphLayoutTool()
                               .addAgent(getAgentName(),
                                         GraphLayoutToolInterface.OUTPUT_AGENT,
                                         java.awt.Color.lightGray);
  }

  public void reInitializeAgent() {
    super.reInitializeAgent();
    receivedIN1 = false;
    sendGoodResult = false;
  }
  
  protected void acting() {
    
    if ((!sendGoodResult) && (stepByStepControlEnable)) {
      setStepByStep(false);
      setActed(false);
      // Firing ready messages to IN1 and IN2
      if (sendReadyToReceiveTimerRanOut && !receivedIN1) {
        fireMessage(new ReadyToReceiveMessage(this),
                    new AgentInterface("IN",1), this);
        setActed(true);
        readyToReceiveFired();
      }
   
      // Printing to OUTPUT or sending to agentEnvironment
      if (receivedIN1 && (synchronizationManager.waitersAreReady())) {
        gui.println(getAgentName() + " : OUTPUT = " + IN1);
        gui.println(getAgentName() + " : Waiters are ready, Firing NumberMessage.");
        synchronizationManager.sendToOutDone();
        NumberMessage message = new NumberMessage(IN1, this);
        AgentInterface agentInterface = new AgentInterface("OUT", 1);
        fireMessage(message, agentInterface, this);
        // ###DEBUG //sendGoodResult = true;
        
        reorganizationManager.setLastOutputValue(IN1);
        //reorganizationManager.updatePotentialOuput();
        
        receivedIN1 = false;
        setActed(true);
        agentHasComputedItsFunction();
        cancelLongTimeNotActed();
      }
      
      if (!hasActed()) {
        if (debug) {
          if (!receivedIN1)
            gui.println(getAgentName() + " : DEBUG : waiting for IN1");
        }
        //pause(mainDelay + 10000);
        startAgent();
        //setActed(true);
      }
    }
    else {
      stopAgent();
      setStepByStep(true);
    }
  }

  protected void setStepByStep(boolean truthValue) {
    if (stepByStep)
      stepByStepControlEnable = truthValue;
  }
  
  public synchronized void print(String text) {
    output.print(text);
  }

  public synchronized void println( String text) {
    this.print( text + newline );
  }
  
  public synchronized void receiveMessage(Message message) {
    String typeId = message.getId();
    messageManager.receivedMessage(message);

    // Agents have to be in cooperative state to accept data for the calculus
    if (typeId.equals(Message.NUMBER_MESSAGE)) {
      if (inCooperativeState) {
        super.receivedDataMessage(message);
        Agent sender = message.getSender();
        if (sender.equals(((AgentOutWithoutControlAcquaintancesManager)acquaintancesManager)
                              .getAgentIN1())) {
          if (!(receivedIN1)) {
            receivedIN1 = true;
            IN1 = ((NumberMessage)message).getValue();
            reorganizationManager.setLastInputValue(1, IN1);
            //reorganizationManager.updatePotentialInput();
            // => should not always update GoalReached
            gui.println("Agent Out : Getting IN1");
          }
          else {
            SystemPrinter.printSystem("ERROR : " + getAgentName());
            errorHappened();
            gui.println("Agent Out : ERROR : not ready to receive from [ "
                        + sender.getAgentName() + " ] IN1.", java.awt.Color.red);
          }
        }
        else {
          SystemPrinter.printSystem("ERROR : " + getAgentName());
          errorHappened();
          gui.println("Agent Out : ERROR : Unknown sender of the NumberMessage : [ "
                      + sender.getAgentName() + " ].", java.awt.Color.red);
        }
      }else // not in cooperative state and receiving DataMessage
        super.receivedDataMessageWhileNotInCooperativeState(message);
    } else if (typeId.equals(Message.NCS_INADEQUATE_DATA_MESSAGE)) {
        sendGoodResult = false;
        reorganizationManager.processMessage(message);
    }
    else
      super.receiveMessage(message);
 
    setActed(true);
    startAgent();
  }

  public void cooperationWithInAcquaintanceMayHaveBeenCancelled(Acquaintance acq) {
  	receivedIN1 = false;
    
//    AgentOutWithoutControlAcquaintancesManager acqMng = 
//      (AgentOutWithoutControlAcquaintancesManager)acquaintancesManager;
//    
//    if (!acqMng.possessingIN1()) {
//      receivedIN1 = false;
//      sendReadyIN1 = false;
//    }
    /*
    if (acqMng.possessingIN1() 
       && ! acqMng.possessingIN2()) {
      receivedIN2 = false;
      sendReadyIN2 = false;
    }
    else if (!acqMng.possessingIN1() 
             && acqMng.possessingIN2()) {
      receivedIN1 = false;
      sendReadyIN1 = false;
    }
    /*
    else {
      SystemPrinter.printSystem("ERROR : " + getAgentName());
      gui.println(getAgentName() 
                  + " : ERROR : received cancelledCooperation notification" 
                  + " but still possessing IN1 and IN2", java.awt.Color.red);
    }*/
  }
  
  public void retryCalculus() {
      receivedIN1 = false;
  }
  
}