/*
 * FunctionalProximity.java
 *
 * Created on 16 d?cembre 2002, 18:05
 */

package epe.annexTools.systematicOrganizationsStatisticTool.functionalProximity;

/**
 *
 * @author  george
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import epe.gui.SystemPrinter;
import epe.gui.systematicOrganizationGUI.FunctionalProximityGUI;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.agentListeners.environmentStateChanged.EnvironmentStateChangedListener;
import epe.agents.agentListeners.ncs.NcsListener;
import epe.agents.specificAgents.environment.AgentEnvironment;
import epe.annexTools.systematicOrganizationsStatisticTool.functionalProximity.functionalProximityFileManager.FunctionalProximityFileManager;
import epe.annexTools.systematicOrganizationsStatisticTool.organizationRunner.OrganizationRunner;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentPool.AgentPool;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentsNeedingIN.AgentsNeedingIN;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.possibleOUTs.PossibleOUTs;
import epe.communication.Message;

public class FunctionalProximity implements NcsListener, ActionListener, EnvironmentStateChangedListener {
  
  protected OrganizationRunner organizationRunner;
  protected AgentPool agentPool;
  protected AgentsNeedingIN agentsNeedingIN;
  protected PossibleOUTs possibleOUTs;
  protected int[] links;
  protected int[] factorialLinks = {5,4,1,3,2,4,3,8,0,6,3,7};
  
  protected FunctionalProximityGUI controlGui;
  
  protected boolean pauseSet = false;
  protected boolean pauseOnCooperationSet = false;
  protected boolean started = false;

  protected java.lang.Thread testerRunner;
  protected Tester tester;
  protected boolean newOrganizationHasToBeLoaded = true;
  
   protected boolean hasLoaded = false;
  protected boolean nextFileHasToBeLoaded = false;
  
  // For saving the Organizations & position
  protected FunctionalProximityFileManager organizationSaver;
  //protected TesterPositionSaver positionSaver;
  protected int testPosition = 0;
  protected int savedPosition = 0;
  
  // Timer to detect if an organization is calculating (i.e. has not been blocked after a while)
  protected Timer calculusTimer; 
  protected final static int ONE_SECOND = 1000;
  protected int calculusTimerDelay = 5*ONE_SECOND;
  protected boolean calculusTimerRanOut = true;
  
  /** Creates a new instance of SystematicOrganizationTester */
  public FunctionalProximity() {
    controlGui = new FunctionalProximityGUI(this);
    controlGui.setVisible(true);
    organizationRunner = new OrganizationRunner();
    agentPool = new AgentPool();
    agentPool.initAgentPool(organizationRunner.getProgrammingEnvironment());
    organizationRunner.addAgents(agentPool);
    organizationSaver = new FunctionalProximityFileManager();
    //positionSaver = new TesterPositionSaver();
    agentsNeedingIN = new AgentsNeedingIN(agentPool);
    possibleOUTs = new PossibleOUTs(agentPool);
    
    calculusTimer = new Timer(calculusTimerDelay, this);
    calculusTimer.setRepeats(false);
    
    //Environment State listener
    ((AgentEnvironment)(agentPool.getAgentAt(10))).addEnvironmentStateChangedListener(this);
    
    runAnOrganization();
    initAsAgentNcsListener();
    
    startTester();
  }

  protected void initAsAgentNcsListener() {
    for (int i = 0; i < agentPool.getSize(); i++) {
      Agent agent = (Agent)agentPool.getAgentAt(i);
      agent.addNcsListener(this);
    }
  }

  public synchronized void startTester() {
    tester = new Tester();
    java.lang.Thread testRunner = new java.lang.Thread(tester);
    pauseSet = false;
    testRunner.start();
  }
  
  protected void reInitializeAgents() {
    for (int i = 0; i < agentPool.getSize(); i++) {
      Agent agent = (Agent)agentPool.getAgentAt(i);
      agent.reInitializeAgent();
    }
  }
  
  public void loadOrganization() {
    clearOldOrganization();
    links = organizationSaver.loadOrganization();
    if (links != null) {
      if (links[0] == -1)
        pause();
      else {
        testPosition = organizationSaver.getLoadOrganizationCounter();
        buildNewOrganization(); 
        hasLoaded = true;
      }
    }
    //else
      //nextFileHasToBeLoaded = true;
    //links = factorialLinks;
  }
  
  protected void launchCalculus() {
    organizationRunner.getProgrammingEnvironment().getCommandPanel().automaticButtonTrigger("14");
    organizationRunner.actionPerformed(new java.awt.event.ActionEvent(this, 1, "Start Working"));
    fireCalculusTimer();
    hasLoaded = false;
  }
  
  
  protected int createLink(int i) {
    String returnMessage = "ERROR";
    // connecting to IN
    returnMessage = agentsNeedingIN.getAgentAt(i)
                      .addAcquaintanceWithReturn(possibleOUTs.getAgentAt(links[i]), 
                                                 "IN", 
                                                 agentsNeedingIN.getAgentPositionAt(i),
                                                 1);
    //System.out.println("ReturnMessage : " + returnMessage);
    if (! returnMessage.equals("SUCCESS"))
      return 1;
    else {
      // connecting to OUT
      returnMessage = possibleOUTs.getAgentAt(links[i])
                        .addAcquaintanceWithReturn(agentsNeedingIN.getAgentAt(i), 
                                                   "OUT", 
                                                   1,
                                                   agentsNeedingIN.getAgentPositionAt(i));
      //System.out.println("ReturnMessage : " + returnMessage);
      if (! returnMessage.equals("SUCCESS"))
        return 1;
      else return 0;
    }
    //System.out.println("One link created");
  }
  
  protected int createLinks() {
    int numberOfErrors = 0;
    
    int i=0;
    while ((i<12) && (numberOfErrors==0)) { // build all the links
      numberOfErrors = numberOfErrors + createLink(i);
      i++;
    }
    return numberOfErrors;
  }

  protected void destroyLink(int i) {
    Agent producerAgent = (Agent)possibleOUTs.getAgentAt(links[i]);
    Agent receiverAgent = (Agent)agentsNeedingIN.getAgentAt(i);
    /*System.out.println("Trying to destroy link : " 
                        + producerAgent.getAgentName() 
                        + "--->" 
                        + receiverAgent.getAgentName());*/
    // receiver's acq deleting
    Acquaintance acqProd = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("IN", 
                                             agentsNeedingIN.getAgentPositionAt(i));
    receiverAgent.getAcquaintancesManager()
                 .cancelCooperation(acqProd);
    Acquaintance acqProdNeutral = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("NEUTRAL", 0);
    receiverAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqProdNeutral);
    // producer's acq deleting
    Acquaintance acqRec = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "OUT");
    producerAgent.getAcquaintancesManager()
                 .cancelCooperation(acqRec);
    Acquaintance acqRecNeutral = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "NEUTRAL");
    producerAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqRecNeutral);
  }
  
  protected void runAnOrganization() {
    organizationRunner.initTest();
  }

  protected void clearOldOrganization() {
    // Clear the old organization
    if (links != null)
      for (int i=0; i<12; i++)
        destroyLink(i);
  }
  
  protected void buildNewOrganization() {
    // Create the new Organization
    int i = createLinks();
    if (i!=0)
      System.out.println("ERROR : while creating/updating an organization");
    else
      System.out.println("Organization created/updated");
  }
  
  public void showLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(true);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void hideLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(false);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void showAgentOutput() {
    for (int i = 0; i<11; i++)
       ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(true);
  }
  
  public void removeAgentOutput() {
    for (int i = 0; i<11; i++)
      ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(false);
  }
  
  
  // Concerning the NcsListener
  public void ncsCreated() {
  }
  
  public void ncsCreated(String type) {
//    SystemPrinter.printSystem("TESTER received SNC : " + type);
//    if (type.equals("NcsBlockingSituationRetryCalculus")
//        ||type.equals("NcsBlockingSituationDeceaseTrustvalue")) {
//      organizationRunner.actionPerformed(new java.awt.event.ActionEvent(this, 1, "Stop all agents"));
//      //reInitializeAgents();
//      newOrganizationHasToBeLoaded = true;
//      tester.notifyThread();
//    }
  }
  
  public void ncsCreated(Message message) {
  }
  // End of NcsListener

  
  public void fireCalculusTimer() {
    calculusTimerRanOut = false;
    calculusTimer.restart();
  }
  
  // Action listener for the timer
  public void actionPerformed(ActionEvent e) {
//    if ((e.getSource()).equals(calculusTimer) && !pauseSet) {
//      SystemPrinter.printSystem("CalculusTimer ran out : too long to give an output or impossible");
//      calculusTimerRanOut = true;
//      organizationRunner.actionPerformed(new java.awt.event.ActionEvent(this, 1, "Stop all agents"));
//      newOrganizationHasToBeLoaded = true;
//      tester.notifyThread();
//    }
  }
  // End of action listener for the timer
  
  protected void waitForAgentReinitialization() {
      synchronized(this) {
      try {
        SystemPrinter.printSystem("TESTER THREAD : " +
                                  "Waiting for a while to be sure all agents are reinitialized");
        System.out.println();
        System.out.println();
        System.out.println();
        Thread.sleep(1000);
        tester.notifyThread();
      } catch(InterruptedException e) {SystemPrinter.printSystem("INTERRUPTED : " + e);}
    }
    }
  
  public void pause() {
    pauseSet = true;
  }
  
  public void unPause() {
    pauseSet = false;
    tester.notifyThread();
  }
  
  public void startTesting() {
    started = true;
    tester.notifyThread();
  }
  
//  public void savePosition() {
//    positionSaver.savePosition(testPosition, 
//                               organizationSaver.getTestedOrganizationNumber(), 
//                               organizationSaver.getSaveOrganizationCounter());
//  }
  
//  public void loadPosition() {
//    positionSaver.loadPosition();
//    testPosition = positionSaver.getPosition();
//    savedPosition = positionSaver.getSavedPosition();
//    organizationSaver.setLoadOrganizationCounter(testPosition);
//    organizationSaver.setSaveOrganizationCounter(savedPosition);
//    organizationSaver.positionTheReader(testPosition);
//  }
//  
  // Concerning the EnvironmentStateChangedListener
  public void environmentReceivedData(int data) {
    SystemPrinter.printSystem("Environment State Changed : found a functional organization");
    SystemPrinter.printSystem("SAVING ONE CALCULATING ORGANIZATION");
    pause();
    organizationRunner.actionPerformed(new java.awt.event.ActionEvent(this, 1, "Stop all agents"));
    organizationSaver.saveOrganization(links, organizationSaver.getTestedOrganizationNumber(),
                                              organizationSaver.getOriginalOrganizationNumber(), 
                                              data);
    newOrganizationHasToBeLoaded = true;
    unPause();
  }
  
  // Inner class to be run as a thread
  public class Tester implements java.lang.Runnable {
    
    public Tester() {
    }
    
    public void run() {
      testTheOrganizations();
    }
    
    protected void testTheOrganizations() {
      waitForEvent();
      while (true) {
        if (started && newOrganizationHasToBeLoaded) {
          if (pauseSet)
              waitForEvent();
          SystemPrinter.printSystem("NEXT ORGANIZATION");
          loadOrganization();
          newOrganizationHasToBeLoaded = false;
          if (hasLoaded)
            launchCalculus();
          else
            newOrganizationHasToBeLoaded = true;
          
        }
        if (!nextFileHasToBeLoaded) {
          waitForEvent();
          waitForAgentReinitialization();
        }
        else
          nextFileHasToBeLoaded = false;
      }
    }
       
       
    protected void waitForEvent() {
      synchronized(this) {
      try {
        SystemPrinter.printSystem("Waiting");
        this.wait();
      } catch(InterruptedException e) {
      }
    }
    }
    
    protected synchronized void notifyThread() {
     notifyAll(); 
    }
    
  }
  // End of inner class
  
}

