  /*
 * SelfOrganizingPseudoFactorielle.java
 *
 * Created on 23 avril 2000, 19:16
 */

package epe.annexTools.systematicOrganizationsStatisticTool.organizationRunner;

import epe.calculus.Test;
import epe.gui.SystemPrinter;
import epe.agents.specificAgents.AgentEquals;
import epe.agents.specificAgents.AgentAddition;
import epe.agents.specificAgents.AgentMultiplication;
import epe.agents.specificAgents.AgentFirstTime;
import epe.agents.specificAgents.environment.AgentEnvironment;
import epe.agents.specificAgents.interfaceAgents.AgentIn;
import epe.agents.specificAgents.interfaceAgents.AgentOut;
import epe.agents.specificAgents.practicalAgents.AgentOne;

import epe.observationTools.ncsObserver.NcsObserver;
import epe.observationTools.messageSendObserver.MessageSendObserver;

import epe.gui.ProgrammingEnvironment ;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentPool.AgentPool;
import epe.agents.acquaintances.TrustValue;


/**
 *
 * @author  Jean-Pierre Georg?
 * @version
 */
public class OrganizationRunner extends Test implements ActionListener, Runnable{
  
    protected Thread executingThread;
    protected boolean stopAllAgentsOrderSend = false;
    protected boolean startWorkingOrderSend = false;
    protected final static int ONE_SECOND = 1000;
    protected int delay = ONE_SECOND/2;

    protected AgentEnvironment agentEnvironment;
    protected AgentOne agentOne1, agentOne2, agentOne3;
    protected AgentOut agentOut;
    protected AgentIn agentIn;
    protected AgentEquals agentEquals;
    protected AgentAddition agentAddition;
    protected AgentMultiplication agentMultiplication;
    protected AgentFirstTime agentFirstTime1, agentFirstTime2;
    
    protected ProgrammingEnvironment programmingEnvironment;
    protected NcsObserver ncsObserver;
    protected MessageSendObserver messageSendObserver;

    /** Creates new SelfOrganizingPseudoFactorial */ 
    public OrganizationRunner() {
    executingThread = new Thread(this);
    executingThread.start();
    programmingEnvironment = new ProgrammingEnvironment();
    ncsObserver = new NcsObserver();
    messageSendObserver = new MessageSendObserver();
    programmingEnvironment.setVisible(true);
    programmingEnvironment.getCommandPanel().initTest(this);
  }
  
  public void addAgents(AgentPool agentPool) {
    for (int i = 0; i < agentPool.getSize(); i++) {
      switch(i) {
        case 0 : this.agentOne1 = (AgentOne)agentPool.getAgentAt(i); break;
        case 1 : this.agentOne2 = (AgentOne)agentPool.getAgentAt(i); break;
        case 2 : this.agentOne3 = (AgentOne)agentPool.getAgentAt(i); break;
        case 3 : this.agentAddition = (AgentAddition)agentPool.getAgentAt(i); break;
        case 4 : this.agentMultiplication = (AgentMultiplication)agentPool.getAgentAt(i); break;
        case 5 : this.agentEquals = (AgentEquals)agentPool.getAgentAt(i); break;
        case 6 : this.agentFirstTime1 = (AgentFirstTime)agentPool.getAgentAt(i); break;
        case 7 : this.agentFirstTime2 = (AgentFirstTime)agentPool.getAgentAt(i); break;
        case 8 : this.agentIn = (AgentIn)agentPool.getAgentAt(i); break;
        case 9 : this.agentOut = (AgentOut)agentPool.getAgentAt(i); break;
        case 10 : this.agentEnvironment = (AgentEnvironment)agentPool.getAgentAt(i); break;
      }
    }
    agentEnvironment.setProgrammingEnvironment(programmingEnvironment);
    agentEnvironment.addNcsListener(ncsObserver);
    agentEnvironment.addMessageSendListener(messageSendObserver);
    
    agentOne1.setProgrammingEnvironment(programmingEnvironment);
    agentOne1.addNcsListener(ncsObserver);
    agentOne1.addMessageSendListener(messageSendObserver);

    agentOne2.setProgrammingEnvironment(programmingEnvironment);
    agentOne2.addNcsListener(ncsObserver);
    agentOne2.addMessageSendListener(messageSendObserver);
    
    agentOne3.setProgrammingEnvironment(programmingEnvironment);
    agentOne3.addNcsListener(ncsObserver);
    agentOne3.addMessageSendListener(messageSendObserver);
    
    agentAddition.setProgrammingEnvironment(programmingEnvironment);
    agentAddition.addNcsListener(ncsObserver);
    agentAddition.addMessageSendListener(messageSendObserver);

    agentMultiplication.setProgrammingEnvironment(programmingEnvironment);
    agentMultiplication.addNcsListener(ncsObserver);
    agentMultiplication.addMessageSendListener(messageSendObserver);
    
    agentFirstTime1.setProgrammingEnvironment(programmingEnvironment);
    agentFirstTime1.addNcsListener(ncsObserver);
    agentFirstTime1.addMessageSendListener(messageSendObserver);
    
    agentFirstTime2.setProgrammingEnvironment(programmingEnvironment);
    agentFirstTime2.addNcsListener(ncsObserver);
    agentFirstTime2.addMessageSendListener(messageSendObserver);
    
    agentOut.setProgrammingEnvironment(programmingEnvironment);
    agentOut.addNcsListener(ncsObserver);
    agentOut.addMessageSendListener(messageSendObserver);
    
    agentIn.setProgrammingEnvironment(programmingEnvironment);
    agentIn.addNcsListener(ncsObserver);
    agentIn.addMessageSendListener(messageSendObserver);
    
    agentEquals.setProgrammingEnvironment(programmingEnvironment);
    agentEquals.addNcsListener(ncsObserver);
    agentEquals.addMessageSendListener(messageSendObserver);
  }
    
  public void initTest() {
    agentOne1.setLocation(0,0);
    agentOne2.setLocation(0,1);
    agentOne3.setLocation(0,2);
    agentAddition.setLocation(2,0);
    agentMultiplication.setLocation(2,1);
    agentFirstTime1.setLocation(2,4);
    agentFirstTime2.setLocation(2,5);
    agentOut.setLocation(5,3);
    agentIn.setLocation(0,5);
    agentEquals.setLocation(4,4);
    agentEnvironment.setLocation(3,8);

    agentEnvironment.start();
    agentOne1.start();
    agentOne2.start();
    agentOne3.start();
    agentAddition.start();
    agentMultiplication.start();
    agentFirstTime1.start();
    agentFirstTime2.start();
    agentOut.start();
    agentIn.start();
    agentEquals.start();

    // Environment, In
    agentEnvironment.addAcquaintance(agentIn, "OUT", 1, TrustValue.FIXEDTRUSTVALUE);
    agentIn.addAcquaintance(agentEnvironment, "IN", 1, TrustValue.FIXEDTRUSTVALUE);
    // Out, Environment
    agentOut.addAcquaintance(agentEnvironment, "OUT", 1, TrustValue.FIXEDTRUSTVALUE);
    agentEnvironment.addAcquaintance(agentOut, "IN", 1, TrustValue.FIXEDTRUSTVALUE);
   
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentOne1);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentOne2);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentOne3);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentOut);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentIn);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentEquals);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentAddition);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentMultiplication);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentFirstTime1);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentFirstTime2);
  
  }
 
  public ProgrammingEnvironment getProgrammingEnvironment() {
    return programmingEnvironment;
  }
  
  protected synchronized void pause(int delay) {
    SystemPrinter.printSystem("Waiting a little.");
      try {
        wait(delay);
      } catch (InterruptedException e) {
      }
  }

  public synchronized void actionPerformed(ActionEvent ev) {
    if (ev.getActionCommand().equals("Stop all agents")) {
      SystemPrinter.printSystem("Received event : stop all agents.");
      stopAllAgentsOrderSend = true;
      notifyAll();
    }
    else { //(ev.getActionCommand().equals("Start Working")) 
      SystemPrinter.printSystem("Received event : start working.");
      startWorkingOrderSend = true;
      notifyAll();
    }
    
    //else gui.printSystem("ERROR : Test : unknown event : " + ev.getActionCommand());
  }

  public void run() {
    while (true) {
      if (stopAllAgentsOrderSend) {
        agentEnvironment.disableAgent();
        agentOne1.disableAgent();
        agentOne2.disableAgent();
        agentOne3.disableAgent();
        agentFirstTime1.disableAgent();
        agentFirstTime2.disableAgent();
        agentAddition.disableAgent();
        agentMultiplication.disableAgent();
        agentEquals.disableAgent();
        agentOut.disableAgent();
        agentIn.disableAgent();
        
        stopAllAgentsOrderSend = false;
      }
      if (startWorkingOrderSend) {
        agentEnvironment.disableAgent();
        agentOne1.disableAgent();
        agentOne2.disableAgent();
        agentOne3.disableAgent();
        agentFirstTime1.disableAgent();
        agentFirstTime2.disableAgent();
        agentAddition.disableAgent();
        agentMultiplication.disableAgent();
        agentEquals.disableAgent();
        agentOut.disableAgent();
        agentIn.disableAgent();

        agentEnvironment.reInitializeOrder();
        agentOne1.reInitializeOrder();
        agentOne2.reInitializeOrder();
        agentOne3.reInitializeOrder();
        agentFirstTime1.reInitializeOrder();
        agentFirstTime2.reInitializeOrder();
        agentAddition.reInitializeOrder();
        agentMultiplication.reInitializeOrder();
        agentEquals.reInitializeOrder();
        agentOut.reInitializeOrder();
        agentIn.reInitializeOrder();

        pause(delay);
        
        agentEnvironment.enableAgent();
        agentOne1.enableAgent();
        agentOne2.enableAgent();
        agentOne3.enableAgent();
        agentFirstTime1.enableAgent();
        agentFirstTime2.enableAgent();
        agentAddition.enableAgent();
        agentMultiplication.enableAgent();
        agentEquals.enableAgent();
        agentOut.enableAgent();
        agentIn.enableAgent();

        agentEnvironment.startAgent();
        agentOne1.startAgent();
        agentOne2.startAgent();
        agentOne3.startAgent();
        agentFirstTime1.startAgent();
        agentFirstTime2.startAgent();
        agentAddition.startAgent();
        agentMultiplication.startAgent();
        agentEquals.startAgent();
        agentOut.startAgent();
        agentIn.startAgent();
        
        startWorkingOrderSend = false;
      }
      
      synchronized(this) {
          try {
            SystemPrinter.printSystem("Waiting for an event.");
              this.wait();
          } catch(InterruptedException e) {}
      }
    }
  }
  
}

