/*
 * ProximityTool.java
 *
 * Created on 25 f?vrier 2003, 14:47
 */

package epe.annexTools.systematicOrganizationsStatisticTool.organizationalProximity;

import epe.gui.SystemPrinter;
import epe.gui.systematicOrganizationGUI.ProximityToolGUI;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.annexTools.systematicOrganizationsStatisticTool.organizationRunner.OrganizationRunner;
import epe.annexTools.systematicOrganizationsStatisticTool.organizationalProximity.organizationalProximityFileManager.OrganizationalProximityFileManager;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentPool.AgentPool;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentsNeedingIN.AgentsNeedingIN;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.possibleOUTs.PossibleOUTs;

/**
 *
 * @author  george
 */
public class ProximityTool {

  protected OrganizationalProximityCalculator calculator;
  protected OrganizationalProximityFileManager fileManager;
  
  protected OrganizationRunner organizationRunner;
  protected AgentPool agentPool;
  protected AgentsNeedingIN agentsNeedingIN;
  protected PossibleOUTs possibleOUTs;
  protected int[] links;

  protected ProximityToolGUI controlGui;
  
  protected boolean pauseSet = false;
  protected boolean pauseOnCooperationSet = false;
  protected boolean started = false;

  protected java.lang.Thread testerRunner;
  protected Tester tester;
  protected boolean newOrganizationHasToBeLoaded = true;
  
//  // For saving the Organizations & position
//  protected OrganizationSaver organizationSaver;
//  protected TesterPositionSaver positionSaver;
  protected int testPosition = 0;
//  protected int savedPosition = 0;
//  protected CalculatingOrganizationSaver calculatingOrganizationSaver;
  
 
  // Creates a new instance of ProximityTool 
  public ProximityTool() {
    calculator = new OrganizationalProximityCalculator();
    fileManager = new OrganizationalProximityFileManager();
    
    //cut paste
    controlGui = new ProximityToolGUI(this);
    controlGui.setVisible(true);
    organizationRunner = new OrganizationRunner();
    agentPool = new AgentPool();
    agentPool.initAgentPool(organizationRunner.getProgrammingEnvironment());
    organizationRunner.addAgents(agentPool);
    //positionSaver = new TesterPositionSaver();
    agentsNeedingIN = new AgentsNeedingIN(agentPool);
    possibleOUTs = new PossibleOUTs(agentPool);
    
    runAnOrganization();
    
    startTester();
  }

  public synchronized void startTester() {
    tester = new Tester();
    java.lang.Thread testRunner = new java.lang.Thread(tester);
    pauseSet = false;
    testRunner.start();
  }
  
  protected void reInitializeAgents() {
    for (int i = 0; i < agentPool.getSize(); i++) {
      Agent agent = (Agent)agentPool.getAgentAt(i);
      agent.reInitializeAgent();
    }
  }
  
  public void loadOrganization() {
    //clearOldOrganization();
    links = fileManager.loadOrganization();
    testPosition = fileManager.getLoadOrganizationCounter();
    //buildNewOrganization();
  }
  
  protected void launchCalculus() {
    int prox = calculator.bestProximity(links);
    System.out.println("Proximity : " + prox);
    
    fileManager.saveOrganization(links, 
                                 fileManager.getTestedOrganizationNumber(),
                                 fileManager.getOriginalOrganizationNumber(),
                                 prox);
    
    //organizationRunner.getProgrammingEnvironment().getCommandPanel().automaticButtonTrigger("4");
    //organizationRunner.actionPerformed(new java.awt.event.ActionEvent(this, 1, "Start Working"));
  }
  
  
  protected int createLink(int i) {
    String returnMessage = "ERROR";
    // connecting to IN
    returnMessage = agentsNeedingIN.getAgentAt(i)
                      .addAcquaintanceWithReturn(possibleOUTs.getAgentAt(links[i]), 
                                                 "IN", 
                                                 agentsNeedingIN.getAgentPositionAt(i),
                                                 1);
    //System.out.println("ReturnMessage : " + returnMessage);
    if (! returnMessage.equals("SUCCESS"))
      return 1;
    else {
      // connecting to OUT
      returnMessage = possibleOUTs.getAgentAt(links[i])
                        .addAcquaintanceWithReturn(agentsNeedingIN.getAgentAt(i), 
                                                   "OUT", 
                                                   1,
                                                   agentsNeedingIN.getAgentPositionAt(i));
      //System.out.println("ReturnMessage : " + returnMessage);
      if (! returnMessage.equals("SUCCESS"))
        return 1;
      else return 0;
    }
    //System.out.println("One link created");
  }
  
  protected int createLinks() {
    int numberOfErrors = 0;
    
    int i=0;
    while ((i<12) && (numberOfErrors==0)) { // build all the links
      numberOfErrors = numberOfErrors + createLink(i);
      i++;
    }
    return numberOfErrors;
  }

  protected void destroyLink(int i) {
    Agent producerAgent = (Agent)possibleOUTs.getAgentAt(links[i]);
    Agent receiverAgent = (Agent)agentsNeedingIN.getAgentAt(i);
    /*System.out.println("Trying to destroy link : " 
                        + producerAgent.getAgentName() 
                        + "--->" 
                        + receiverAgent.getAgentName());*/
    // receiver's acq deleting
    Acquaintance acqProd = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("IN", 
                                             agentsNeedingIN.getAgentPositionAt(i));
    receiverAgent.getAcquaintancesManager()
                 .cancelCooperation(acqProd);
    Acquaintance acqProdNeutral = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("NEUTRAL", 0);
    receiverAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqProdNeutral);
    // producer's acq deleting
    Acquaintance acqRec = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "OUT");
    producerAgent.getAcquaintancesManager()
                 .cancelCooperation(acqRec);
    Acquaintance acqRecNeutral = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "NEUTRAL");
    producerAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqRecNeutral);
  }
  
  protected void runAnOrganization() {
    organizationRunner.initTest();
  }

  protected void clearOldOrganization() {
    // Clear the old organization
    if (links != null)
      for (int i=0; i<12; i++)
        destroyLink(i);
  }
  
  protected void buildNewOrganization() {
    // Create the new Organization
    int i = createLinks();
    if (i!=0)
      System.out.println("ERROR : while creating/updating an organization");
    else
      System.out.println("Organization created/updated");
  }
  
  public void showLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(true);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void hideLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(false);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void showAgentOutput() {
    for (int i = 0; i<11; i++)
       ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(true);
  }
  
  public void removeAgentOutput() {
    for (int i = 0; i<11; i++)
      ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(false);
  }
  
  protected void waitForAgentReinitialization() {
      synchronized(this) {
      try {
        SystemPrinter.printSystem("TESTER THREAD : Waiting for a while to be sure all agents" 
                                  +"are reinitialized");
        System.out.println();
        System.out.println();
        System.out.println();
        Thread.sleep(1000);
        tester.notifyThread();
      } catch(InterruptedException e) {SystemPrinter.printSystem("INTERRUPTED : " + e);}
    }
    }
  
  public void pause() {
    pauseSet = true;
  }
  
  public void unPause() {
    pauseSet = false;
    tester.notifyThread();
  }
  
  public void startTesting() {
    started = true;
    tester.notifyThread();
  }
  
  public void nextTesting() {
    newOrganizationHasToBeLoaded = true;
    tester.notifyThread();
  }
  
  // Inner class to be run as a thread
  public class Tester implements java.lang.Runnable {
    
    public Tester() {
    }
    
    public void run() {
      testTheOrganizations();
    }
    
    protected void testTheOrganizations() {
      waitForEvent();
      while (true) {
        if (started && newOrganizationHasToBeLoaded) {
          if (pauseSet)
              waitForEvent();
          SystemPrinter.printSystem("NEXT ORGANIZATION");
          loadOrganization();
          //newOrganizationHasToBeLoaded = false;
          launchCalculus();
          
        }
        //waitForEvent();
        //waitForAgentReinitialization();
      }
    }
       
    protected void waitForEvent() {
      synchronized(this) {
      try {
        SystemPrinter.printSystem("Waiting");
        this.wait();
      } catch(InterruptedException e) {
      }
    }
    }
    
    protected synchronized void notifyThread() {
     notifyAll(); 
    }
    
  }
  // End of inner class

}
