package epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder;


import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentPool.AgentPool;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.agentsNeedingIN.AgentsNeedingIN;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.possibleOUTs.PossibleOUTs;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.organizationSaver.OrganizationSaver;
import epe.annexTools.systematicOrganizationsStatisticTool.systematicOrganizationBuilder.loopStateSaver.LoopStateSaver;
import epe.annexTools.systematicOrganizationsStatisticTool.organizationRunner.OrganizationRunner;


import epe.gui.systematicOrganizationGUI.SystematicOrganizationGUI;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;


public class SystematicOrganizationBuilder {
  
  protected OrganizationRunner organizationRunner;
  protected AgentPool agentPool;
  protected AgentsNeedingIN agentsNeedingIN;
  protected PossibleOUTs possibleOUTs;
  protected int[] links;
  protected int[] loopInit = {0,0,0,0,0,0,0,0,0,0,0,0};
  
  protected SystematicOrganizationGUI controlGui;
  protected boolean isAgentGUIShown = true;
  
  protected boolean pauseSet = false;
  protected boolean pauseOnCooperationSet = false;

  protected Builder builder;
  protected java.lang.Thread builderRunner;
  
  // for the recursiveFunction
  double numberOfBuiltOrganizations = 0;
  double numberOfSearchedOrganizations = 0;
  double numberOfCooperativeOrganizations = 0;
  int numberOfErrors = 0;
  
  // For saving the Organizations
  protected OrganizationSaver organizationSaver;
  protected LoopStateSaver loopStateSaver;
  
  /** Creates a new instance of SystematicOrganizationBuilder */
  public SystematicOrganizationBuilder() {
    controlGui = new SystematicOrganizationGUI(this);
    controlGui.setVisible(true);
    organizationRunner = new OrganizationRunner();
    agentPool = new AgentPool();
    agentPool.initAgentPool(organizationRunner.getProgrammingEnvironment());
    organizationRunner.addAgents(agentPool);
    organizationSaver = new OrganizationSaver();
    loopStateSaver = new LoopStateSaver();
    
    initOrganization();
  }
  
  // Inner class to be run as a thread
  public class Builder implements java.lang.Runnable {
    
    public Builder() {
    }
    
    public void run() {
      buildAnOrganization();
    }
    
    protected void buildAnOrganization() {
      agentsNeedingIN = new AgentsNeedingIN(agentPool);
      possibleOUTs = new PossibleOUTs(agentPool);
      links = new int[12];
      toggleAgentGUI();
      recursiveBuilder(0);
      System.out.println("All organizations built : " + numberOfBuiltOrganizations);
      System.out.println("All organizations built : " + numberOfSearchedOrganizations);
    }

    protected void recursiveBuilder(int depth) {
      int hasToSaveCounter = 0;
      if (depth == 11) { // OUT IN2 : end of recursivity
        for (links[11]=loopInit[11]; links[11]<9; links[11]++) { // OUT IN2
          numberOfErrors = createLink(11);
          waitForKeyPress();
          if (numberOfErrors ==0) {
            numberOfBuiltOrganizations ++;
            //System.out.println("One organization built : " + numberOfBuildedOrganizations);
            //if (false) {//(numberOfBuildedOrganizations % 10000 == 0) {
            if (allAgentsInCooperativeState()) {
              numberOfCooperativeOrganizations++;
              //System.out.println("Saving...");
              organizationSaver.saveOrganization(links);
              hasToSaveCounter++;
              // Option : to be able to work while the simulation is running...
              /*
              synchronized(this) {
              try {
                this.wait(60);
              } catch(InterruptedException e) {;}
              }
               */
              if (hasToSaveCounter > 10000) {
                saveLoopState();
                hasToSaveCounter = 0;
              }
              if (pauseOnCooperationSet)
                waitForEvent();
              }
            destroyLink(11);
          }
          numberOfSearchedOrganizations++;
          if (pauseSet)
              waitForEvent();
        }
        loopInit[11] = 0;
      }
      else { // not end case, launch recursivity
        for (links[depth]=loopInit[depth]; links[depth]<9; links[depth]++) {
          if (depth < 7)
            System.out.println(indent(depth) + "Loop " + depth + "_" + links[depth] + " : " + numberOfSearchedOrganizations);
          numberOfErrors = createLink(depth);
          waitForKeyPress();
          if (numberOfErrors == 0) {
            recursiveBuilder(depth + 1);
            destroyLink(depth);
          }
          else numberOfSearchedOrganizations = numberOfSearchedOrganizations 
                                               + java.lang.Math.pow(9, 11-depth); 
          if (pauseSet)
              waitForEvent();
        }
        loopInit[depth] = 0;
      } 
    }  
    
    protected void waitForEvent() {
      synchronized(this) {
      try {
        System.out.println("Waiting");
        this.wait();
      } catch(InterruptedException e) {}
    }
    }
    
    protected synchronized void notifyThread() {
     notifyAll(); 
    }
    
  }
  // End of inner class
    
  protected int createLink(int i) {
    String returnMessage = "ERROR";
    // test if provider is the agent itself => blocking situation
    if (agentsNeedingIN.getAgentAt(i).equals(possibleOUTs.getAgentAt(links[i]))) {
      return 1;
    }
    // test if the providers are the same agent => not acceptable as programming rules
    else if (agentsNeedingIN.getAgentAt(i)
               .getAcquaintancesManager()
               .isAnInOutAcquaintance(possibleOUTs.getAgentAt(links[i]), "IN")){
      return 1;
    }
    else {
      // connecting to IN
      returnMessage = agentsNeedingIN.getAgentAt(i)
                        .addAcquaintanceWithReturn(possibleOUTs.getAgentAt(links[i]), 
                                                   "IN", 
                                                   agentsNeedingIN.getAgentPositionAt(i),
                                                   1);
      //System.out.println("ReturnMessage : " + returnMessage);
      if (! returnMessage.equals("SUCCESS"))
        return 1;
      else {
        // connecting to OUT
        returnMessage = possibleOUTs.getAgentAt(links[i])
                          .addAcquaintanceWithReturn(agentsNeedingIN.getAgentAt(i), 
                                                     "OUT", 
                                                     1,
                                                     agentsNeedingIN.getAgentPositionAt(i));
        //System.out.println("ReturnMessage : " + returnMessage);
        if (! returnMessage.equals("SUCCESS"))
          return 1;
        else return 0;
      }
      //System.out.println("One link created");
    }
  }
  
  protected int createLinks() {
    int numberOfErrors = 0;
    
    int i=0;
    while ((i<12) && (numberOfErrors==0)) { // build all the links
      numberOfErrors = numberOfErrors + createLink(i);
      i++;
    }
    return numberOfErrors;
  }

  protected void destroyLink(int i) {
    Agent producerAgent = (Agent)possibleOUTs.getAgentAt(links[i]);
    Agent receiverAgent = (Agent)agentsNeedingIN.getAgentAt(i);
    /*System.out.println("Trying to destroy link : " 
                        + producerAgent.getAgentName() 
                        + "--->" 
                        + receiverAgent.getAgentName());*/
    // receiver's acq deleting
    Acquaintance acqProd = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("IN", 
                                             agentsNeedingIN.getAgentPositionAt(i));
    receiverAgent.getAcquaintancesManager()
                 .cancelCooperation(acqProd);
    Acquaintance acqProdNeutral = receiverAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance("NEUTRAL", 0);
    receiverAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqProdNeutral);
    // producer's acq deleting
    Acquaintance acqRec = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "OUT");
    producerAgent.getAcquaintancesManager()
                 .cancelCooperation(acqRec);
    Acquaintance acqRecNeutral = producerAgent
                            .getAcquaintancesManager()
                            .getAcquaintances()
                            .getAcquaintance(receiverAgent, "NEUTRAL");
    producerAgent.getAcquaintancesManager()
                 .removeAcquaintance(acqRecNeutral);
  }
  
  protected void clearAllAcquaintances() {
    for (int i = 0; i<11; i++)
       ((Agent)agentPool.getAgentAt(i)).getAcquaintancesManager().removeAllAcquaintances();
  }
  
  protected void waitForKeyPress() {
    /*
    try {
      System.out.println("Press a key");
      int lock1 = System.in.read();
      int lock2 = System.in.read();
    } catch (java.io.IOException e) {System.out.println(e);}; */
  }
  
  protected void strongWaitForKeyPress() {
    try {
      System.out.println("Press a key");
      System.in.read();
      System.in.read();
    } catch (java.io.IOException e) {System.out.println(e);};  
  }
   
  protected void initOrganization() {
    organizationRunner.initTest();
  }
  
  public void showLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(true);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void hideLinks() {
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().setShowLinksEnabled(false);
   organizationRunner.getProgrammingEnvironment().getEnvironmentBackground().repaint();
  }
  
  public void showAgentOutput() {
    for (int i = 0; i<11; i++)
       ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(true);
  }
  
  public void removeAgentOutput() {
    for (int i = 0; i<11; i++)
      ((Agent)agentPool.getAgentAt(i)).setAgentTextOutputEnabled(false);
  }

  public void toggleAgentGUI() {
    if (isAgentGUIShown) {
      hideLinks();
      removeAgentOutput();
      isAgentGUIShown = false;
    }
    else {
      showLinks();
      showAgentOutput();
      isAgentGUIShown = true;
    }
  }
  
  public void printInfo() {
    System.out.println("Number of searched organizations : " + numberOfSearchedOrganizations);
    System.out.println("Number of built organizations : " + numberOfBuiltOrganizations);
    System.out.println("Number of cooperative organizations  : " + numberOfCooperativeOrganizations );
  }
  
  protected boolean allAgentsInCooperativeState() {
    for (int i = 0; i<11; i++) {
      Agent agent = ((Agent)agentPool.getAgentAt(i));
      agent.getReorganizationManager().checkCooperativeState();
      if (!agent.inCooperativeState()) {
        //System.out.println(agent.getAgentName() + " NOT in cooperative state");
        return false;
      }
      //System.out.println(agent.getAgentName() + " in cooperative state");
    }
    return true;
  }
  
  public void pause() {
    pauseSet = true;
  }
  
  public void pauseOnCooperation() {
    if (pauseOnCooperationSet)
      pauseOnCooperationSet = false;
    else
      pauseOnCooperationSet = true;
  }

  public synchronized void unpause() {
    pauseSet = false;
    builder.notifyThread();
  }
  
  public synchronized void oneStep() {
    pauseSet = false;
    builder.notifyThread();
    pause();
  }
  
  public synchronized void startBuilding() {
    builder = new Builder();
    builderRunner = new java.lang.Thread(builder);
    pauseSet = false;
    builderRunner.start();
  }
  
  public synchronized void saveLoopState() {
    for (int i = 0; i < 12; i++)
      loopInit[i] = links[i];
    loopStateSaver.saveLoopState(loopInit, 
                                 numberOfBuiltOrganizations, 
                                 numberOfSearchedOrganizations, 
                                 numberOfCooperativeOrganizations,
                                 organizationSaver.getSaveOrganizationCounter(),
                                 organizationSaver.getLoadOrganizationCounter());
  }
  
  public synchronized void loadPreviousLoopState() {
    loopStateSaver.loadLoopState();
    loopInit = loopStateSaver.getLoopInit();
    numberOfBuiltOrganizations = loopStateSaver.getNumberOfBuiltOrganizations();
    numberOfSearchedOrganizations = loopStateSaver.getNumberOfSearchedOrganizations();
    numberOfCooperativeOrganizations = loopStateSaver.getNumberOfCooperativeOrganizations();
    organizationSaver.setSaveOrganizationCounter(loopStateSaver.getSaveOrganizationCounter());
    organizationSaver.setLoadOrganizationCounter(loopStateSaver.getLoadOrganizationCounter());
    builder = new Builder();
    builderRunner = new java.lang.Thread(builder);
    pauseSet = true;
    builderRunner.start();
    //manualStateSetter();
  }
  
  protected void manualStateSetter() {
    loopStateSaver.loadLoopState();
    //loopInit = loopStateSaver.getLoopInit();
    numberOfBuiltOrganizations = loopStateSaver.getNumberOfBuiltOrganizations();
    numberOfSearchedOrganizations = loopStateSaver.getNumberOfSearchedOrganizations();
    numberOfCooperativeOrganizations = loopStateSaver.getNumberOfCooperativeOrganizations();
    organizationSaver.setSaveOrganizationCounter(746785);
    organizationSaver.setLoadOrganizationCounter(0);
    builder = new Builder();
    builderRunner = new java.lang.Thread(builder);
    pauseSet = true;
    builderRunner.start();
  }
  
  protected String indent(int depth) {
    switch(depth) {
      case 0 : return "";
      case 1 : return " ";
      case 2 : return "  ";
      case 3 : return "    ";
      case 4 : return "     ";
      case 5 : return "      ";
      case 6 : return "       ";
      case 7 : return "        ";
      case 8 : return "         ";
      case 9 : return "          ";
      case 10 : return "           ";
      case 11 : return "            ";
      default : return "";
    }
  }
  
  /*
  public synchronized void unpause() {
    builder.unpause();
  }
  
  public synchronized void start() {
    builder.unpause();
  }
  */
  
  /*
  protected void buildAnOrganization_notRecursive() {
    double numberOfBuildedOrganizations = 0;
    double numberOfSearchedOrganizations = 0;
    double numberOfCooperativeOrganizations = 0;
    agentsNeedingIN = new AgentsNeedingIN(agentPool);
    possibleOUTs = new PossibleOUTs(agentPool);
    links = new int[12];
    int numberOfErrors;
    
    for (links[0]=0; links[0]<9; links[0]++) { // + IN1
      System.out.println("Loop 1_" + links[0] + " : " + numberOfSearchedOrganizations);
      numberOfErrors = createLink(0);
      waitForKeyPress();
      if (numberOfErrors == 0) {
        for (links[1]=0; links[1]<9; links[1]++) { // + IN2
          System.out.println("Loop 2_" + links[1] + " : " + numberOfSearchedOrganizations);
          numberOfErrors = createLink(1);
          waitForKeyPress();
          if (numberOfErrors == 0) {
            for (links[2]=0; links[2]<9; links[2]++) { // * IN1
              System.out.println("Loop 3_" + links[2] + " : " + numberOfSearchedOrganizations);
              numberOfErrors = createLink(2);
              waitForKeyPress();
              if (numberOfErrors == 0) {
                for (links[3]=0; links[3]<9; links[3]++) { // * IN2
                  System.out.println("Loop 4_" + links[3] + " : " + numberOfSearchedOrganizations);
                  numberOfErrors = createLink(3);
                  waitForKeyPress();
                  if (numberOfErrors == 0) {
                    for (links[4]=0; links[4]<9; links[4]++) { // = IN1
                      System.out.println("Loop 5_" + links[4] + " : " + numberOfSearchedOrganizations);
                      numberOfErrors = createLink(4);
                      waitForKeyPress();
                      if (numberOfErrors == 0) {
                        for (links[5]=0; links[5]<9; links[5]++) { // = IN2
                          System.out.println("Loop 6_" + links[5] + " : " + numberOfSearchedOrganizations);
                          numberOfErrors = createLink(5);
                          waitForKeyPress();
                          if (numberOfErrors == 0) {
                            for (links[6]=0; links[6]<9; links[6]++) { // F1 IN1
                              System.out.println("Loop 7_" + links[6] + " : " + numberOfSearchedOrganizations);
                              numberOfErrors = createLink(6);
                              waitForKeyPress();
                              if (numberOfErrors == 0) {
                                for (links[7]=0; links[7]<9; links[7]++) { // F1 IN2
                                  //System.out.println("Loop 8 " + numberOfSearchedOrganizations);
                                  numberOfErrors = createLink(7);
                                  waitForKeyPress();
                                  if (numberOfErrors == 0) {
                                    for (links[8]=0; links[8]<9; links[8]++) { // F2 IN1
                                      //System.out.println("Loop 9 " + numberOfSearchedOrganizations);
                                      numberOfErrors = createLink(8);
                                      waitForKeyPress();
                                      if (numberOfErrors == 0) {
                                        for (links[9]=0; links[9]<9; links[9]++) { // F2 IN2
                                          //System.out.println("Loop 10 " + numberOfSearchedOrganizations);
                                          numberOfErrors = createLink(9);
                                          waitForKeyPress();
                                          if (numberOfErrors == 0) {
                                            for (links[10]=0; links[10]<9; links[10]++) { // OUT IN1
                                              //System.out.println("Loop 11 " + numberOfSearchedOrganizations);
                                              numberOfErrors = createLink(10);
                                              waitForKeyPress();
                                              if (numberOfErrors == 0) {
                                                for (links[11]=0; links[11]<9; links[11]++) { // OUT IN2
                                                  //System.out.println("Loop 12 " + numberOfSearchedOrganizations);
                                                  numberOfErrors = createLink(11);
                                                  waitForKeyPress();
                                                  if (numberOfErrors ==0) {
                                                    numberOfBuildedOrganizations ++;
                                                    //System.out.println("One organization built : " + numberOfBuildedOrganizations);
                                                    //if (false) {//(numberOfBuildedOrganizations % 10000 == 0) {
                                                    if (allAgentsInCooperativeState()) {
                                                      //showLinks();
                                                      //showAgentOutput();
                                                      numberOfCooperativeOrganizations++;
                                                      System.out.println("Organization built : " + numberOfBuildedOrganizations);
                                                      System.out.println("Cooperative organization !! : " + numberOfCooperativeOrganizations );
                                                      if (pauseSet)
                                                        strongWaitForKeyPress();
                                                      //hideLinks();
                                                      //removeAgentOutput();
                                                    }
                                                    destroyLink(11);
                                                  } //end if 12
                                                  numberOfSearchedOrganizations++;
                                                } //end loop 12
                                                destroyLink(10);
                                              } // end if 11
                                              else numberOfSearchedOrganizations=numberOfSearchedOrganizations+9; 
                                            } // end loop 11
                                            destroyLink(9);
                                          } // end if 10
                                          else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,2); 
                                        } // end loop 10
                                        destroyLink(8);
                                      } // end if 9
                                      else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,3); 
                                    } // end loop 9
                                    destroyLink(7);
                                  } // end if 8
                                  else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,4); 
                                } // end loop 8
                                destroyLink(6);
                              } // end if 7
                              else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,5); 
                            } // end loop 7
                            destroyLink(5);
                          } // end if 6
                          else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,6); 
                        } // end loop 6
                        destroyLink(4);
                      } // end if 5
                      else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,7); 
                    } // end loop 5
                    destroyLink(3);
                  } // end if 4
                  else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,8); 
                } // end loop 4
                destroyLink(2);
              } // enf if 3
              else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,9); 
            } // end loop 3
            destroyLink(1);
          } // end if 2
          else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,10); 
        } // end loop 2
        destroyLink(0);
      } // end if 1
      else numberOfSearchedOrganizations=numberOfSearchedOrganizations+java.lang.Math.pow(9,11); 
    } // end loop 1
    System.out.println("All organizations built : " + numberOfBuildedOrganizations);
    System.out.println("All organizations built : " + numberOfSearchedOrganizations);
      
  }*/
  
  
}




