package epe.calculus;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import epe.gui.ProgrammingEnvironment;
import epe.gui.SystemPrinter;
import epe.agents.acquaintances.TrustValue;
import epe.agents.specificAgents.AgentAddition;
import epe.agents.specificAgents.AgentConstant;
import epe.agents.specificAgents.AgentMultiplication;
import epe.agents.specificAgents.environment.AgentEnvironmentReceiver;
import epe.agents.specificAgents.interfaceAgents.AgentOutWithoutControl;
import epe.observationTools.messageSendObserver.MessageSendObserver;
import epe.observationTools.ncsObserver.NcsObserver;


public class ThreadsTest extends Test implements ActionListener, Runnable{
  
    protected Thread executingThread;
    protected boolean stopAllAgentsOrderSend = false;
    protected boolean startWorkingOrderSend = false;
    protected final static int ONE_SECOND = 1000;
    protected int delay = ONE_SECOND/2;

    protected AgentEnvironmentReceiver agentEnvironment;
    protected AgentConstant agentConstantA, agentConstantB, agentConstantC;
    protected AgentOutWithoutControl agentOut;
    protected AgentAddition[] agentAddition;
    protected AgentMultiplication agentMultiplication;
    
    protected ProgrammingEnvironment programmingEnvironment;
    protected NcsObserver ncsObserver;
    protected MessageSendObserver messageSendObserver;
    
    protected int threadNumber = 5000;

  public ThreadsTest() {
    executingThread = new Thread(this);
    executingThread.start();
    programmingEnvironment = new ProgrammingEnvironment();
    ncsObserver = new NcsObserver();
    messageSendObserver = new MessageSendObserver();
    programmingEnvironment.setVisible(true);
    programmingEnvironment.getCommandPanel().initTest(this);
    initTest();
  }
  
  public void initTest() {
    agentEnvironment = new AgentEnvironmentReceiver("", programmingEnvironment.getCommandPanel());
    agentEnvironment.setProgrammingEnvironment(programmingEnvironment);
    agentEnvironment.addNcsListener(ncsObserver);
    agentEnvironment.addMessageSendListener(messageSendObserver);
          
    agentConstantA = new AgentConstant("A", 2);
    agentConstantA.setProgrammingEnvironment(programmingEnvironment);
    agentConstantA.addNcsListener(ncsObserver);
    agentConstantA.addMessageSendListener(messageSendObserver);

    agentConstantB = new AgentConstant("B", 10);
    agentConstantB.setProgrammingEnvironment(programmingEnvironment);
    agentConstantB.addNcsListener(ncsObserver);
    agentConstantB.addMessageSendListener(messageSendObserver);
    
    agentConstantC = new AgentConstant("C", 100);
    agentConstantC.setProgrammingEnvironment(programmingEnvironment);
    agentConstantC.addNcsListener(ncsObserver);
    agentConstantC.addMessageSendListener(messageSendObserver);
        
    agentAddition = new AgentAddition[threadNumber];
    for (int i = 0; i<threadNumber; i++) {
    	agentAddition[i] = new AgentAddition(new Integer(i).toString());
	    agentAddition[i].setProgrammingEnvironment(programmingEnvironment);
	    agentAddition[i].addNcsListener(ncsObserver);
	    agentAddition[i].addMessageSendListener(messageSendObserver);
    }

    agentMultiplication = new AgentMultiplication("");
    agentMultiplication.setProgrammingEnvironment(programmingEnvironment);
    agentMultiplication.addNcsListener(ncsObserver);
    agentMultiplication.addMessageSendListener(messageSendObserver);
    
    agentOut = new AgentOutWithoutControl("", programmingEnvironment.getCommandPanel());
    agentOut.setProgrammingEnvironment(programmingEnvironment);
    agentOut.addNcsListener(ncsObserver);
    agentOut.addMessageSendListener(messageSendObserver);
    
    
    agentConstantA.setLocation(0,0);
    agentConstantB.setLocation(0,1);
    agentConstantC.setLocation(0,2);
    for (int i = 0; i<threadNumber; i++) {
    	int randomShiftX = (int)((java.lang.Math.random()-0.5)*1000);
    	int randomShiftY = (int)((java.lang.Math.random()-0.5)*1000);
    	agentAddition[i].setLocation(2,2,randomShiftX, randomShiftY);
    	}
    agentMultiplication.setLocation(2,1);
    agentOut.setLocation(4,4);
    agentEnvironment.setLocation(3,8);

    agentEnvironment.start();
    agentConstantA.start();
    agentConstantB.start();
    agentConstantC.start();
    for (int i = 0; i<threadNumber; i++)
    	agentAddition[i].start();
    agentMultiplication.start();
    agentOut.start();

    // Out, Environment
    agentOut.addAcquaintance(agentEnvironment, "OUT", 1, TrustValue.FIXEDTRUSTVALUE);
    agentEnvironment.addAcquaintance(agentOut, "IN", 1, TrustValue.FIXEDTRUSTVALUE);
  
    // All Agents <--> AgentOut  (NEUTRAL)
    agentConstantA.addAcquaintance(agentOut, "NEUTRAL", 0);
    agentConstantB.addAcquaintance(agentOut, "NEUTRAL", 0);
    agentConstantC.addAcquaintance(agentOut, "NEUTRAL", 0);
    for (int i = 0; i<threadNumber; i++)
    	agentAddition[i].addAcquaintance(agentOut, "NEUTRAL", 0);
    agentMultiplication.addAcquaintance(agentOut, "NEUTRAL", 0);
    
    agentOut.addAcquaintance(agentConstantA, "NEUTRAL", 0);
    agentOut.addAcquaintance(agentConstantB, "NEUTRAL", 0);
    agentOut.addAcquaintance(agentConstantC, "NEUTRAL", 0);
    for (int i = 0; i<threadNumber; i++)
    	agentOut.addAcquaintance(agentAddition[i], "NEUTRAL", 0);
    agentOut.addAcquaintance(agentMultiplication, "NEUTRAL", 0);
   
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentConstantA);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentConstantB);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentConstantC);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentOut);
    for (int i = 0; i<threadNumber; i++)
    	programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentAddition[i]);
    programmingEnvironment.getEnvironmentBackground().getLinks().updateLink(agentMultiplication);
  
  }
 
  protected synchronized void pause(int delay) {
    SystemPrinter.printSystem("Waiting a little.");
      try {
        wait(delay);
      } catch (InterruptedException e) {
      }
  }

  public synchronized void actionPerformed(ActionEvent ev) {
    if (ev.getActionCommand().equals("Stop all agents")) {
      SystemPrinter.printSystem("Received event : stop all agents.");
      stopAllAgentsOrderSend = true;
      notifyAll();
    }
    else if (ev.getActionCommand().equals("Start Working")) {
      SystemPrinter.printSystem("Received event : start working.");
      startWorkingOrderSend = true;
      notifyAll();
    }
    else if (ev.getActionCommand().equals("Feedback")) {
      SystemPrinter.printSystem("Received event : Feedback.");
      agentEnvironment.feedback();
    }
    else if (ev.getActionCommand().equals("Clear Logs & Feedback")) {
      SystemPrinter.printSystem("Received event : Clear logs & Feedback.");
      clearAgentLogs();
      agentEnvironment.feedback();
    }
    else SystemPrinter.printSystem("ERROR : Test : unknown event : " + ev.getActionCommand());
  }

  public void clearAgentLogs() {
  	agentConstantA.getGUI().clearLog();
    agentConstantB.getGUI().clearLog();
    agentConstantC.getGUI().clearLog();
    for (int i = 0; i<threadNumber; i++)
    	agentAddition[i].getGUI().clearLog();
    agentMultiplication.getGUI().clearLog();
    agentOut.getGUI().clearLog();
    agentEnvironment.getGUI().clearLog();
  }
  
  public void run() {
    while (true) {
      if (stopAllAgentsOrderSend) {
        agentEnvironment.disableAgent();
        agentConstantA.disableAgent();
        agentConstantB.disableAgent();
        agentConstantC.disableAgent();
        for (int i = 0; i<threadNumber; i++)
        	agentAddition[i].disableAgent();
        agentMultiplication.disableAgent();
        agentOut.disableAgent();
        
        stopAllAgentsOrderSend = false;
      }
      if (startWorkingOrderSend) {
        agentEnvironment.disableAgent();
        agentConstantA.disableAgent();
        agentConstantB.disableAgent();
        agentConstantC.disableAgent();
        for (int i = 0; i<threadNumber; i++)
        	agentAddition[i].disableAgent();
        agentMultiplication.disableAgent();
        agentOut.disableAgent();

        agentEnvironment.reInitializeOrder();
        agentConstantA.reInitializeOrder();
        agentConstantB.reInitializeOrder();
        agentConstantC.reInitializeOrder();
        for (int i = 0; i<threadNumber; i++)
        	agentAddition[i].reInitializeOrder();
        agentMultiplication.reInitializeOrder();
        agentOut.reInitializeOrder();

        pause(delay);
        
        agentEnvironment.enableAgent();
        agentConstantA.enableAgent();
        agentConstantB.enableAgent();
        agentConstantC.enableAgent();
        for (int i = 0; i<threadNumber; i++)
        	agentAddition[i].enableAgent();
        agentMultiplication.enableAgent();
        agentOut.enableAgent();

        agentEnvironment.startAgent();
        agentConstantA.startAgent();
        agentConstantB.startAgent();
        agentConstantC.startAgent();
        for (int i = 0; i<threadNumber; i++)
        	agentAddition[i].startAgent();
        agentMultiplication.startAgent();
        agentOut.startAgent();
        
        startWorkingOrderSend = false;
      }
      
      synchronized(this) {
          try {
            SystemPrinter.printSystem("Waiting for an event.");
              this.wait();
          } catch(InterruptedException e) {}
      }
    }
  }
  
}

