package epe.communication;

	import java.util.Vector;
	
	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.acquaintances.AcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;


public class MessageManager extends Object {
  
	  protected Acquaintances acquaintances;
	  protected  Vector<Object> adressees;
	  protected Agent owner;
	  protected GUI gui;

  public MessageManager(Agent owner) {
    this.owner = owner;
    gui = owner.getGUI();
    gui.println("MessageManager created");
  }
  
  public void forwardMessage(Message message, 
                             							AgentInterface agentInterface,
                             							Agent sender) {
    
    AcquaintancesManager acquaintancesManager = owner.getAcquaintancesManager();
    acquaintances =  acquaintancesManager.getAcquaintances();
    adressees = acquaintances.findAcquaintance(agentInterface);
    int rang;
    
    if (adressees.size() == 0) {
      SystemPrinter.printSystem("ERROR : " + owner.getAgentName());
      owner.errorHappened();
      gui.println("MessageManager : ERROR : no known adressee for "
                         + "the message [" + message.getId()
                         + "] from : [" + sender.getAgentName() + "]"
                         + " to interface [" + agentInterface.getType() 
                         + agentInterface.getPosition() 
                         +"].", java.awt.Color.red);
      owner.getAcquaintancesManager().showAllAcquaintances();
    }
    else {
    	String s;
	    s = "MessageManager : " + adressees.size() 
	        + " known adressee(s) for the message [ "
	        + message.getId()
	        + " ] from (this / sender / content sender) [ " 
	        + owner.getAgentName()
	        + " / "
	        + sender.getAgentName()
	        + " / "
	        + message.getSender().getAgentName()
	        + " ] : "; 
	    for (int i=0; i < adressees.size(); i++ ) {
	      rang = ((Integer)adressees.elementAt(i)).intValue();
	      s = s + (acquaintances.getAcquaintance(rang).getAgent().getAgentName() + " ");
	    }
	    s = s + ".";
	    gui.println(s);
	      
      for (int i=0; i < adressees.size(); i++ ) {
        rang = ((Integer)adressees.elementAt(i)).intValue();
        acquaintances.getAcquaintance(rang).getAgent().receiveMessage(message.copyMessage());
  
        Agent adressee = acquaintances.getAcquaintance(rang).getAgent();
        if (message.getSender().equals(adressee)) {
          gui.println(owner.getAgentName() + " DEBUG : ERROR : sender["
                + message.getSender()
                + "/" + message.getSender().getAgentName()
                + "] = adressee["
                + adressee 
                + "/" + adressee.getAgentName()
                + "]", java.awt.Color.red);
          SystemPrinter.printSystem(owner.getAgentName() + " : DEBUG : ERROR : sender = adressee");
          owner.errorHappened();
        }
		    if (((message.getId().equals(Message.NUMBER_MESSAGE))
		        || (message.getId().equals(Message.BOOLEAN_MESSAGE)))
		      & (! acquaintancesManager.isAnInOutAcquaintance(adressee, "OUT"))) {
			    gui.println(owner.getAgentName() + " : DEBUG : ERROR : sending DataMessage to acq not OUT", java.awt.Color.red);
			    SystemPrinter.printSystem(owner.getAgentName() + " : DEBUG : ERROR : sending DataMessage to acq not OUT");
			    owner.errorHappened();
		    }
      }
	  gui.println("MessageManager : all messages have been sent.");
    }
  }

  
  public void forwardPersonalMessage(Message message, 
                                     							Agent adressee,
                                     							String type, //IN, OUT, NEUTRAL
                                     							Agent sender) {
    gui.println("MessageManager : sending personnal message [" 
                + message.getId()
                + " ] from (this / sender / content sender) [ " 
                + owner.getAgentName()
                + " / "
                + sender.getAgentName()
                + " / "
                + message.getSender().getAgentName()
                + " ] to : " 
                + adressee.getAgentName() + ".");
    adressee.receiveMessage(message.copyMessage());
  
      AcquaintancesManager acquaintancesManager = owner.getAcquaintancesManager();
      if (((message.getId().equals(Message.NUMBER_MESSAGE))
            || (message.getId().equals(Message.BOOLEAN_MESSAGE)))
          & (! acquaintancesManager.isAnInOutAcquaintance(adressee, "OUT"))) {
            gui.println(owner.getAgentName() + " : DEBUG : ERROR " + 
            					  ": sending DataMessage to acq not OUT", java.awt.Color.red);
            SystemPrinter.printSystem(owner.getAgentName() + " : DEBUG : ERROR " + 
            													": sending DataMessage to acq not OUT");
            owner.errorHappened();
        }
  }
  
  public void sendMessageToAllAcquaintances(Message message) {
    Acquaintances acquaintances = owner.getAcquaintancesManager().getAcquaintances();
    Agent sender = message.getSender();;
    Vector<Object> adressees = acquaintances.getAllNeutralAcquaintances();
    if (adressees.size()==0)
      gui.println("WARNING : " + owner.getAgentName() + " has no Acquaintances.");
    else {
      for (int i=0; i<adressees.size(); i++) {
        int pos = ((Integer)adressees.elementAt(i)).intValue();
        Agent adressee = acquaintances.getAcquaintance(pos).getAgent();
        String type = acquaintances.getAcquaintance(pos).getInterfaceType();
        forwardPersonalMessage(message, adressee, type, sender);
      }
    }
  }
  
  public void sendMessageToAllAcquaintancesButSender(Message message) {
    Acquaintances acquaintances = owner.getAcquaintancesManager().getAcquaintances();
    Agent sender = message.getSender();;
    Vector<Object> adressees = acquaintances.getAllNeutralAcquaintances();
    if (adressees.size()==0)
      gui.println("WARNING : " + owner.getAgentName() + " has no Acquaintances.");
    else {
      for (int i=0; i<adressees.size(); i++) {
        int pos = ((Integer)adressees.elementAt(i)).intValue();
        Agent adressee = acquaintances.getAcquaintance(pos).getAgent();
        String type = acquaintances.getAcquaintance(pos).getInterfaceType();
        if (!(adressee.equals(sender)))
          forwardPersonalMessage(message, adressee, type, sender);
      }
    }
  }
  
  public void receivedMessage(Message message) {
    if (owner.isVerbose()) {
      String typeId = message.getId();
      Agent sender = message.getSender();
      gui.println(owner.getAgentName() + " : received message : " + typeId 
                  + " from [" + sender.getAgentName() + "]", java.awt.Color.lightGray);
    }
  }
  
}