package epe.communication.messages.ncs;

import epe.communication.Message;
import epe.agents.Agent;
import epe.agents.agentInterface.AgentInterface;

public class NCSInformedNeedInMessage extends Message {
  
	  protected int effectValue;
	  protected int initialEffectValue = 40;
	  protected int decreaseValue = 20;
	  protected AgentInterface interfaceType;
	  protected int requiredThreshold;
	  protected String information; // "HIGHER" or "LOWER"
	  // phase : Each time the system gets a new feedback from the environment, 
	  // a new phase is initiated (phase counter is incremented).
	  protected int phase; 

  public NCSInformedNeedInMessage(Agent sender,
										 							String information,
										 							String dataType, 
                                  int position,
                                  int requiredThreshold,
								  								int phase) {
    this.id = Message.NCS_INFORMED_NEED_IN_MESSAGE;                 
    effectValue = initialEffectValue;
    this.sender = sender;
    this.information = information;
    interfaceType = new AgentInterface("IN", position, dataType);
    this.requiredThreshold = requiredThreshold;
    this.phase = phase;
  }

  public void decreaseEffectValue() {
    effectValue = effectValue - decreaseValue;
  }

  public boolean isEffective() {
    return (effectValue > 0);
  }

  public String getInformation() {
	  return information;
  }  
  
  public String getDataType() {
    return interfaceType.getDataType();
  }

  public AgentInterface getInterfaceType() {
    return interfaceType;
  }
   
	public void setInterfaceType(AgentInterface interfaceType) {
		this.interfaceType = interfaceType;
	}
	
  public int getRequiredThreshold() {
    return requiredThreshold;
  }
  
  public int getEffectValue() {
	  return effectValue;
	}
  
  public int getPhase() {
  	return phase;
  }
 
  public Message copyMessage() {
  	NCSInformedNeedInMessage returnMessage;
  	try {
  		returnMessage =  (NCSInformedNeedInMessage)((Object)this.clone());
  		returnMessage.setInterfaceType(this.getInterfaceType().copyAgentInterface());
  		return returnMessage;
  	} catch (java.lang.CloneNotSupportedException e) {
  		return null;
  	}
  }

}