package epe.gui;

	import epe.gui.animator.Animator;
	import epe.gui.soundPlayers.*;
	import epe.gui.textOutput.AgentTextOutputPanel;
	import epe.agents.Agent;
	
	import java.awt.Color;

public class GUI extends Object {

  protected Agent owner;
  protected Animator animator;
  protected AgentTextOutputPanel textOutput;
  protected ErrorSoundPlayer errorSoundPlayer;
  protected TimerSoundPlayer timerSoundPlayer;
  protected AdequacySoundPlayer adequacySoundPlayer;
  
  // Colors
  public static Color red = Color.red;
  public static Color black = Color.black;
  public static Color blue = Color.blue;
  public static Color cyna = Color.cyan;
  public static Color darkGray = Color.darkGray;
  public static Color gray = Color.gray;
  public static Color green = Color.green;
  public static Color lightGray = Color.lightGray;
  public static Color magenta = Color.magenta;
  public static Color orange = Color.orange;
  public static Color pink = Color.pink;
  public static Color yellow = Color.yellow;
  
  
  private java.util.Locale currentLocale = new java.util.Locale("fr", "FR");


  public GUI(Agent owner) {
    this.owner = owner;
    textOutput = new AgentTextOutputPanel(owner);
    animator = new Animator(owner);
    errorSoundPlayer = new ErrorSoundPlayer();
    timerSoundPlayer = new TimerSoundPlayer();
    adequacySoundPlayer = new AdequacySoundPlayer();
  }
  
  public void reInitialize() {
    textOutput = new AgentTextOutputPanel(owner);
  }
  
  public void showAgentGUIinEnvironment() {
    animator.showAnimatorInEnvironment();
    animator.initAnimator();
  }
  
  public void showAgentTextOutputPanel() {
    textOutput.showAgentTextOutputPanel();
  }
  
  public void print(String text) {
    if (owner.isAgentTextOutputEnabled())
      textOutput.print(text);
  }

  public void println(String text) {
    if (owner.isAgentTextOutputEnabled())
      textOutput.println(text);
  }
  
  public void print(String text, java.awt.Color color) {
    if (owner.isAgentTextOutputEnabled())
      textOutput.print(text, color);
  }

  public void println(String text, java.awt.Color color) {
    if (owner.isAgentTextOutputEnabled())
      textOutput.println(text, color);
  }
  
  public void gotoNextLine() {
    if (owner.isAgentTextOutputEnabled())
      textOutput.gotoNextLine();
  }
  
  public void printDate() {
    if (owner.isAgentTextOutputEnabled())
      textOutput.printDate();
  }
  
  public synchronized void start() {
    animator.startAnimation();
  }

  public synchronized void stop() {
    animator.stopAnimation();
  }
  
  public void setLocation(int xLocation, int yLocation, int shiftX, int shiftY) {
    animator.setLocation(xLocation, yLocation, shiftX, shiftY);
  }
  
  public int getLocationX() {
    return animator.getLocationX();
  }
  
  public int getLocationY() {
    return animator.getLocationY();
  }
  
  public int getSizeX() {
    return animator.getSizeX();
  }
  
  public int getSizeY() {
    return animator.getSizeY();
  }
  
  public Animator getAnimator() {
   return animator; 
  }
  
  public void show() {
   animator.getFrame().show();
  }
  
  public void errorNotification() {
    errorSoundPlayer.playErrorSound();
  }
  
  public void timerNotification() {
    timerSoundPlayer.playTimerSound();
  }
  
  public void adequacyNotification() {
    adequacySoundPlayer.playAdequacySound();
  }
  
  public String getTime() {
    java.util.Date today;
    String outTime;
    java.text.DateFormat timeFormatter;
    timeFormatter = java.text.DateFormat.getTimeInstance(java.text.DateFormat.DEFAULT,
                                                         currentLocale);
    today = new java.util.Date();
    outTime = timeFormatter.format(today);
    return outTime;
  }

  public void clearLog() {
  	textOutput.clearLog();
  }
  
}