package epe.gui.animator;


import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentListener;
import java.net.URL;

import javax.swing.JApplet;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

import epe.gui.GUI;
import epe.gui.SystemPrinter;
import epe.agents.Agent;


public class Animator extends JApplet implements ActionListener, ComponentListener {

  private static final long serialVersionUID = 1L;
	protected Agent owner;
  protected GUI gui;
  protected JInternalFrame f;
  protected ImageSQPanel imageSQPanel;
  protected int frameNumber = -1;
  protected int initialDelay = 500;
  protected int initialStoppingDelay = 5; // number of frames drawn after stop is called
  protected int stoppingDelay = initialStoppingDelay ;
  protected int delay;
  protected boolean frozen = false;
  protected Timer timer;
  protected boolean stopCalled = false;
  protected int locationX = 0, locationY = 0;
  protected int row = 0, column = 0;
  protected int sizeX = 70, sizeY = 70;
//  protected String path = "D:/Programmation/Java/MyPrograms/EmergentProgramming/emergentProgramming/animator/imagesSmall/T";
  protected String relativePath = "data/gui/animator/imagesSmall/t";
  protected String absolutePath;
  protected URL absoluteURL;
  
   //Invoked only when this is run as an application
  public Animator(Agent owner) {
    this.owner = owner;
    gui = owner.getGUI();
   
    Image[] waving = new Image[10];
    //gui.printSystem("Trying to get : " + absolutePath);
    for (int i = 1; i <= 10; i++) {
      //absolutePath = getClass().getResource("/" + relativePath + i + ".gif").getFile();
      absoluteURL = getClass().getResource("/" + relativePath + i + ".gif");
      waving[i-1] =
      Toolkit.getDefaultToolkit().getImage(absoluteURL);
    }
    
    f = new JInternalFrame(owner.getAgentName());
    setForeground(java.awt.Color.black);
    f.setResizable(false);
    f.setBounds(20, 20, sizeX, sizeY);
    f.setFrameIcon(null);
    
    buildUI(f.getContentPane(), waving);
    f.setSize(new Dimension(sizeX, sizeY));
    f.setVisible(true);
    
    f.addComponentListener(this);
    //gui.printSystem("Animator created");
  }
  
  public void showAnimatorInEnvironment() {
    owner.getProgrammingEnvironment().addAgentGUI(f);
  }
  
  public synchronized void initAnimator() {
    startAnimation();
    /*
    try {
          wait(initialDelay);
        } catch (InterruptedException e) { }
     */
    stopAnimation();
  }
  
  public void setFrameName(String name) {
    f.setTitle(name);
  }
  
  //Note: Container must use BorderLayout, which is the
  //default layout manager for content panes.
  void buildUI(Container container, Image[] dukes) {
    int fps = 10;

    //How many milliseconds between frames?
    delay = (fps > 0) ? (1000 / fps) : 100;

    //Set up a timer that calls this object's action handler
    timer = new Timer(delay, this);
    timer.setInitialDelay(0);
    timer.setCoalesce(true);

    imageSQPanel = new ImageSQPanel(dukes);
    container.add(imageSQPanel, BorderLayout.CENTER);

    imageSQPanel.addMouseListener(new AnimatorMouseListener(owner));
  }  

  public void setLocation(int row, int column, int shiftX, int shiftY) {
    this.row = row;
    this.column = column;
    java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
    locationX = ((screenSize.width-sizeX)/5)*row + 10 + shiftX;
    locationY = ((screenSize.height-sizeY)/10)*column + 10 + shiftY;
    f.setLocation(locationX, locationY);
    
    owner.getBackground().getLinks().updateLink(owner);
    //owner.getBackground().repaint();
    
    //f.repaint();
  }
  
  public int getLocationX() {
   return locationX;
  }
  
  public int getLocationY() {
   return locationY;
  }
  
  public int getSizeX() {
   return sizeX;
  }
  
  public int getSizeY() {
   return sizeY;
  }
  
  public JInternalFrame getFrame() {
   return f; 
  }
  
  public synchronized void startAnimation() {
    if (frozen) {
      //Do nothing.  The user has requested that we
      //stop changing the image.
    } else {
      //Start animating!
      if (!timer.isRunning()) {
        stopCalled = false;
        timer.start();
      }
    }
  }

  public synchronized void stopAnimation() {
    //Stop the animating thread.
    if (!frozen) 
      if (timer.isRunning()) {
        stoppingDelay = initialStoppingDelay;
        stopCalled = true;
      }
  }
  
  //Timer event handled
  public void actionPerformed(ActionEvent e) {
    //gui.printSystem("Animator : Running");
    //Advance the animation frame.
    if ((stopCalled)&&(stoppingDelay == 0))
      timer.stop();
    else if (stopCalled) {
      frameNumber++;
      //Display it.
      stoppingDelay--;
      imageSQPanel.repaint();
    }
    else {
      frameNumber++;
      //Display it.
      imageSQPanel.repaint();
    }
  }

  //Window events handled
  public void componentShown(final java.awt.event.ComponentEvent p1) {
    SystemPrinter.printSystem("Visible");
  }
  
  public void componentResized(final java.awt.event.ComponentEvent p1) {
    sizeX = (int)f.getSize().getWidth(); 
    sizeY = (int)f.getSize().getHeight();
    owner.getBackground().getLinks().updateLink(owner);
    owner.getBackground().repaint();
  }
  
  public void componentHidden(final java.awt.event.ComponentEvent p1) {
    SystemPrinter.printSystem("Hidden");
    f.show();
  }
  
  public void componentMoved(final java.awt.event.ComponentEvent p1) {
    locationX = (int)f.getLocation().getX();
    locationY = (int)f.getLocation().getY();
    owner.getBackground().getLinks().updateLink(owner);
    owner.getBackground().repaint();
  }
  
  // Inner Class
  class ImageSQPanel extends JPanel{

    private static final long serialVersionUID = 1L;
    Image dukesWave[];

    public ImageSQPanel(Image[] dukesWave) {
      this.dukesWave = dukesWave;
    }

    //Draw the current frame of animation.
    public void paintComponent(Graphics g) {
      super.paintComponent(g); //paint background

      //Paint the frame into the image.
      try {
        g.drawImage(dukesWave[frameNumber%10], 0, 0, this);

      } catch (ArrayIndexOutOfBoundsException e) {
        //On rare occasions, this method can be called
        //when frameNumber is still -1.  Do nothing.
        return;
      }
    }
  }

}
