package epe.gui.textOutput;

	import java.awt.Color;
	import java.awt.Dimension;
	import java.awt.FlowLayout;
	import java.awt.Rectangle;
	import java.awt.event.ComponentListener;
	import java.awt.event.MouseEvent;
	import java.awt.event.MouseListener;
	import java.util.Vector;
	
	import javax.swing.JPanel;
	import javax.swing.event.DocumentListener;
	import javax.swing.text.BadLocationException;
	import javax.swing.text.Document;
	import javax.swing.text.Position;
	import javax.swing.text.Style;
	import javax.swing.text.StyleConstants;
	import javax.swing.text.StyleContext;
	
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;

public class AgentTextOutputPanel extends javax.swing.JFrame {
  
    private static final long serialVersionUID = 1L;
    protected Agent owner;
    private javax.swing.JScrollPane outputScrollPane;
	  private javax.swing.JButton showAcquaintancesButton;
	  protected javax.swing.JButton showAgentStateInfoButton;
	  protected boolean positionned = false;
	  private boolean positionning = false;
	  public Object lock = new Object();
	  
	  protected Vector<Object> textBuffer;
	  protected javax.swing.JTextPane outputTextPane;
	  protected Document doc;
	  protected Position docEndPosition;
	  protected Style style;
	  private TextOutputPrinter textOutputPrinter;
	  
	  final int W = 650;
	  final int H = 350;
	  
	  // How many lines are memorized (needed for memory problems)
	  protected int maxHistorySize = 100000;
	  protected int currentHistorySize = 0;
	  protected int deleteSize = 10000;
	  
	  protected String currentTime;
  
  public AgentTextOutputPanel(Agent owner) {
    this.owner = owner;
    initComponents();
    this.setTitle(owner.getAgentName());
    pack();
  }
  
  private void initComponents() {
    showAcquaintancesButton = new javax.swing.JButton();
    showAgentStateInfoButton = new javax.swing.JButton();
    outputScrollPane = new javax.swing.JScrollPane();
    
    // Concerning the outputTexPane
    outputTextPane = new javax.swing.JTextPane();
    //outputTextPane = new javax.swing.JTextArea();
    outputTextPane.addComponentListener(new TextPaneListener(this));
    
    //doc = outputTextPane.getDocument();
    doc = outputTextPane.getStyledDocument();
    docEndPosition = doc.getEndPosition();
    doc.addDocumentListener(new AgentTextOutputPanelDocumentListener(this));
    Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
    style = outputTextPane.addStyle("black", def);
    StyleConstants.setFontFamily(style, "SansSerif");
    StyleConstants.setFontSize(style, 10);
    initStyles();
    try {
      doc.insertString(docEndPosition.getOffset()-1, "\n\n\n", style);
    } catch (BadLocationException ble) {
      SystemPrinter.printSystem("Couldn't insert initial text.");
    }
    
    // Buttons initialization
    showAcquaintancesButton.setText("Show Acquaintances");
    showAcquaintancesButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        showAcquaintancesButtonActionPerformed(evt);
      }
    }
    );
    showAgentStateInfoButton.setText("Show State Info");
    showAgentStateInfoButton.addActionListener(new java.awt.event.ActionListener() {
    	public void actionPerformed(java.awt.event.ActionEvent evt) {
    		showAgentStateInfoButtonActionPerformed(evt);
    	}
    }
    );
    JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(showAcquaintancesButton);
    buttonPanel.add(showAgentStateInfoButton);
    getContentPane().add(buttonPanel, java.awt.BorderLayout.NORTH);
    
    this.setDefaultCloseOperation(HIDE_ON_CLOSE);
    
    outputScrollPane.setViewportView(outputTextPane);
    outputScrollPane.setPreferredSize(new Dimension(W, H));
    outputScrollPane.getVerticalScrollBar()
    								.addMouseListener(new AgentTextOutputPanelMouseListener(this));
    getContentPane().add(outputScrollPane, java.awt.BorderLayout.CENTER);
    
    // Initialize the TextBuffer
    textBuffer = new Vector<Object>();
    
    // Launch the TextOutputPrinter
    textOutputPrinter = new TextOutputPrinter(this);
    new Thread(textOutputPrinter).start();
  }
  
  protected void initStyles() {
    Style s;
    s = outputTextPane.addStyle("black", style);
    StyleConstants.setForeground(s, java.awt.Color.black);
    s = outputTextPane.addStyle("blue", style);
    StyleConstants.setForeground(s, java.awt.Color.blue);
    s = outputTextPane.addStyle("cyan", style);
    StyleConstants.setForeground(s, java.awt.Color.cyan);
    s = outputTextPane.addStyle("darkGray", style);
    StyleConstants.setForeground(s, java.awt.Color.darkGray);
    s = outputTextPane.addStyle("gray", style);
    StyleConstants.setForeground(s, java.awt.Color.gray);
    s = outputTextPane.addStyle("green", style);
    StyleConstants.setForeground(s, java.awt.Color.green);
    s = outputTextPane.addStyle("lightGray", style);
    StyleConstants.setForeground(s, java.awt.Color.lightGray);
    s = outputTextPane.addStyle("magenta", style);
    StyleConstants.setForeground(s, java.awt.Color.magenta);
    s = outputTextPane.addStyle("orange", style);
    StyleConstants.setForeground(s, java.awt.Color.orange);
    s = outputTextPane.addStyle("pink", style);
    StyleConstants.setForeground(s, java.awt.Color.pink);
    s = outputTextPane.addStyle("red", style);
    StyleConstants.setForeground(s, java.awt.Color.red);
    s = outputTextPane.addStyle("yellow", style);
    StyleConstants.setForeground(s, java.awt.Color.yellow);
  }
  
  public void print(String text, Color color) {
    textBuffer.addElement(new BufferedText(text, color));
    textOutputPrinter.newDataAvailable();
  }
  	
  public void print(String text) {
    this.print(text, Color.black);
  }
  
  public void println(String text) {
    this.println(text, Color.black);
  }
  
  public void println(String text, Color color) {
    if (!text.equals("") && !text.equals(".")) {
    	String newTime = owner.getGUI().getTime();
    	if (!newTime.equals(currentTime))
    		currentTime = newTime;
    	else
    		newTime = "              ";
      this.print(newTime + " : " + text + "\n", color );
      
    }
    else
      this.print(text + "\n", color );
  }
  
  public void gotoNextLine() {
    this.print("\n", Color.black );
  }
  
  public void printDate() {
  	String newTime = owner.getGUI().getTime();
  	if (!newTime.equals(currentTime))
  		currentTime = newTime;
  	else
  		newTime = "              ";
    this.print(newTime + " : " , Color.black );
  }
  
  public void showAgentTextOutputPanel() {
    setVisible(true);
    requestFocus();
    print("");
  }
  
  public void setPositionning() {
    positionning = true;
  }
  
  public void setPositionned() {
    updatePosition();
    positionned = true;
    positionning = false;
  }
  
  public void setUnPositionned() {
    positionned = false;
  }
  
  public void updatePosition() {
    outputScrollPane.getViewport().getViewPosition();
  }
  
  private void showAcquaintancesButtonActionPerformed(java.awt.event.ActionEvent evt) {
    owner.getAcquaintancesManager().showAllAcquaintances();
  }
  
  private void showAgentStateInfoButtonActionPerformed(java.awt.event.ActionEvent evt) {
  	owner.getReorganizationManager().showAgentStateInfo();
  }
  
  public void updateView() {
    if ((!positionned)&&(!positionning)) {
      int docEnd = docEndPosition.getOffset()-1;
      Rectangle rect = new Rectangle(0, docEnd, 0, 0);
      outputTextPane.scrollRectToVisible(rect);
    }
  }
  
  public void clearLog() {
  	int begin = doc.getStartPosition().getOffset();
  	int end = doc.getLength();
  	synchronized(lock) {
      try {
        doc.remove(begin, end);
      } catch (javax.swing.text.BadLocationException ble) {
        SystemPrinter.printSystem("ERROR : Couldn't clear log.");
      }
      currentHistorySize = 0;
    }
  }
  
  // Management of mouseEvents on scrollBars
  class AgentTextOutputPanelMouseListener implements MouseListener  {
    AgentTextOutputPanel textOutput;
    
    public AgentTextOutputPanelMouseListener(AgentTextOutputPanel textOutput) {
      this.textOutput = textOutput;
    }
    public void mouseClicked(MouseEvent e) {
      textOutput.setUnPositionned();
      textOutput.updateView();
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
    public void mousePressed(MouseEvent e) {
      textOutput.setPositionning();
    }
    public void mouseReleased(MouseEvent e) {
      textOutput.updatePosition();
      textOutput.setPositionned();
    }
  }
  
  // Management of events in the document
  class AgentTextOutputPanelDocumentListener implements DocumentListener  {
    AgentTextOutputPanel textOutput;
    
    public AgentTextOutputPanelDocumentListener(AgentTextOutputPanel textOutput) {
      this.textOutput = textOutput;
    }
    public void changedUpdate(final javax.swing.event.DocumentEvent p1) {
    }
    public void removeUpdate(final javax.swing.event.DocumentEvent p1) {
    }
    public void insertUpdate(final javax.swing.event.DocumentEvent p1) {
      //int pos = p1.getOffset() + p1.getLength();
      //textOutput.updateView(pos);
    }
  }
  
  // Management of events in the TextPane
  class TextPaneListener implements ComponentListener {
    AgentTextOutputPanel textOutput;
    
    public TextPaneListener(AgentTextOutputPanel textOutput) {
      this.textOutput = textOutput;
    }
    public void componentMoved(final java.awt.event.ComponentEvent p1) {
    }
    public void componentShown(final java.awt.event.ComponentEvent p1) {
    }
    public void componentResized(final java.awt.event.ComponentEvent p1) {
      textOutput.updateView();
    }
    public void componentHidden(final java.awt.event.ComponentEvent p1) {
    }
  }
}