package epe.observationTools.graphLayoutTool;

	import java.awt.Dimension;
	import epe.gui.ProgrammingEnvironment;
	import epe.agents.agentListeners.reorganization.ReorganizationListener;
	import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

// @author  Bastien Hofmann
public class GraphLayoutToolInterface implements ReorganizationListener {
  
  private ProgrammingEnvironment environment;
  private GraphLayoutToolLogOutput log;
  
  /** ToolManager to use */
  private final ToolManager toolManager;
  
  /** Creates a new instance of GraphLayoutToolInterface */
  public GraphLayoutToolInterface(ProgrammingEnvironment env) {
    environment = env;
    log = new GraphLayoutToolLogOutput();
    log.setBounds(log.getNormalBounds());
    //log.setResizable(true);
    //log.setClosable(true);
    //log.setIconifiable(true);
    //log.setMaximizable(true);
    //log.setVisible(true);
    environment.getMainJDesktopPane().add(log, javax.swing.JLayeredPane.PALETTE_LAYER);
    this.toolManager = new ToolManager(env, log);
    environment.getMainJDesktopPane().add(this.toolManager.frame, javax.swing.JLayeredPane.PALETTE_LAYER);
    this.toolManager.frame.setBounds(0, 1024, 2500, 1500);
    this.toolManager.frame.setResizable(true);
    this.toolManager.frame.setClosable(true);
    this.toolManager.frame.setIconifiable(true);
    this.toolManager.frame.setMaximizable(true);
  }
  
  /* statics of available types */
  /* normal agent*/
  public static final int NORMAL_AGENT = 0;
  /* the agent is the input agent */
  public static final int INPUT_AGENT = 1;
  /* the agent is the output agent */
  public static final int OUTPUT_AGENT = 2;
  /* the agent is the environment agent */
  public static final int ENVIRONMENT_ENVIRONMENT = 3;
  
  /** adds a new agent to the tool
   * @param Label of the agent to be added
   * @param Type of the agent to be added
   * @param Color of the box of the agent to be added
   * @return true if the agent was added successfully
   */
  public boolean addAgent(java.lang.String label, int type, java.awt.Color color) {
    boolean ok = this.toolManager.agentManager.add(label, type, color);
    if (ok) {
      this.toolManager.organizationManager.modified = true;
      this.toolManager.update();
    }
    return(ok);
  }
  
  /** remove the agent with the specified label
   * @param Label of the agent to be removed
   * @return true if the agent was removed
   */
  public boolean removeAgent(java.lang.String label) {
    boolean ok = this.toolManager.agentManager.remove(label);
    if (ok) {
      this.toolManager.organizationManager.modified = true;
      this.toolManager.update();
    }
    return(ok);
  }
  
  /** Returns the number of agents stored in the GraphLayoutToolInterface
   * @return Agent count
   */
  public int getAgentCount() {
    return(this.toolManager.agentManager.agents.size());
  }
  
  /** Returns the label of the agent at the specified index
   * @return Label of the agent
   * @exception ArrayIndexOutOfBoundsException if the index is out of bounds
   */
  public String getAgent(int index) throws ArrayIndexOutOfBoundsException {
    return(this.toolManager.agentManager.get(index).label);
  }
  
  /** adds a new link to the tool
   * @param Label of the source agent
   * @param Label of the target agent
   * @return true if the link was added successfully
   */
  public boolean addLink(java.lang.String source, java.lang.String target) {
    boolean ok = this.toolManager.linkManager.add(source, target);
    if (ok) {
      this.toolManager.organizationManager.modified = true;
      this.toolManager.update();
    }
    return(ok);
  }
  
  /** remove the link specified by his two labels
   * @param Label of the source agent
   * @param Label of the target agent
   * @return true if the link was removed successfully
   */
  public boolean removeLink(java.lang.String source, java.lang.String target) {
    boolean ok = this.toolManager.linkManager.remove(source, target);
    if (ok) {
      this.toolManager.organizationManager.modified = true;
      this.toolManager.update();
    }
    return(ok);
  }
  
  /** Stop reorganizing automatically (for batch adding...)
   */
  public void pause() {
    this.toolManager.stop();
  }
  
  /** End of the pause
   */
  public void endPause() {
    if (this.toolManager.frame.isShowing()) {
      this.toolManager.start();
    }
  }
  
  /** open the tool window to show the organized graph
   * @param left of the window
   * @param top of the window
   * @param width of the window
   * @param height of the window
   */
  public void show(int left, int top, int width, int height) {
      /*
        this.toolManager.frame.setBounds(new Rectangle(left, top, width, height));
        this.toolManager.frame.show();
        this.toolManager.start();
       */
    
    this.toolManager.frame.setBounds(left, top, width, height);
    this.toolManager.frame.setVisible(true);
    this.toolManager.frame.repaint();
    this.toolManager.start();
    visibilityResize();
  }

  public void initiallyShown(int left, int top, int width, int height) {
    this.toolManager.frame.setBounds(left, top, width, height);
    this.toolManager.frame.setVisible(true);
    this.toolManager.frame.repaint();
    this.toolManager.start();
    //hide();
  }
  
   public void visibilityResize() {
    Dimension d = this.toolManager.panel.getSize();
    // if the size changed or on first time, recreate the buffer
    if ((this.toolManager.panel.buffer == null) 
     || (d.width != this.toolManager.panel.bufferSize.width) 
     || (d.height != this.toolManager.panel.bufferSize.height)) {
      this.toolManager.panel.buffer = this.toolManager.panel.createImage(d.width, d.height);
      this.toolManager.panel.bufferSize = d;
      this.toolManager.agentManager.dimension = d;
      if (this.toolManager.panel.bufferGraphics != null) {
        this.toolManager.panel.bufferGraphics.dispose();
      }
      this.toolManager.panel.bufferGraphics = this.toolManager.panel.buffer.getGraphics();
      this.toolManager.panel.bufferGraphics.setFont(this.toolManager.panel.getFont());
      this.toolManager.agentManager.fontMetrics = this.toolManager.panel.bufferGraphics.getFontMetrics();
    }
  }
   
  /** hides the tool window
   */
  public void hide() {
    this.toolManager.stop();
    //this.toolManager.frame.hide();
    this.toolManager.frame.setVisible(false);
  }
  
  // Section "ReorganizationListener"
  public void linkAdded(String type, String source, String target) {
    addLink(source, target);
  }
  
  public void linkRemoved(String type, String source, String target) {
    removeLink(source, target);
  }
  // End of section "ReorganizationListener"
  
}
