/*
 * toolManager.java
 *
 * Created on May 27, 2002, 2:48 PM
 */

package epe.observationTools.graphLayoutTool;

import javax.swing.JInternalFrame;

import epe.gui.ProgrammingEnvironment;
import epe.observationTools.graphLayoutTool.agents.AgentManager;
import epe.observationTools.graphLayoutTool.links.LinkManager;
import epe.observationTools.graphLayoutTool.organizers.CouplesOrganizer;
import epe.observationTools.graphLayoutTool.organizers.EnvironmentOrganizer;
import epe.observationTools.graphLayoutTool.organizers.InputsOrganizer;
import epe.observationTools.graphLayoutTool.organizers.LeavesOrganizer;
import epe.observationTools.graphLayoutTool.organizers.LinksOrganizer;
import epe.observationTools.graphLayoutTool.organizers.OrganizationManager;
import epe.observationTools.graphLayoutTool.organizers.OutputsOrganizer;
import epe.observationTools.graphLayoutTool.organizers.ReclusesOrganizer;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutPanel;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;
import epe.observationTools.graphLayoutTool.ui.PanelListener;

/**
 *
 * @author  hofmann
 */
public class ToolManager implements Runnable {
  
  /**
   * options to change if needed
   */
  
  /* Wether the graph is animated or not */
  public boolean animating = true;
  
  /* Wether the graph needs to be organized or not */
  public boolean organizing = true;
  
  /* Wether the percentage of well placed agents is show or not */
  public boolean showWellPlacedPercentage = false;
  
  /* Wether the Log is shown or not */
  public boolean showLog = false;
  
  /* AgentManager */
  public final AgentManager agentManager;
  
  /* LinkManager */
  public final LinkManager linkManager;
  
  /* OrganizationManager */
  public final OrganizationManager organizationManager;
  
  /* Frame containing the display */
  //public final javax.swing.JFrame frame;
  public final javax.swing.JInternalFrame frame;
  
  /* Panel displaying the graph */
  public final GraphLayoutPanel panel;
  
  /** PanelListener of the panel
   */
  private PanelListener panelListener;
  
  /* Thread used to animate the graph */
  private Thread thread;
  
  /* Wether the thread is running or not */
  public boolean running = false;
  
  /* Last proportion of well organized agents */
  public double lastProportion = 0;
  
  private ProgrammingEnvironment environment;
  private GraphLayoutToolLogOutput log;
  
  public ToolManager(ProgrammingEnvironment env, GraphLayoutToolLogOutput log) {
    this.log = log;
    environment = env;
    // managers
    this.agentManager = new AgentManager();
    this.linkManager = new LinkManager(agentManager);
    this.agentManager.addObserver(this.linkManager);
    // ui
    //this.frame = new JFrame("GraphLayoutTool");
    this.frame = new JInternalFrame("GraphLayoutTool");
    this.panel = new GraphLayoutPanel(this, log);
    this.panelListener = new PanelListener(this, log);
    this.panel.addComponentListener(panelListener);
    this.panel.addMouseListener(panelListener);
    this.panel.addMouseMotionListener(panelListener);
    this.frame.getContentPane().add(this.panel);
    this.frame.setDefaultCloseOperation(javax.swing.JInternalFrame.DO_NOTHING_ON_CLOSE);
    this.frame.addInternalFrameListener(new internalFrameEventHandler());
    
    // OrganizationManager
    this.organizationManager = new OrganizationManager(this, log);
    // Graph organizers
    this.organizationManager.addGraphOrganizer(new ReclusesOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new EnvironmentOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new OutputsOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new InputsOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new CouplesOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new LeavesOrganizer(this, log));
    this.organizationManager.addGraphOrganizer(new LinksOrganizer(this, log));
  }
  
  public ProgrammingEnvironment getEnvironment() {
    return environment;
  }
  
  /** updates the graph display
   */
  public void update() {
    panel.repaint();
  }
  
  public void run() {
    while (this.running) {
      if (this.organizing) {
        this.lastProportion = this.organizationManager.organize();
        //this.agentManager.moveAgents();
        this.organizationManager.space();
        this.agentManager.moveAgents();
        frame.repaint();
      }
      try {
        Thread.sleep(100);
      } catch (InterruptedException e) {
        this.running = false;
      }
    }
  }
  
  /** Creates the thread that starts the animation and the organization
   */
  public void start() {
    if (this.thread == null) {
      this.thread = new Thread(this);
      this.thread.setPriority(Thread.MIN_PRIORITY);
      this.running = true;
      this.thread.start();
    }
  }
  
  /** Stops the animation and the organization
   */
  public void stop() {
    this.running = false;
    this.thread = null;
  }
  
  
  // Inner class to handle the internalFrame events
  public class internalFrameEventHandler implements javax.swing.event.InternalFrameListener {
    public void internalFrameClosing(javax.swing.event.InternalFrameEvent internalFrameEvent) {
      stop();
      frame.setVisible(false);
      log.println("DEBUG : internalFrameClosing");
    }
    public void internalFrameClosed(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
    public void internalFrameActivated(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
    public void internalFrameDeactivated(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
    public void internalFrameDeiconified(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
    public void internalFrameIconified(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
    public void internalFrameOpened(javax.swing.event.InternalFrameEvent internalFrameEvent) {
    }
  }
  
}
