/*
 * AgentManager.java
 *
 * Created on April 24, 2002, 3:06 PM
 */

package epe.observationTools.graphLayoutTool.agents;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.util.Observable;

/**
 * AgentManager
 * Stores and paints the Agents to be drawn
 * @author  Bastien Hofmann
 */
public class AgentManager extends Observable {
    
    /**
     * options to change if needed
     */
    
    /* Wether the agents are drawn white when the mouse is over a related agent or not*/
    public boolean highlightRelated = true;
    
    /* Wether the label of the agents is drawn white when it is well placed or not*/
    public boolean highlightWellPlaced = false;
    
    
    /** Vector<Object> of the agents managed by this AgentManager */
    public final AgentVector agents;
        
    /** Agent being dragged
     */
    public static Agent picked;
    
    /** Agent having the mouve over it
     */
    public static Agent pointed;
        
    /** Dimension of the panel */
    public Dimension dimension;
    
    /** FontMetrics */
    public FontMetrics fontMetrics;
    
    /** Creates a new instance of AgentManager */
    public AgentManager() {
        this.agents = new AgentVector();
        this.dimension = new Dimension(400,400);
    }
    
    /** Returns the agent located at the specified index
     * @param index of the agent wanted
     * @return the agent found
     * @exception ArrayIndexOutOfBoundsException - index is out of range
     */
    public Agent get(int index) throws ArrayIndexOutOfBoundsException {
        return(this.agents.agent(index));
    }
    
    /** Adds a new agent to the AgentManager with the specified label
     * @param label to put on the box of the agent
     * @return true if added successfully
     */
    public boolean add(java.lang.String label,int type, java.awt.Color color) {
        // if no Agent with this label exists
        if (this.agents.indexOf(label)==-1) {
            // adds a new agent
            return(this.agents.add(new Agent(label, type, color, this)));
        }
        // An Agent already exists with this label
        return(false);
    }
    
    /** Remove the specified Agent from the AgentManager
     * @param label of the Agent to be removed
     * @return true if the agent has been successfully removed
     */
    public boolean remove(java.lang.String label) {
        // if an Agent exists with this label
        int index = this.agents.indexOf(label);
        if (index>-1) {
            // remove all the links involving this agent
            /*if (this.linkManager != null) {
                this.linkManager.removeLinksFromAgent(this.agents.agent(index));
            }*/
            // tell the observers that an agent is being removed
            this.setChanged();
            this.notifyObservers(this.get(index));
            // removing the agent himself
            this.agents.remove(index);
            return(true);
        }
        // No Agent has been found
        return(false);
    }
    
    /** Paints the agent on the image given in parameter
     * @param image to draw the Agents on
     */
    public void paint(java.awt.Graphics gr) {
        // paints one agent by one
        for (int i = 0; i < this.agents.size(); i++) {
            this.agents.agent(i).paint(gr);
        }
    }
    
    /** Returns the agent located at the specified position
     * @param x coordinate in pixels of the point
     * @param y coordinate in pixels of the point
     * @return The agent at the position specified
     */
    public Agent getAgentFromPoint(int x, int y) {
        /* the agent are painted in the order of the vector
         * so the topmost is the last on the vector
         * so we have to start from the end of the vector
         * to find the agent we see
         */
        for (int i = this.agents.size() - 1; i >= 0; i--) {
            Agent a = this.agents.agent(i);
            int l = a.getLeft();
            int t = a.getTop();
            // if the point is in the agent boundaries than it's the one
            if (l <= x
                && t <= y
                && l + a.getWidth() >= x
                && t + a.getHeight() >= y) {
                    return(a);
            }
        }
        return(null);
    }
    
    /** Moves the agents to their new positions
     */
    public void moveAgents() {
        for (int i = 0; i < this.agents.size(); i++) {
            this.get(i).move();
        }        
    }
    
}
