/*
 * AgentVector.java
 *
 * Created on April 29, 2002, 9:25 AM
 */

package epe.observationTools.graphLayoutTool.agents;


/**
 *
 * @author  hofmann
 */
public class AgentVector extends java.util.Vector<Object> {
    
  private static final long serialVersionUID = 1L;

    /** Creates a new instance of AgentVector */
    public AgentVector() {
    }
    
    /** Returns the agent located at the specified index
     * @param index of the agent to return
     * @return An agent if found
     */
    public Agent agent(int param) {
        return((Agent)this.get(param));
    }
    
    /** Returns the index of the agent with the specified label
     * @param Label of the agent searched
     * @return Index of the agent if found, -1 otherwise
     */
    public int indexOf(String lb) {
        // looks every agent to find one with the specified label
        for (int i = 0; i < this.size(); i++) {
            if (this.agent(i).label.equals(lb)) {
                return(i);
            }
        }
        return(-1);
    }
    
        
    /** Get the left of the more to the left agent
     * @return The minimum left in percentage
     */
    public int getMinLeft() {
        // initializing with the maximum float value
        int min = Integer.MAX_VALUE;
        
        // return 0 if this vector has no elements
        if (this.size() == 0) {
            return(0);
        }

        // finds the more to the left agent
        for (int i = 0; i < this.size(); i++) {
            Agent ag = this.agent(i);
            int l = ag.getLeft();
            if (l < min) {
                min = l;
            }
        }
        return(min);
    }

    /** Get the left of the more to the right agent
     * @return The minimum left in percentage
     */
    public int getMaxLeft() {
        // initializing with the minimum float value
        int max = Integer.MIN_VALUE;
        
        // return 0 if this vector has no elements
        if (this.size() == 0) {
            return(0);
        }
        // finds the more to the right agent
        for (int i = 0; i < this.size(); i++) {
            Agent ag = this.agent(i);
            int l = ag.getLeft();
            if (l > max) {
                max = l;
            }
        }
        return(max);
    }

    /** Get the top of the more to the top agent
     * @return The minimum left in percentage
     */
    public int getMinTop() {
        // initializing with the maximum float value
        int min = Integer.MAX_VALUE;
        
        // return 0 if this vector has no elements
        if (this.size() == 0) {
            return(0);
        }
        // finds the more to the top agent
        for (int i = 0; i < this.size(); i++) {
            Agent ag = this.agent(i);
            int l = ag.getTop();
            if (l < min) {
                min = l;
            }
        }
        return(min);
    }

    /** Get the top of the more to the bottom agent
     * @return The minimum left in percentage
     */
    public int getMaxTop() {
        // initializing with the minimum float value
        int max = Integer.MIN_VALUE;
        
        // return 0 if this vector has no elements
        if (this.size() == 0) {
            return(0);
        }
        // finds the more to the bottom agent
        for (int i = 0; i < this.size(); i++) {
            Agent ag = this.agent(i);
            int l = ag.getTop();
            if (l > max) {
                max = l;
            }
        }
        return(max);
    }

}
