/*
 * GridOrganizer.java
 *
 * Created on April 26, 2002, 1:48 PM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

/**
 *
 * @author  hofmann
 */
public class CouplesOrganizer extends GraphOrganizer {
  
  /* toolManager to use */
  private ToolManager toolManager;
  
  private GraphLayoutToolLogOutput log;
  
  /** Creates a new instance of CouplesOrganizer
   * @param toolManager to use
   */
  public CouplesOrganizer(ToolManager tm, GraphLayoutToolLogOutput log) {
    this.toolManager = tm;
    this.log = log;
  }
  
  /** arrange the agents
   */
  public void arrangeAgents() {
    // for every agent couple selected
    for (int i = 0; i < this.selectedAgents.size(); i += 2) {
      // get the two agents and their coordinates
      Agent ag1 = this.selectedAgents.agent(i);
      Agent ag2 = this.selectedAgents.agent(i + 1);
      int l1, l2, t1, t2, r1, r2, b1, b2;
      boolean m1, m2;
      l1 = ag1.getLeft();
      t1 = ag1.getTop();
      r1 = ag1.getLeft() + ag1.getWidth();
      b1 = ag1.getTop() + ag1.getHeight();
      l2 = ag2.getLeft();
      t2 = ag2.getTop();
      r2 = ag2.getLeft() + ag2.getWidth();
      b2 = ag2.getTop() + ag2.getHeight();
      // if the two agents are not one on top of the other
      if (!((((l2 >= l1 - GraphOrganizer.SPACE) && (l2 <= r1 + GraphOrganizer.SPACE))
      || ((r2 >= l1 - GraphOrganizer.SPACE) && (r2 <= r1 + GraphOrganizer.SPACE)))
      && ((t2 - b1 == GraphOrganizer.SPACE) || (t1 - b2 == GraphOrganizer.SPACE)))) {
        // is ag1 moving
        m1 = (ag1.offsetX == 0) && (ag1.offsetY == 0);
        // is ag2 moving
        m2 = (ag2.offsetX == 0) && (ag2.offsetY == 0);
        if (!m1 && !m2) {
          m1 = true;
          m2 = true;
        }
        // if ag1 is higher than ag2
        if (ag1.getTop() < ag2.getTop()) {
          // if ag1 is moving he will move more
          if (m1) {
            // if ag1 is still not at the good position and ag2 is moving, he will be moved too
            if (!ag1.moveTo(ag2.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2 + 1, ag2.getTop() - ag1.getHeight() - GraphOrganizer.SPACE + 1) && m2) {
              ag2.moveTo(ag1.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2, ag1.getTop() + ag1.getHeight() + GraphOrganizer.SPACE);
            }
            // only ag2 will move
          } else {
            ag2.moveTo(ag1.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2, ag1.getTop() + ag1.getHeight() + GraphOrganizer.SPACE);
          }
          // ag2 is higher than ag1
        } else {
          // if ag1 is moving he will move more
          if (m1) {
            // if ag1 is still not at the good position and ag2 is moving, he will be moved too
            if (!ag1.moveTo(ag2.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2 + 1, ag2.getTop() + ag2.getHeight() + GraphOrganizer.SPACE + 1) && m2) {
              ag2.moveTo(ag1.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2, ag1.getTop() - ag2.getHeight() - GraphOrganizer.SPACE);
            }
          } else {
            // only ag2 will move
            ag2.moveTo(ag1.getLeft() + (ag2.getWidth() - ag1.getWidth()) / 2, ag1.getTop() - ag2.getHeight() - GraphOrganizer.SPACE);
          }
        }
      }
    }
  }
  
  /** selects the agents that the organizer knows how to place
   * and returns the ones he can't
   * @param Vector<Object> of the agents it should organize
   * @return Vector<Object> of the agents he couldn't organize
   */
  public AgentVector selectAgents(AgentVector grp) {
    this.selectedAgents = new AgentVector();
    for (int i = 0; i < grp.size(); i++) {
      Agent ag1 = grp.agent(i);
      for (int j = 0; j < this.toolManager.agentManager.agents.size(); j++) {
        Agent ag2 = this.toolManager.agentManager.get(j);
        if ((ag1 == ag2) || (this.selectedAgents.indexOf(ag2.label) > -1)) {
          continue;
        }
        if ((this.toolManager.linkManager.links.indexOf(ag1, ag2) > -1) && (this.toolManager.linkManager.links.indexOf(ag2, ag1) > -1)) {
          if (this.toolManager.linkManager.linksAsSource(ag2) == 1) {
            this.selectedAgents.add(ag2);
            this.selectedAgents.add(ag1);
            // prints the name of the agent selected on the standard output
            log.println("CouplesOrganizer : S?lection de (" + ag2.label + ", " + ag1.label + ")");
            grp.remove(ag2);
          } else {
            this.selectedAgents.add(ag1);
            this.selectedAgents.add(ag2);
            // prints the name of the agent selected on the standard output
            log.println("CouplesOrganizer : S?lection de (" + ag1.label + ", " + ag2.label + ")");
            grp.remove(ag1);
          }
          i--;
          break;
        }
      }
    }
    return(grp);
  }
  
  /** Return true if the agent given in parameter is well placed
   * @param Agent to check the position
   * @return True if the agent is in good place, false otherwise
   */
  protected boolean isWellPlaced(Agent ag1) {
    int pos = this.selectedAgents.indexOf(ag1);
    Agent ag2;
    if (pos < 0) {
      return(true);
    } else if (pos % 2 == 1) {
      ag2 = this.selectedAgents.agent(pos - 1);
    } else {
      ag2 = this.selectedAgents.agent(pos + 1);
    }
    int l1, l2, t1, t2, r1, r2, b1, b2;
    l1 = ag1.getLeft();
    t1 = ag1.getTop();
    r1 = ag1.getLeft() + ag1.getWidth();
    b1 = ag1.getTop() + ag1.getHeight();
    l2 = ag2.getLeft();
    t2 = ag2.getTop();
    r2 = ag2.getLeft() + ag2.getWidth();
    b2 = ag2.getTop() + ag2.getHeight();
    // if the two agents are not one on top of the other
    if (!((((l2 >= l1 - GraphOrganizer.SPACE) && (l2 <= r1 + GraphOrganizer.SPACE))
    || ((r2 >= l1 - GraphOrganizer.SPACE) && (r2 <= r1 + GraphOrganizer.SPACE)))
    && ((t2 - b1 == GraphOrganizer.SPACE) || (t1 - b2 == GraphOrganizer.SPACE)))) {
      return(false);
    }
    return(true);
  }
  
  /** Arrange the agent given in parameter
   * @param Agent to arrange
   */
  protected void arrangeAgent(Agent ag) {
  }
  
  /** Returns the bouding rect in wich the agent must contain
   */
  protected Rect getBoundRect(Agent ag) {
    return(null);
  }
  
}
