/*
 * GridOrganizer.java
 *
 * Created on April 26, 2002, 1:48 PM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

/**
 *
 * @author  hofmann
 */
public class EnvironmentOrganizer extends GraphOrganizer {
    
    /** toolManager */
    private final ToolManager toolManager;
    
    private GraphLayoutToolLogOutput log;
    
    /** Creates a new instance of GridOrganizer */
    public EnvironmentOrganizer(ToolManager tm, GraphLayoutToolLogOutput log) {
        this.toolManager = tm;
        this.log = log;
    }
    
    /** selects the agents that the organizer knows how to place
     * and returns the ones he can't
     * @param Vector<Object> of the agents it should organize
     * @return Vector<Object> of the agents he couldn't organize
     */
    public AgentVector selectAgents(AgentVector grp) {
        this.selectedAgents = new AgentVector();
        // selects only the environment, input and output agents
        for (int i = 0; i < grp.size(); i++) {
            Agent ag = grp.agent(i);
            if (ag.type == Agent.ENVIRONMENT || ag.type == Agent.INPUT || ag.type == Agent.OUTPUT) {
                this.selectedAgents.add(ag);
                // prints the name of the selected agent on the stantard output
                log.println("EnvironmentOrganizer : S?lection de " + ag.label);
                grp.remove(ag);
                i--;
            }
        }
        return(grp);
    }
    
    /** Return true if the agent given in parameter is well placed
     * @param Agent to check the position
     * @return True if the agent is in good place, false otherwise
     */
    /*protected boolean isWellPlaced(Agent ag) {
        int l = ag.getLeft(), t = ag.getTop();
        
        if (ag.type == Agent.ENVIRONMENT) {
            return((l >= (this.toolManager.agentManager.dimension.width - ag.getWidth()) / 2 - GraphOrganizer.SPACE)
            && (l <= (this.toolManager.agentManager.dimension.width - ag.getWidth()) / 2 + GraphOrganizer.SPACE)
            && (t >= this.toolManager.agentManager.dimension.height - ag.getHeight() - 5 - GraphOrganizer.SPACE)
            && (t <= this.toolManager.agentManager.dimension.height - ag.getHeight() - 5));
        // if it's the output agent
        } else if (ag.type == Agent.OUTPUT) {
            return((l >= this.toolManager.agentManager.dimension.width - ag.getWidth() - 5 - GraphOrganizer.SPACE)
            && (l <= this.toolManager.agentManager.dimension.width - ag.getWidth() - 5)
            && (t >= this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 5 - GraphOrganizer.SPACE)
            && (t <= this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 5 + GraphOrganizer.SPACE));
        // if it's the input agent
        } else if (ag.type == Agent.INPUT) {
            return((l >= 5) && (l <= 5 + GraphOrganizer.SPACE)
            && (t >= this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 5 - GraphOrganizer.SPACE)
            && (t <= this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 5 + GraphOrganizer.SPACE)); 
        }
        return(true);
    }*/
    
    /** Arrange the agent given in parameter
     * @param Agent to arrange
     */
    protected void arrangeAgent(Agent ag) {
        // if it's the environment agent
        if (ag.type == Agent.ENVIRONMENT) {
            ag.moveTo((this.toolManager.agentManager.dimension.width - ag.getWidth()) / 2,
                this.toolManager.agentManager.dimension.height - ag.getHeight() - SPACE);
        // if it's the output agent
        } else if (ag.type == Agent.OUTPUT) {
            ag.moveTo(this.toolManager.agentManager.dimension.width - ag.getWidth() - SPACE,
                this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - SPACE); 
        // if it's the input agent
        } else if (ag.type == Agent.INPUT) {
            ag.moveTo(SPACE, this.toolManager.agentManager.dimension.height/2 - ag.getHeight() * 3 - SPACE); 
        }
    }
    
    /** Returns the bouding rect in wich the agent must contain
     */
    protected Rect getBoundRect(Agent ag) {
        Rect rect = new Rect();
        
        if (ag.type == Agent.ENVIRONMENT) {
            rect.left = (this.toolManager.agentManager.dimension.width - ag.getWidth()) / 2 - SPACE;
            rect.right = (this.toolManager.agentManager.dimension.width - ag.getWidth()) / 2 + SPACE + ag.getWidth();
            rect.top = this.toolManager.agentManager.dimension.height - ag.getHeight() - 2 * SPACE;
            rect.bottom = this.toolManager.agentManager.dimension.height - ag.getHeight() - SPACE + ag.getHeight();
        // if it's the output agent
        } else if (ag.type == Agent.OUTPUT) {
            rect.left = this.toolManager.agentManager.dimension.width - ag.getWidth() - 2 * SPACE;
            rect.right = this.toolManager.agentManager.dimension.width - ag.getWidth() - SPACE;
            rect.top = this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 2 * SPACE;
            rect.bottom = this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 2 * GraphOrganizer.SPACE;
        // if it's the input agent
        } else if (ag.type == Agent.INPUT) {
            rect.left = SPACE;
            rect.right = 2 * GraphOrganizer.SPACE;
            rect.top = this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 2 * GraphOrganizer.SPACE;
            rect.bottom = this.toolManager.agentManager.dimension.height - ag.getHeight() * 3 - 2 * GraphOrganizer.SPACE;
        }
        rect.right += ag.getWidth();
        rect.bottom += ag.getTop();
        return(rect);
    }
    
}
