/*
 * GraphOrganizer.java
 *
 * Created on April 26, 2002, 10:37 AM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
    
/**
 *
 * @author  hofmann
 */
public abstract class GraphOrganizer {
 
    protected class Rect {
        public int left;
        public int top;
        public int right;
        public int bottom;
    }
    /* Space between the agents in pixels */
    public static final int SPACE = 20;
    public static final int MIN_SPACE = 10;
    
    /** Vector<Object> of selected agents */
    protected AgentVector selectedAgents;
    
    /** selects the agents that the organizer knows how to place
     * and returns the ones he can't
     * @param Vector<Object> of the agents it should organize
     * @return Vector<Object> of the agents he couldn't organize
     */
    public abstract AgentVector selectAgents(AgentVector grp);

    /** Return the vector containing the selected agents
     * @return Vector<Object> of the selected agents
     */
    public AgentVector getSelectedAgents() {
        return(this.selectedAgents);
    }
    
    /** Arrange the agent given in parameter
     * @param Agent to arrange
     */
    protected abstract void arrangeAgent(Agent ag);
    
    /** arrange the agents
     */
    public void arrangeAgents() {
        for (int i = 0; i < this.selectedAgents.size(); i++) {
            if (!this.isWellPlaced(this.selectedAgents.agent(i))) {
                this.arrangeAgent(this.selectedAgents.agent(i));
            }
        }
    }
    
    /** Returns the bouding rect in wich the agent must contain
     */
    protected abstract Rect getBoundRect(Agent ag);
    
    /** Return true if the agent given in parameter is well placed
     * @param Agent to check the position
     * @return True if the agent is in good place, false otherwise
     */
    protected boolean isWellPlaced(Agent ag) {
        int l = ag.getLeft(),
            t = ag.getTop(),
            r = l + ag.getWidth(),
            b = t + ag.getHeight();
        Rect rect = this.getBoundRect(ag);
        return(l >= rect.left && r <= rect.right && t >= rect.top && b <= rect.bottom);
    }
    
    /** Return the count of well placed agents
     * @return count of well placed agents
     */
    public int getWellPlacedCount() {
        int c = 0;
        
        for (int i = 0; i < this.selectedAgents.size(); i++) {
            if (this.selectedAgents.agent(i).wellPlaced = this.isWellPlaced(this.selectedAgents.agent(i))) {
                c++;
            }
        }
        return(c);
    }
    
    /** Return the proportion of well placed agents
     * @return Proportion of well placed agents
     */
    public double getWellPlacedProportion() {
        if (this.selectedAgents.size() == 0) {
            return(1d);
        }
        return(((double)(this.getWellPlacedCount())) / ((double)(this.selectedAgents.size())));
    }
    
}
