/*
 * GridOrganizer.java
 *
 * Created on April 26, 2002, 1:48 PM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.links.Link;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;


/**
 *
 * @author  hofmann
 */
public class LeavesOrganizer extends GraphOrganizer {
  
  /* toolManager */
  private ToolManager toolManager;
  
  private GraphLayoutToolLogOutput log;
  
  /** Creates a new instance of LeavesOrganizer
   * @param toolManager to use
   */
  public LeavesOrganizer(ToolManager tm, GraphLayoutToolLogOutput log) {
    this.toolManager = tm;
    this.log = log;
  }
  
  /** arrange the agents
   * @param Level of the arrangement, the first one has level1, then 2 etc...
   */
  public void arrangeAgents() {
    // for every agent selected
    for (int i = 0; i < this.selectedAgents.size(); i++) {
      Agent ag = this.selectedAgents.agent(i);
      int linkAsSource = this.toolManager.linkManager.getFirstLinkAsSource(ag);
      int linkAsTarget = this.toolManager.linkManager.getFirstLinkAsTarget(ag);
      if (linkAsTarget==-1) { // it's just a source
        if (linkAsSource>-1) {
          Agent tg = ((Link)(this.toolManager.linkManager.links.get(linkAsSource))).target;
          if (!((tg.getLeft() - (ag.getLeft() + ag.getWidth()) >= GraphOrganizer.SPACE)
          && (tg.getLeft() - (ag.getLeft() + ag.getWidth()) <= 2*GraphOrganizer.SPACE)
          && ((tg.getTop() - (ag.getTop() + ag.getHeight()) <= GraphOrganizer.SPACE)
          && (ag.getTop() - (tg.getTop() + tg.getHeight()) <= GraphOrganizer.SPACE)))) {
            ag.moveTo(tg.getLeft() - ag.getWidth() - GraphOrganizer.SPACE, tg.getTop());
          }
        }
      } else if (linkAsSource==-1) { // it's just a target
        if (linkAsTarget>-1) {
          Agent src = ((Link)(this.toolManager.linkManager.links.get(linkAsTarget))).source;
          if (!(((ag.getLeft() - (src.getLeft() + src.getWidth())) == GraphOrganizer.SPACE)
          && ((src.getTop() - (ag.getTop() + ag.getHeight()) <= GraphOrganizer.SPACE)
          && (ag.getTop() - (src.getTop() + src.getHeight()) <= GraphOrganizer.SPACE)))) {
            ag.moveTo(src.getLeft() + src.getWidth() + GraphOrganizer.SPACE, src.getTop());
          }
        }
      }
    }
  }
  
  /** selects the agents that the organizer knows how to place
   * and returns the ones he can't
   * @param Vector<Object> of the agents it should organize
   * @return Vector<Object> of the agents he couldn't organize
   */
  public AgentVector selectAgents(AgentVector grp) {
    this.selectedAgents = new AgentVector();
    for (int i = 0; i < grp.size(); i++) {
      Agent ag = grp.agent(i);
      // selects all the agents being only source or only target
      if ((this.toolManager.linkManager.isSource(ag) ^
      this.toolManager.linkManager.isTarget(ag))) {
        this.selectedAgents.add(ag);
        // prints the name of the agent selected on the standard output
        log.println("LeavesOrganizer : S?lection de " + ag.label);
        grp.remove(ag);
        i--;
      }
    }
    return(grp);
  }
  
  /** Return true if the agent given in parameter is well placed
   * @param Agent to check the position
   * @return True if the agent is in good place, false otherwise
   */
  protected boolean isWellPlaced(Agent ag) {
    int linkAsSource = this.toolManager.linkManager.getFirstLinkAsSource(ag);
    int linkAsTarget = this.toolManager.linkManager.getFirstLinkAsTarget(ag);
    if (linkAsTarget==-1) { // it's just a source
      if (linkAsSource>-1) {
        Agent tg = ((Link)(this.toolManager.linkManager.links.get(linkAsSource))).target;
        if (!((tg.getLeft() - (ag.getLeft() + ag.getWidth()) >= GraphOrganizer.SPACE)
        && (tg.getLeft() - (ag.getLeft() + ag.getWidth()) <= 2*GraphOrganizer.SPACE)
        && ((tg.getTop() - (ag.getTop() + ag.getHeight()) <= GraphOrganizer.SPACE)
        && (ag.getTop() - (tg.getTop() + tg.getHeight()) <= GraphOrganizer.SPACE)))) {
          return(false);
        }
      }
    } else if (linkAsSource==-1) { // it's just a target
      if (linkAsTarget>-1) {
        Agent src = ((Link)(this.toolManager.linkManager.links.get(linkAsTarget))).source;
        if (!(((ag.getLeft() - (src.getLeft() + src.getWidth())) == GraphOrganizer.SPACE)
        && ((src.getTop() - (ag.getTop() + ag.getHeight()) <= GraphOrganizer.SPACE)
        && (ag.getTop() - (src.getTop() + src.getHeight()) <= GraphOrganizer.SPACE)))) {
          return(false);
        }
      }
    }
    return(true);
  }
  
  /** Arrange the agent given in parameter
   * @param Agent to arrange
   */
  protected void arrangeAgent(Agent ag) {
  }
  
  /** Returns the bouding rect in wich the agent must contain
   */
  protected Rect getBoundRect(Agent ag) {
    return(null);
  }
  
}
