/*
 * GridOrganizer.java
 *
 * Created on April 26, 2002, 1:48 PM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

/**
 *
 * @author  hofmann
 */
public class LinksOrganizer extends GraphOrganizer {
    
    /* ToolManager */
    private ToolManager toolManager;
    
    private GraphLayoutToolLogOutput log;
    
    /** Creates a new instance of GridOrganizer
     * @param toolManager to use
     */
    public LinksOrganizer(ToolManager tm, GraphLayoutToolLogOutput log) {
        this.toolManager = tm;
        this.log = log;
    }
    
    /** arrange the agents
     * @param Level of the arrangement, the first one has level1, then 2 etc...
     */
    public void arrangeAgents() {                
        // for every agent selected
        for (int i = 0; i < this.selectedAgents.size(); i++) {
            Agent ag = this.selectedAgents.agent(i);
            AgentVector av = this.toolManager.linkManager.getTargets(ag);
            AgentVector sources = this.toolManager.linkManager.getSources(ag);
            int nw = sources.getMaxLeft() - sources.getMinLeft() + ag.getWidth() + GraphOrganizer.SPACE * 2;
            for (int j = 0; j < av.size(); j++) {
                if (this.toolManager.linkManager.links.indexOf(av.agent(j), ag) > -1) {
                    av.remove(j);
                    j--;
                }
            }
            int l = av.getMinLeft();
            int t = av.getMinTop();
            int b = av.getMaxTop();
            if (b == t) {
                if (nw < l && ((ag.getLeft() > l - ag.getWidth() - GraphOrganizer.SPACE)
                        || (ag.getLeft() < l/2))) {
                    ag.moveTo(l - ag.getWidth() - GraphOrganizer.SPACE, ag.getTop());
                }
            } else {
                if (((ag.getLeft() > l - ag.getWidth() - GraphOrganizer.SPACE)
                    || (ag.getLeft() < l/2))
                    || (ag.getTop() < t - GraphOrganizer.SPACE) || (ag.getTop() > b + GraphOrganizer.SPACE)) {
                    if (nw >= l) {
                        ag.moveTo(ag.getLeft(), t + (b - t)/2);
                    } else {
                        ag.moveTo(l - ag.getWidth() - GraphOrganizer.SPACE, t + (b - t)/2);
                    }
                }
            }
        }
    }
    
    /** selects the agents that the organizer knows how to place
     * and returns the ones he can't
     * @param Vector<Object> of the agents it should organize
     * @return Vector<Object> of the agents he couldn't organize
     */
    public AgentVector selectAgents(AgentVector grp) {
        this.selectedAgents = new AgentVector();
        for (int i = 0; i < grp.size(); i++) {
            Agent ag = grp.agent(i);
            // selects all the agents being only source or only target
            if (this.toolManager.linkManager.isSource(ag)) {
                this.selectedAgents.add(ag);
                // prints the name of the agent selected on the standard output
               log.println("LinksOrganizer : S?lection de " + ag.label);
                grp.remove(ag);
                i--;
            }
        }
        return(grp);
    }
    
    /** Return true if the agent given in parameter is well placed
     * @param Agent to check the position
     * @return True if the agent is in good place, false otherwise
     */
    protected boolean isWellPlaced(Agent ag) {
        return(true);
    }
    
    /** Arrange the agent given in parameter
     * @param Agent to arrange
     */
    protected void arrangeAgent(Agent ag) {
    }
    
    /** Returns the bouding rect in wich the agent must contain
     */
    protected Rect getBoundRect(Agent ag) {
        return(null);
    }
    
}
