/*
 * OrganizationManager.java
 *
 * Created on April 26, 2002, 10:45 AM
 */

package epe.observationTools.graphLayoutTool.organizers;

import java.util.Vector;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

/**
 *
 * @author  hofmann
 */
public class OrganizationManager {
    
    /** Tool interface
     */
    public final ToolManager toolManager;
    
    /** GraphOrganizers who will organize the agents
     */
    private Vector<Object> organizers;
    
    /** SpacingManager moving the agents appart from each others
     */
    private SpacingManager spacingManager;
    
    /** Tells the organization manager if the agents or the links have been modified
     */
    public boolean modified = true;
    
    private GraphLayoutToolLogOutput log;
    
    /** Creates a new instance of OrganizationManager */
    public OrganizationManager(ToolManager tm, GraphLayoutToolLogOutput log) {
      this.log = log;
       this.toolManager = tm;
       this.organizers = new Vector<Object>();
       this.spacingManager = new SpacingManager(tm);
    }
    
    /** adds a GraphOrganizer to organize the graph
     * @param GraphOrganizer to add
     */
    public void addGraphOrganizer(GraphOrganizer go) {
        this.organizers.add(go);
    }
    
    /** Organize the graph
     */
    public double organize() {
        int i;
        double prop;
        
        if (this.toolManager.agentManager.agents.size() == 0) {
            return(1);
        }
        // prints a message on the standard output to inform to user of the start of the reorganization
        if (this.modified) {
            log.println("------- Re-selection --------");
            this.modified = false;
            AgentVector v = new AgentVector();
            v.addAll(this.toolManager.agentManager.agents);
            // first all the organizers have to select the agents they will arrange
            for (i = 0; i < this.organizers.size(); i++) {
                v = ((GraphOrganizer)this.organizers.get(i)).selectAgents(v);
            }
        }
        prop = 0d;
        // then, they have to arrange them
        for (i = 0 ; i < this.organizers.size(); i++) {
            ((GraphOrganizer)this.organizers.get(i)).arrangeAgents();
            prop += ((GraphOrganizer)this.organizers.get(i)).getWellPlacedProportion();
        }
        prop = prop / (double)this.organizers.size();
        return(prop);
    }
    
    public int space() {
        return(this.spacingManager.arrangeAgents());
    }
}
