/*
 * GridOrganizer.java
 *
 * Created on April 26, 2002, 1:48 PM
 */

package epe.observationTools.graphLayoutTool.organizers;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentVector;
import epe.observationTools.graphLayoutTool.ui.GraphLayoutToolLogOutput;

/**
 *
 * @author  hofmann
 */
public class OutputsOrganizer extends GraphOrganizer {
    
    /* toolManager to use */
    private ToolManager toolManager;
    
    /* Output Agent */
    Agent outputAgent;
    
    private GraphLayoutToolLogOutput log;
    
    /** Creates a new instance of GridOrganizer
     * @param The toolManager to use
     */
    public OutputsOrganizer(ToolManager tm, GraphLayoutToolLogOutput log) {
        this.toolManager = tm;
        this.log = log;
    }
    
    /** arrange the agents
     * @param Level of the arrangement, the first one has level1, then 2 etc...
     */
    public void arrangeAgents() {
        // moving them
        for (int i = 0; i < this.selectedAgents.size(); i++) {
            Agent ag = this.selectedAgents.agent(i);
            if ((ag.getLeft() > this.outputAgent.getLeft() - ag.getWidth() - GraphOrganizer.SPACE)
                || (ag.getLeft() < this.outputAgent.getLeft() * 3/4)
                || (ag.getTop() < this.outputAgent.getTop() * (i + 1)/(this.selectedAgents.size() + 1) - GraphOrganizer.SPACE * 2)
                || (ag.getTop() > this.outputAgent.getTop() * (i + 1)/(this.selectedAgents.size() + 1) + GraphOrganizer.SPACE * 2)) {
                    ag.moveTo(this.outputAgent.getLeft() - ag.getWidth() - GraphOrganizer.SPACE * 2, this.outputAgent.getTop() * (i + 1)/(this.selectedAgents.size() + 1));
                /*} else {
                    ag.moveTo(this.outputAgent.getLeft() - ag.getWidth() - GraphOrganizer.SPACE * 2, ag.getTop());
                }*/
            }
        }
    }
    
    /** selects the agents that the organizer knows how to place
     * and returns the ones he can't
     * @param Vector<Object> of the agents it should organize
     * @return Vector<Object> of the agents he couldn't organize
     */
    public AgentVector selectAgents(AgentVector grp) {
        this.selectedAgents = new AgentVector();
        outputAgent = null;
        for (int i = 0; i < this.toolManager.agentManager.agents.size(); i++) {
            if (this.toolManager.agentManager.get(i).type == Agent.OUTPUT) {
                outputAgent = this.toolManager.agentManager.get(i);
                break;
            }
        }
        if (outputAgent != null) {
            for (int i = 0; i < grp.size(); i++) {
                Agent ag = grp.agent(i);
                // selects the agents having a link with the output agent
                if (this.toolManager.linkManager.links.indexOf(ag.label, outputAgent.label) > -1) {
                    this.selectedAgents.add(ag);
                    // prints the name of the selected agent on the standard output
                    log.println("OutputsOrganizer : S?lection de " + ag.label);
                }
            }
        }
        return(grp);
    }
    
    /** Return true if the agent given in parameter is well placed
     * @param Agent to check the position
     * @return True if the agent is in good place, false otherwise
     */
    protected boolean isWellPlaced(Agent ag) {
        //int i = this.selectedAgents.indexOf(ag);
        /*if ((ag.getLeft() >= this.outputAgent.getLeft() - ag.getWidth() - GraphOrganizer.SPACE)
            || (ag.getLeft() <= this.outputAgent.getLeft() * 3/4)
            || (ag.getTop() <= this.outputAgent.getTop() * (i + 1)/(this.selectedAgents.size() + 1) - GraphOrganizer.SPACE * 2)
            || (ag.getTop() >= this.outputAgent.getTop() * (i + 1)/(this.selectedAgents.size() + 1) + GraphOrganizer.SPACE * 2)) {
            return(false);
        }*/
        return(true);
    }
    
    /** Arrange the agent given in parameter
     * @param Agent to arrange
     */
    protected void arrangeAgent(Agent ag) {
    }
    
    /** Returns the bouding rect in wich the agent must contain
     */
    protected Rect getBoundRect(Agent ag) {
        return(null);
    }
    
}
