/*
 * PanelListener.java
 *
 * Created on May 16, 2002, 2:55 PM
 */

package epe.observationTools.graphLayoutTool.ui;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPopupMenu;

import epe.observationTools.graphLayoutTool.ToolManager;
import epe.observationTools.graphLayoutTool.agents.Agent;
import epe.observationTools.graphLayoutTool.agents.AgentManager;

/**
 *
 * @author  hofmann
 */
public class PanelListener implements ComponentListener, MouseListener, MouseMotionListener, ActionListener {
  
  /* offset of the click on the pick */
  private int xOffset;
  private int yOffset;
  /* last mouse mouve event */
  public MouseEvent me;
  
  /** ToolManager to use
   */
  private ToolManager toolManager;
  
  /** Popup menu with options
   */
  private JPopupMenu menu;
  
  /** MenuItem to stop the animation
   */
  private JCheckBoxMenuItem menuAnimating;
  private JCheckBoxMenuItem menuOrganizing;
  private JCheckBoxMenuItem menuHighlightWellPlaced;
  private JCheckBoxMenuItem menuHighlightRelated;
  private JCheckBoxMenuItem menuShowWellPlacedPercentage;
  private JCheckBoxMenuItem menuShowLog;

  private GraphLayoutToolLogOutput log;
  
  /** Creates a new instance of PanelListener */
  public PanelListener(ToolManager tm, GraphLayoutToolLogOutput log) {
    this.toolManager = tm;
    this.log = log;
    this.menu = new JPopupMenu();
    this.menuAnimating = new JCheckBoxMenuItem("Animer",true);
    this.menuAnimating.addActionListener(this);
    this.menu.add(this.menuAnimating);
    this.menuOrganizing = new JCheckBoxMenuItem("Organiser",true);
    this.menuOrganizing.addActionListener(this);
    this.menu.add(this.menuOrganizing);
    this.menuHighlightWellPlaced = new JCheckBoxMenuItem("Indiquer les agents bien plac�s",true);
    this.menuHighlightWellPlaced.addActionListener(this);
    this.menu.add(this.menuHighlightWellPlaced);
    this.menuHighlightRelated = new JCheckBoxMenuItem("Indiquer les agents reli�s",true);
    this.menuHighlightRelated.addActionListener(this);
    this.menu.add(this.menuHighlightRelated);
    this.menuShowWellPlacedPercentage = new JCheckBoxMenuItem("Afficher le pourcentage",true);
    this.menuShowWellPlacedPercentage.addActionListener(this);
    this.menu.add(this.menuShowWellPlacedPercentage);
    this.menuShowLog = new JCheckBoxMenuItem("Afficher le log",true);
    this.menuShowLog.addActionListener(this);
    this.menu.add(this.menuShowLog);
  }
  
  public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
    JCheckBoxMenuItem item = (JCheckBoxMenuItem)actionEvent.getSource();
    if (item == this.menuOrganizing) {
      this.toolManager.organizing = item.getState();
    } else if (item == this.menuAnimating) {
      this.toolManager.animating = item.getState();
    } else if (item == this.menuHighlightWellPlaced) {
      this.toolManager.agentManager.highlightWellPlaced = item.getState();
    } else if (item == this.menuHighlightRelated) {
      this.toolManager.agentManager.highlightRelated = item.getState();
    } else if (item == this.menuShowWellPlacedPercentage) {
      this.toolManager.showWellPlacedPercentage = item.getState();
    } else if (item == this.menuShowLog) {
      this.toolManager.showLog = item.getState();
      log.setVisible(true);
    }
    this.toolManager.start();
  }
  
  public void componentHidden(java.awt.event.ComponentEvent componentEvent) {
  }
  
  public void componentMoved(java.awt.event.ComponentEvent componentEvent) {
  }
  
  public void componentResized(java.awt.event.ComponentEvent componentEvent) {
    Dimension d = this.toolManager.panel.getSize();
    // if the size changed or on first time, recreate the buffer
    if ((this.toolManager.panel.buffer == null) 
     || (d.width != this.toolManager.panel.bufferSize.width) 
     || (d.height != this.toolManager.panel.bufferSize.height)) {
      this.toolManager.panel.buffer = this.toolManager.panel.createImage(d.width, d.height);
      this.toolManager.panel.bufferSize = d;
      this.toolManager.agentManager.dimension = d;
      if (this.toolManager.panel.bufferGraphics != null) {
        this.toolManager.panel.bufferGraphics.dispose();
      }
      this.toolManager.panel.bufferGraphics = this.toolManager.panel.buffer.getGraphics();
      this.toolManager.panel.bufferGraphics.setFont(this.toolManager.panel.getFont());
      this.toolManager.agentManager.fontMetrics = this.toolManager.panel.bufferGraphics.getFontMetrics();
    }
  }
  
  public void componentShown(java.awt.event.ComponentEvent componentEvent) {
  }
  
  public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
  }
  
  public void mouseDragged(java.awt.event.MouseEvent e) {
    // move the agent picked
    if (AgentManager.picked != null) {
      AgentManager.picked.setLeft(e.getX() - this.xOffset);
      AgentManager.picked.setTop(e.getY() - this.yOffset);
      // repaint the graph
      e.getComponent().repaint();
    }
    e.consume();
  }
  
  public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
  }
  
  public void mouseExited(java.awt.event.MouseEvent e) {
    // the highlights must be removed
    if (AgentManager.picked == null) {
      AgentManager.pointed = null;
      this.me = null;
      this.toolManager.linkManager.setPointed(null);
      // repaint the graph
      e.getComponent().repaint();
    }
  }
  
  public void mouseMoved(java.awt.event.MouseEvent e) {
    int x = e.getX();
    int y = e.getY();
    this.me = new MouseEvent(e.getComponent(),e.getID(),e.getWhen(),e.getModifiers(),e.getX(),e.getY(),e.getClickCount(),false);
    // highlighting the agents if needed
    Agent p = this.toolManager.agentManager.getAgentFromPoint(x, y);
    this.toolManager.linkManager.setPointed(p);
            /*if (this.pointed != p) {
                this.pointed = p;
                if (p != null) {
                    for (int i = 0; i < this.agentManager.agents.size(); i++) {
                        Agent ag = this.agentManager.agents.getAgent(i);
                        if ((this.linkManager.find(this.pointed,ag) > -1) ||
                            (this.linkManager.find(ag,this.pointed) > -1)) {
                                ag.highlight = true;
                        } else {
                            ag.highlight = false;
                        }
                    }
                } else {
                    for (int i = 0; i < this.agentManager.agents.size(); i++) {
                        this.agentManager.agents.getAgent(i).highlight = false;
                    }
                }
            }*/
    // repaint the graph
    e.getComponent().repaint();
    e.consume();
  }
  
  public void mousePressed(java.awt.event.MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON1) {
      int x = e.getX();
      int y = e.getY();
      AgentManager.picked = this.toolManager.agentManager.getAgentFromPoint(x, y);
      if (AgentManager.picked != null) {
        this.xOffset = x - AgentManager.picked.getLeft();
        this.yOffset = y - AgentManager.picked.getTop();
      }
    }
    e.consume();
  }
  
  public void mouseReleased(java.awt.event.MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON1) {
                /*if (Agent.picked != null) {
                    if ((System.currentTimeMillis() - this.clickTime)  < 250) {
                        Agent.picked = null;
                        for (int i = 0; i < this.agentManager.agents.size(); i++) {
                            this.agentManager.agents.getAgent(i).highlight = false;
                        }
                    } else {
                        Agent.picked.setLeft(e.getX() - this.xOffset);
                        Agent.picked.setTop(e.getY() - this.yOffset);*/
      AgentManager.picked = null;
                    /*}
                }
                // repaint the graph
                e.getComponent().repaint();*/
    } else {
      this.menuAnimating.setSelected(this.toolManager.animating);
      this.menuOrganizing.setSelected(this.toolManager.organizing);
      this.menuHighlightRelated.setSelected(this.toolManager.agentManager.highlightRelated);
      this.menuHighlightWellPlaced.setSelected(this.toolManager.agentManager.highlightWellPlaced);
      this.menuShowWellPlacedPercentage.setSelected(this.toolManager.showWellPlacedPercentage);
      this.menuShowLog.setSelected(this.toolManager.showLog);
      this.menuAnimating.setState(this.toolManager.animating);
      this.menuOrganizing.setState(this.toolManager.organizing);
      this.menuHighlightRelated.setState(this.toolManager.agentManager.highlightRelated);
      this.menuHighlightWellPlaced.setState(this.toolManager.agentManager.highlightWellPlaced);
      this.menuShowWellPlacedPercentage.setState(this.toolManager.showWellPlacedPercentage);
      this.menuShowLog.setState(this.toolManager.showLog);
      this.menu.show(this.toolManager.panel, e.getX(), e.getY());
    }
    e.consume();
  }
  
}
