package epe.reorganization;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.AcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.messages.contract.RequestingCooperationMessage;
	import epe.communication.messages.information.CyclePingMessage;
	import epe.communication.messages.ncs.NCSInformedNeedInMessage;
	import epe.communication.messages.ncs.NCSNeedInMessage;
	import epe.communication.messages.ncs.NCSNeedOutMessage;
	import epe.reorganization.bestOfWorseManagement.WaitForProposalsTimerManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintance;
	import java.util.Vector;

public class ReorganizationEngine {
	
		protected ReorganizationManager owner;
		protected WaitForProposalsTimerManager waitForProposalsTimerManager;
		protected Agent ownerAgent;
		protected GUI gui;
		
		// For the cycle detection : set by CyclingPingMessageProcessor if loop detected
		boolean cycleDetected = false;
		
	public ReorganizationEngine(ReorganizationManager owner,
															WaitForProposalsTimerManager waitManager) {
		this.owner = owner;
		waitForProposalsTimerManager = waitManager;
		ownerAgent = owner.ownerAgent;
		gui = owner.gui;
	}

	public void manageNCSs() {
		owner.detectInOutNCSs();
		if (!ownerAgent.inCooperativeState()) {
		  if (owner.firedNcsTimerRanOut || owner.firedRequestTimerRanOut)
				solveInOutNCSs();
		  else
				gui.println("Not ready to solve NCSs", java.awt.Color.magenta);
		}
		else {
		  if (owner.managingUselessnessForTheFirstTime ) {
				owner.managingUselessnessForTheFirstTime = false;
				owner.fireUselessness();
		  }
    
			// UselessnessNCSs solving	
		  owner.detectUselessnessNCSs();
		  if ((! owner.isUsefull()) & owner.uselessnessTimerRanOut) {
			// ###LOW_DEBUG : uselessnessNCS
			//SystemPrinter.printSystem(ownerAgent.getAgentName() + " : DEBUG : Solving uselessness");
			//solveUselessnessNCSs();
		  }
		  /*
		  //###LOW_DEBUG : uselessnessNCS
		  if (!ownerAgent.longTimeNotActed()) 
			gui.println("longTimeNotActed == false", java.awt.Color.red);
		  else
			gui.println("longTimeNotActed == true", java.awt.Color.red);
		  if (!ownerAgent.blockingSituationNeedsSolving()) 
			gui.println("blockingSituationNeedsSolving == false", java.awt.Color.red);
		  else
			gui.println("blockingSituationNeedsSolving == true", java.awt.Color.red);
		  */
	  
		  // BlockingSituationNCSs solving
		  if (ownerAgent.blockingSituationNeedsSolving()) {
				if (ownerAgent.longTimeNotActed()) {
				  solveBlockingSituation();
				}
				ownerAgent.setBlockingSituationNeedsSolving(false);
		  }
		  
		  // waitForProposalsTimerRanOutSNC
		  for (int i=1; i<=owner.numberOfINs(); i++) {
			  if ((waitForProposalsTimerManager.mustChooseProposal(i))
			  	 && (!waitForProposalsTimerManager.managedToChooseProposal(i))) {
			  	solveWaitForProposalsTimerRanOutNcs(i);
			  }
		  }
		  
		  // If not goalReached => coninue the Higher/Lower Search
		  if ((owner.isInHigherLowerReorganization()) 
		  	 && (!owner.isGoalReached())) {
		  	solveGoalNotReachedNcs();
		  }
		  
		}
	}

	protected void solveInOutNCSs() {
	  // Tries to find INs and OUTs + transmits the NCS to the other agents
	  ownerAgent.setActed(false);
	  gui.println("Trying to solve InOutNCSs", java.awt.Color.magenta);
	  if (!(owner.needIN() | owner.needOUT()))
			gui.println("No InOutNCSs to solve", java.awt.Color.magenta);
	  else {
	  	solveInNCS();
	  	solveOutNCS();
	  }
	}
	
	protected void solveInNCS() {
		for (int i=1; i<=owner.numberOfINs(); i++) {
			//gui.println("Examining IN" + i, java.awt.Color.magenta);
			if (owner.needIN(i)) {
			  String dataType = owner.dataTypeNeeded("IN", i);
			  AgentInterface interfaceType = new AgentInterface("IN", i, dataType);
			  if (!(owner.potentialInOutAcquaintances
								 	 .knowingPotentialInOutAcquaintances(interfaceType))) {
			  	// The agent doesn't know any potential agent
			  	solveInNCSWithoutPotentialAcquaintances(interfaceType, dataType, i);
			  }
			  else {
			  	// The agent knows some potential agents
				  solveInNCSWithPotentialAcquaintances(interfaceType, dataType, i);
			  }
			}
	  }
	}
		
	protected void solveInNCSWithoutPotentialAcquaintances(AgentInterface interfaceType,
														   													 String dataType,
														   													 int position) {
		gui.println("Don't know any potential [IN:"
								+ interfaceType.getPosition() + ":"
								+ interfaceType.getDataType() + "]", java.awt.Color.magenta);
		owner.showAllPotentialAcquaintances();
		if (owner.readyToFireNCSs()) {
			if (!owner.isInHigherLowerReorganization()) {
				NCSNeedInMessage message = new NCSNeedInMessage(ownerAgent, dataType, position);
				ownerAgent.fireMessageToAllAcquaintances(message);
				// Decrease its links : self-doubt
				ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
			}
			else { // inHigherLowerReorganization
				// Change strategy if no one is answering : normal needIn
				if (owner.numberOfNeedInHLNCSWithoutAnswer
						>= owner.maxNumberOfNeedInHLNCSWithoutAnswer) {
					owner.reInitializenumberOfNeedInHLNCSWithoutAnswer();
					NCSNeedInMessage message = new NCSNeedInMessage(ownerAgent,dataType, position);
					ownerAgent.fireMessageToAllAcquaintances(message);
					// Decrease its links : self-doubt
					ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
				}
				else {
					if (owner.getHigherLowerGoal().equals("HIGHER")) {
						NCSInformedNeedInMessage message 
						= new NCSInformedNeedInMessage(ownerAgent,
																					 "HIGHER",
																					 owner.dataTypeNeeded("IN", position), 
																					 position,
																					 owner.getInputThreshold(position),
																					 owner.getCurrentReorganizationPhase());
						gui.println("InformedNeedIn[HIGHER]: InputThreshold = "
												+ owner.getInputThreshold(position), java.awt.Color.orange);
						ownerAgent.fireMessageToAllAcquaintances(message);
					}
					else if (owner.getHigherLowerGoal().equals("LOWER")) {
						NCSInformedNeedInMessage message 
						= new NCSInformedNeedInMessage(ownerAgent,
																					 "LOWER",
																					 owner.dataTypeNeeded("IN", position), 
																					 position,
																					 owner.getInputThreshold(position),
																					 owner.getCurrentReorganizationPhase());
						gui.println("InformedNeedIn[LOWER]: InputThreshold = "
												+ owner.getInputThreshold(position), java.awt.Color.orange);
						ownerAgent.fireMessageToAllAcquaintances(message);
					}
					else {
						// ERROR
					}
					owner.numberOfNeedInHLNCSWithoutAnswer++;
				}
			}
			owner.ncsFired();
			ownerAgent.notifyNcsListeners();
			ownerAgent.setActed(true);
		}
		else
			gui.println("Not ready to fire NCSs", java.awt.Color.magenta);
	}
	
	protected void solveInNCSWithPotentialAcquaintances(AgentInterface interfaceType,
												   											  		String dataType,
												   											  		int position) {
		gui.println("I know some potential IN", java.awt.Color.magenta);
		owner.showAllPotentialAcquaintances();
		if (!owner.requestManager.hasRequestedIN(position)) {
	  	// ###DEBUG test CyclePing clearing of potentialAcqs
	  	clearPotentialAcqsOfCycles("IN");
			PotentialInOutAcquaintance potentialInAcquaintance 
				= choosePotentialInOutAcquaintance(interfaceType);
			if (potentialInAcquaintance != null) {
				Agent potentialInAgent = potentialInAcquaintance.getAgent();
				gui.println("Chosen potentialAcq : " 
										+ potentialInAgent.getAgentName(), java.awt.Color.magenta);
				owner.potentialInOutAcquaintances.setRequested(potentialInAgent,interfaceType, true);
				owner.potentialInOutAcquaintances.setNewDataType(potentialInAgent, 
																												 interfaceType, dataType);
				owner.requestManager.setRequestedIN(position);
				RequestingCooperationMessage message 
					= new RequestingCooperationMessage(ownerAgent, interfaceType, 
																						 owner.getPotentialInput(position));
				ownerAgent.fireMessage(message,potentialInAgent, "NEUTRAL", ownerAgent);
				owner.requestFired();
				ownerAgent.setActed(true);
			}
			else
				gui.println("ChoosePotentialAcqIN failled.", java.awt.Color.magenta);
		}
		else
			gui.println(ownerAgent.getAgentName() 
									+ " has already requested some agent[IN"
									+ position + "] and is waiting for an answer.", java.awt.Color.magenta);
	}
	
 	protected void solveOutNCS() {
		if (owner.needOUT() && (!owner.isInHigherLowerReorganization() 
														|| owner.possessPotentialOutput())) {
			String dataType = owner.dataTypeNeeded("OUT", 1);
			AgentInterface interfaceType = new AgentInterface("OUT", 1, dataType);
			if (!(owner.potentialInOutAcquaintances
								 .knowingPotentialInOutAcquaintances(interfaceType))) {
				// The agent doesn't know any potential agent
			  SolveOutNCSWithoutPotentialAcquaintances(interfaceType);
			}
			else { 
				// The agent knows some potential agents
			  solveOutNCSWithPotentialAcquaintances(interfaceType, dataType);
			}
	  }
 	}
 
 	protected void SolveOutNCSWithoutPotentialAcquaintances(AgentInterface interfaceType) {
 		gui.println("Don't know any potential [OUT:"
 								+ interfaceType.getPosition() + ":"
 								+ interfaceType.getDataType() + "]", java.awt.Color.magenta);
 		owner.showAllPotentialAcquaintances();
 		if (owner.readyToFireNCSs()) {
 			ownerAgent.notifyNcsListeners();
 			NCSNeedOutMessage message = new NCSNeedOutMessage(ownerAgent, 
																							 					interfaceType, 
																							 					owner.getPotentialOutput());
 			gui.println("NeedOut : PotentialOutput = "
 									+  owner.getPotentialOutput(), java.awt.Color.orange);
 			ownerAgent.fireMessageToAllAcquaintances(message);
 			owner.ncsFired();
 			ownerAgent.notifyNcsListeners();
 			// Decrease its links : self-doubt
 			if (!owner.isInHigherLowerReorganization())
 				ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
 			ownerAgent.setActed(true);
 		}
 		else
 			gui.println("Not ready to fire NCSs", java.awt.Color.magenta);
 	}
 	
 	protected void solveOutNCSWithPotentialAcquaintances(AgentInterface interfaceType, 
														 													 String dataType) {
 		gui.println("I know some potential OUT", java.awt.Color.magenta);
 		owner.showAllPotentialAcquaintances();
 		if (!owner.requestManager.hasAlreadyRequestedOUT()) {
			// ###DEBUG test CyclePing clearing of potentialAcqs
		  clearPotentialAcqsOfCycles("OUT");
 			PotentialInOutAcquaintance potentialOutAcquaintance 
 				= choosePotentialInOutAcquaintance(interfaceType);
 			if (potentialOutAcquaintance != null) {
 				Agent potentialOutAgent = potentialOutAcquaintance.getAgent();
 				gui.println("Chosen potentialAcq : " 
 										+ potentialOutAgent.getAgentName(), java.awt.Color.magenta);
 				owner.potentialInOutAcquaintances.setRequested(potentialOutAgent, interfaceType, true);
 				owner.potentialInOutAcquaintances.setNewDataType(potentialOutAgent, 
 																												 interfaceType, dataType);
 				owner.requestManager.setRequestedOUT();
 				int proposedINnumber = potentialOutAcquaintance.getProposedINnumber();
 				RequestingCooperationMessage message 
 					= new RequestingCooperationMessage(ownerAgent, interfaceType,
 																						 proposedINnumber, owner.getPotentialOutput());
 				ownerAgent.fireMessage(message,potentialOutAgent, "NEUTRAL", ownerAgent);
 				owner.requestFired();
 				ownerAgent.setActed(true);
 			}
 			else
 				gui.println("ChoosePotentialAcqOUT failed.", java.awt.Color.magenta);
 		}
 		else
 			gui.println(ownerAgent.getAgentName() 
 									+ " has already requested some agent[OUT] and is waiting for an answer.",
 									java.awt.Color.magenta);
 	}
 	
	protected void solveUselessnessNCSs() {
	  ownerAgent.getAcquaintancesManager()
	  					.decreaseTrustValue(new AgentInterface("OUT"), 
																										 "uselessness");
	}

	protected void clearPotentialAcqsOfCycles(String direction) {
		if (owner.potentialInOutAcquaintances.size() > 0) {
			int i = 0;
			PotentialInOutAcquaintance testedAcq;
			testedAcq = owner.potentialInOutAcquaintances.getPotentialInOutAcquaintance(i);

			// send CyclePing to check if this link might produce a cycle
			gui.println("DEBUG : direction = " + direction, GUI.red);
			gui.println("DEBUG : i = " + i, GUI.red);
			gui.println("DEBUG : testing : " + testedAcq.getAgent().getAgentName(), GUI.red);
			if (testedAcq.getInterfaceType().equals(direction)) {
				checkCycle(direction, testedAcq.getAgent());
			}
			else
				gui.println("DEBUG : wrong direction = " + testedAcq.getInterfaceType() , GUI.red);
		  
		  PotentialInOutAcquaintance acq;
			while (i<owner.potentialInOutAcquaintances.size()) {
				acq = owner.potentialInOutAcquaintances.getPotentialInOutAcquaintance(i);
				gui.println("DEBUG : i (in loop) = " + i, GUI.red);
				gui.println("DEBUG : testing (in loop) : " + acq.getAgent().getAgentName(), GUI.red);
				if (acq.equals(testedAcq)) {
					gui.println("DEBUG : allready tested", GUI.red);
					// allready checked and no cycle
					i++;
				}
				else {
					// send CyclePing to check if this link might produce a cycle
					gui.println("DEBUG : direction(in loop) = " + direction, GUI.red);
					if (acq.getInterfaceType().equals(direction)) {
						checkCycle(direction, acq.getAgent());
					}
					else
						gui.println("DEBUG : wrong direction(in loop) = " + acq.getInterfaceType() , GUI.red);
				  testedAcq = acq;
				}
			}
		}
	}
	
	public void setCycleDetected(boolean cycleDetected) {
		this.cycleDetected = cycleDetected;
	}
	
	public boolean checkCycle(String direction, Agent testedAgent) { 
		setCycleDetected(false);
	  // ###DEBUG : test CyclePing
	  gui.println("Send Cycle Ping", java.awt.Color.red);
		ownerAgent.fireMessage(new CyclePingMessage(direction, testedAgent, 
	  																						ownerAgent, ownerAgent),
	  											 testedAgent, "NEUTRAL", ownerAgent);
	  // This should remove the agent from the potentialAcqs if there exist a cycle
	  return cycleDetected;
	}
	
	public void solveBlockingSituation() {
	  gui.println("Solving Blocking Situation", java.awt.Color.pink);
	  ownerAgent.notifyNcsListeners("NcsBlockingSituationRetryCalculus");
	  owner.solveBlockingSituationCounter--;
	  ownerAgent.retryCalculus();
	  if (owner.solveBlockingSituationCounter == 0) {
			ownerAgent.notifyNcsListeners("NcsBlockingSituationDeceaseTrustvalue");
			owner.solveBlockingSituationCounter = owner.solveBlockingSituationCounterInit;
			ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("blockingSituation");
	  }
	  ownerAgent.setActed(true);
	}

	protected void solveWaitForProposalsTimerRanOutNcs(int pos) {
		// Remove output :
		AcquaintancesManager acqsManager = ownerAgent.getAcquaintancesManager(); 
		if (acqsManager.possessingOUT())
			acqsManager.cancelOutAcquaintances();
		
		// Choose bestOfWorse
		waitForProposalsTimerManager.chooseBestOfWorseProposal(pos);
	}
	
	protected void solveGoalNotReachedNcs() {
		if (owner.readyToFireNCSs()) {
  		for (int i = 1; i <= ownerAgent.getNumberOfINs(); i++) {
				String information = owner.getHigherLowerGoal();
				gui.println("GoalNotReached : Sending NCSInformedNeedIn[" + information + 
			  						"][IN" + i + " : " +
			  						owner.getInputThreshold(i) + 
			  						"] to all acquaintances.", java.awt.Color.orange);
				NCSInformedNeedInMessage message 
					= new NCSInformedNeedInMessage(
									ownerAgent, information,
									owner.dataTypeNeeded("IN", i), 
									i, owner.getInputThreshold(i),
									owner.getCurrentReorganizationPhase());
				ownerAgent.fireMessageToAllAcquaintances(message);
				
				// Launch waitingForProposalsTimer
				waitForProposalsTimerManager.waitForProposals(i);
				
				owner.ncsFired();
				ownerAgent.notifyNcsListeners();
  		}
		}
	}
	
	protected PotentialInOutAcquaintance choosePotentialInOutAcquaintance(AgentInterface type) {
	  // Chooses among the potential IO agents
	  
		// Select relevant acqs
		Vector acqs = new java.util.Vector();
	  acqs = selectRelevantPotentialAcquaitances(type);
	  gui.println("DEBUG : Number of selected potentialAcqs :" 
								+ acqs.size(), java.awt.Color.cyan);
	  
	  if (acqs.size()!=0) {
		  // Choose the best acq
	  	PotentialInOutAcquaintance acq;
		  if (owner.isInHigherLowerReorganization()
		  		&& (type.getType().equals("OUT"))) {
		  	acq = chooseBestPotentialOUTWithPrejudice(acqs, type);
		  	gui.println("DEBUG : Chosen potentialAcq :" + 
										acq.getAgent().getAgentName(), java.awt.Color.cyan);
		  }
		  else {
		  	acq =chooseBestPotentialAcquaintanceDependingOnTrustvalue(acqs, type);
		  	gui.println("DEBUG : Chosen potentialAcq :" + 
										acq.getAgent().getAgentName(), java.awt.Color.cyan);
		  }
		  return acq;
	  } else {
			// If it doesn't find any potentialAcq
			gui.println("WARNING(choosePotentialAcquaintance) : " + ownerAgent.getAgentName() 
									+ " : no free PotentialAcquaintance found matching the type: ["
									+ type.getType() + " : "
									+ type.getPosition() + " : "
									+ type.getDataType()
									+ "]", java.awt.Color.magenta);
			return null;
		}
	}

	protected Vector<Object> selectRelevantPotentialAcquaitances(AgentInterface type) {
		int i = 0;
		PotentialInOutAcquaintance acq;
		Vector<Object> acqs = new java.util.Vector<Object>();
		while (i<owner.potentialInOutAcquaintances.size()) {
			acq = owner.potentialInOutAcquaintances.getPotentialInOutAcquaintance(i);
			
			if ((ownerAgent.getAcquaintancesManager().isAnAcquaintance(acq.getAgent()))
					&& (acq.getInterface().equals(type))) {
				if (acq.isRequested()) {
					gui.println("Examining potentialAcq [" + acq.getAgent().getAgentName() + 
											"] : already requested.", java.awt.Color.magenta);
					i++;
				}
				else if ((type.getType().equals("IN"))
						&&(owner.potentialInOutAcquaintances.isAlreadyRequestedAsIN(acq.getAgent()))) {
					gui.println("Examining potentialAcq [" + acq.getAgent().getAgentName() +
											"] : already requested as IN.", java.awt.Color.magenta);
					i++;
				}
     		// ###DEBUG Prevent InputOuput crossing (but no good for AgentFirstTime)
				else if ((ownerAgent.getAcquaintancesManager()
													  .isAnInOutAcquaintance(acq.getAgent(), "IN"))
								|| (ownerAgent.getAcquaintancesManager()
													 		.isAnInOutAcquaintance(acq.getAgent(), "OUT"))
								|| (isAgentAllreadyRequested(acq.getAgent()))) {
					gui.println("Examining potentialAcq [" + acq.getAgent().getAgentName() + 
											"[ : already IN or OUT.", java.awt.Color.magenta);
					owner.potentialInOutAcquaintances
							 .removePotentialInOutAcquaintance(acq.getAgent(), type);
				}
				else {
					acqs.addElement(acq);
					i++;
				}
			} else
				i++;
		}
		return acqs;
	}
	
	private boolean isAgentAllreadyRequested(Agent agent) {
		PotentialInOutAcquaintance acq;
		for (int i = 0; i < owner.potentialInOutAcquaintances.size(); i++) {
			acq = owner.potentialInOutAcquaintances.getPotentialInOutAcquaintance(i);
			if ((acq.getAgent().equals(agent))
					&& (acq.isRequested()))
				return true;
		}
		return false;
			
	}

	// ###DEBUG : choosePotentialAcq upgrading. Present time : depending on the trustValue
	// Note : choose first a potenial acq wich is also in need 
	// (ensures a complete organization is found quickly)
	protected PotentialInOutAcquaintance chooseBestPotentialAcquaintanceDependingOnTrustvalue(
							Vector acqs, AgentInterface type) {
		// check if a potentailAcq is also in need
		PotentialInOutAcquaintance acq;
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			if (acq.isInNeedInOutNcs())
				return acq;
		}
		
		// Choose depending on trustValue
		gui.println("DEBUG init choose depending on trustValue", java.awt.Color.cyan);
		PotentialInOutAcquaintance acq1;
		PotentialInOutAcquaintance acq2;
		int acq1TrustValue;
		int acq2TrustValue;
		gui.println("DEBUG getting acq1", java.awt.Color.cyan);
		acq1 = (PotentialInOutAcquaintance)acqs.elementAt(0);
		gui.println("DEBUG getting trustvalue", java.awt.Color.cyan);
		acq1TrustValue = ownerAgent.getAcquaintancesManager()
															 .getAcquaintances()
															 .getAcquaintance(acq1.getAgent(), "NEUTRAL")
															 .getTrustValue().getValue();
		gui.println("DEBUG For", java.awt.Color.cyan);
		for (int j=1 ; j<acqs.size(); j++) {
			acq2 = (PotentialInOutAcquaintance)acqs.elementAt(j);
			acq2TrustValue = ownerAgent.getAcquaintancesManager()
																 .getAcquaintances()
																 .getAcquaintance(acq2.getAgent(), "NEUTRAL")
																 .getTrustValue().getValue();
			gui.println("DEBUG trustValue acq1 = " + acq1TrustValue, java.awt.Color.cyan);
			gui.println("DEBUG trustValue acq2 = " + acq2TrustValue, java.awt.Color.cyan);
			if (acq2TrustValue > acq1TrustValue)
				acq1 = acq2;
		}
		gui.println("DEBUG : Chosen potentialAcq :" + 
								acq1.getAgent().getAgentName(), java.awt.Color.cyan);
		return acq1;
	}

	protected PotentialInOutAcquaintance chooseBestPotentialOUTWithPrejudice(
							Vector acqs, AgentInterface type) {
		gui.println("Choose acq considering prejudice and multiplicity.", java.awt.Color.cyan);
		// Priority strategy : 
		// 		priority-0 : acq also in need
		// 		priority-1 : 0-prej, Multiple
		// 		priority-2 : prej, Multiple
		// 		priority-3 : 0-prej, unique
		// 		priority-4 : prej, Multiple
		PotentialInOutAcquaintance acq;
		int prejudice;
		// P0 : Search for an Acq which is also in need 
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			if (prejudice == -1) {
				gui.println("Found agent in need : Prejudice = " + prejudice, java.awt.Color.cyan);
				return acq;
			}
		}
		
		// P1 : else search for 0-prejudice and Multiple
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			String multiplicity = acq.getMultiplicity();
			if ((prejudice == 0) && (multiplicity.equals(Acquaintance.MULTIPLE))) {
				// Exception case
				if ((acq.isAnInputThresholdReceiverFromMe())
					 &&(acq.isInGoalReachedState())) {
					// Search if there exists a least prejudice and multiple which is not a thresholdAcq
					PotentialInOutAcquaintance acqAux1;
					PotentialInOutAcquaintance acqAux2;
					for (int j=0 ; j<acqs.size(); j++) {
						acqAux1 = (PotentialInOutAcquaintance)acqs.elementAt(j);
						int prejudiceAux1 = acqAux1.getPrejudice();
						String multiplicityAux1 = acqAux1.getMultiplicity();
						if ((multiplicityAux1.equals(Acquaintance.MULTIPLE))
								&& (!acqAux1.isAnInputThresholdReceiverFromMe())) {
							// ###DEBUG : test Exception case strategy (is !goalReached needed ?)
              //	 &&(!acqAux.isInGoalReachedState())) {
							for (int k=j; k<acqs.size(); k++) {
								acqAux2 = (PotentialInOutAcquaintance)acqs.elementAt(k);
								int prejudiceAux2 = acqAux2.getPrejudice();
								String multiplicityAux2 = acqAux2.getMultiplicity();
								if ((multiplicityAux2.equals(Acquaintance.MULTIPLE))
										&& (!acqAux2.isAnInputThresholdReceiverFromMe())
										&& (prejudiceAux2 < prejudiceAux1))
									acqAux1 = acqAux2;
							}
							gui.println("Found Exception Case : Prejudice = " + prejudice, java.awt.Color.cyan);
							return acqAux1;
						}
					}
				}
				// Normal Case
				gui.println("Found priority-1 : Prejudice = " + prejudice + 
										" / Multiplicity = " + multiplicity, java.awt.Color.cyan);
				return acq;
			}
		}
				
		// P2-exception case : else search for least prejudice and Multiple but
		// test for excepion case : try fist to find one with !goalReached
		// even if higher prej.
		PotentialInOutAcquaintance acqAux;
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			String multiplicity = acq.getMultiplicity();
			if ((multiplicity.equals(Acquaintance.MULTIPLE))
					&&(!acq.isInGoalReachedState())) {
				for (int j=i; j<acqs.size(); j++) {
					acqAux = (PotentialInOutAcquaintance)acqs.elementAt(j);
					int prejudiceAux = acqAux.getPrejudice();
					String multiplicityAux = acqAux.getMultiplicity();
					if ((multiplicityAux.equals(Acquaintance.MULTIPLE))
							&& (!acqAux.isInGoalReachedState())
							&& (prejudiceAux < prejudice))
						acq = acqAux;
				}
				gui.println("Found priority-2-exceptionCase : Prejudice = " + prejudice + 
										" / Multiplicity = " + multiplicity, java.awt.Color.cyan);
				return acq;
			}
		}
		
		// P2 : else search for least prejudice and Multiple
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			String multiplicity = acq.getMultiplicity();
			if (multiplicity.equals(Acquaintance.MULTIPLE)) {
		
				
				for (int j=i; j<acqs.size(); j++) {
					acqAux = (PotentialInOutAcquaintance)acqs.elementAt(j);
					int prejudiceAux = acqAux.getPrejudice();
					String multiplicityAux = acqAux.getMultiplicity();
					if ((multiplicityAux.equals(Acquaintance.MULTIPLE))
							&& (prejudiceAux < prejudice))
						acq = acqAux;
				}
				gui.println("Found priority-2 : Prejudice = " + prejudice + 
										" / Multiplicity = " + multiplicity, java.awt.Color.cyan);
				return acq;
			}
		}
		
		// P3 : else search for 0-prejudice and Unique
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			String multiplicity = acq.getMultiplicity();
			if (prejudice == 0) {
				gui.println("Found priority-3 : Prejudice = " + prejudice + 
										" / Multiplicity = " + multiplicity, java.awt.Color.cyan);
				return acq;
			}
		}
		
		// P4 : else search for least prejudice and Unique
		for (int i=0 ; i<acqs.size(); i++) {
			acq = (PotentialInOutAcquaintance)acqs.elementAt(i);
			prejudice = acq.getPrejudice();
			String multiplicity = acq.getMultiplicity();
			for (int j=i; j<acqs.size(); j++) {
				acqAux = (PotentialInOutAcquaintance)acqs.elementAt(j);
				int prejudiceAux = acqAux.getPrejudice();
				if (prejudiceAux < prejudice)
					acq = acqAux;
			}
			gui.println("Found priority-4 : Prejudice = " + prejudice + 
									" / Multiplicity = " + multiplicity, java.awt.Color.cyan);
			return acq;
		}
		// Should never happen
		return null;
	}
		
}
