package epe.reorganization;

	import java.awt.event.ActionEvent;
	import java.awt.event.ActionListener;
	import javax.swing.Timer;
	
	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.CancellingCooperationMessage;
	import epe.communication.messages.information.GoalReachedMessage;
	import epe.communication.messages.information.PotentialOutputMessage;
	import epe.communication.messages.ncs.NCSNoPotentialOutputMessage;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;
	import epe.reorganization.thresholdAcquaintances.ThresholdAcquaintance;
	import epe.reorganization.thresholdAcquaintances.ThresholdAcquaintances;
	import epe.reorganization.bestOfWorseManagement.BestOfWorseManager;
	import epe.reorganization.bestOfWorseManagement.WaitForProposalsTimerManager;
	import epe.reorganization.bestOfWorseManagement.ReorganizationLimiter;
	

public abstract class ReorganizationManager extends Object 
                                            implements ActionListener {
  
	  protected Agent ownerAgent;
	  protected ReorganizationEngine reorganizationEngine;
	  protected ReorganizationMessageProcessor messageProcessor;
	  protected PotentialInOutAcquaintances potentialInOutAcquaintances;
	  protected ThresholdAcquaintances thresholdAcquaintances;
	  protected RequestManager requestManager;
	  protected LinkMultiplicityManager linkMultiplicityManager;
	  protected GUI gui;
	  
	  protected final static int ONE_SECOND = 1000;
	  // firedNcsTimer : dictates how long an agents waits before sending again the same NCS
	  protected Timer firedNcsTimer; 
	  // firedRequestTimer : dictates how long an agent waits after requesting an agent 
	  // for cooperation  
	  protected Timer firedRequestTimer;
	  protected int firedNcsTimerDelay = 400;//1*ONE_SECOND;//400
	  protected int firedRequestTimerDelay = 1*ONE_SECOND;//5
	  protected boolean firedNcsTimerRanOut = true;
	  protected boolean firedRequestTimerRanOut = true;
	  protected int maxNumberOfRejects = 5;
	  
	  // True if the agent is (indirectly) conncected to the OutputAgent
	  protected boolean managingUselessnessForTheFirstTime = true;
	  protected boolean connectedToOutputFlag = false;
	  protected Timer uselessnessTimer;
	  protected boolean uselessnessTimerRanOut = true;
	  protected int uselessnessTimerDelay = 30*ONE_SECOND;//30
	
	  // For the management of solveBlockingSituation
	  protected int solveBlockingSituationCounterInit = 2;
	  protected int solveBlockingSituationCounter = solveBlockingSituationCounterInit;
	  
	  // For the Higher/Lower Feedback
	  
	  // reorganizationPhase : needed for initialization
	  // : is the agent entering a reorganization phase
	  protected boolean reorganizationPhase = false;
	  
	  protected int[] lastInputValue = {0, 0};
		protected int[] inputThreshold = {0, 0};
		protected int[] potentialInput = {0, 0};
		protected boolean[] possessPotentialInput = {false, false};
	  
	  protected int lastOutputValue = 0;
	  protected int outputThreshold;
	  protected int potentialOutput = 0;
	  protected boolean possessPotentialOutput = false;
	  
	  // To know if the agent tries to get a higher or a lower output value during reorganization
	  protected String higherLowerGoal = "UNKNOWN"; // "HIGHER" / "LOWER"
	  protected int currentImposingPrejudice;
	  // To know if the agent is in a classic reorganization or if it has to find Higher/lower acqs
	  protected boolean inHigherLowerReorganization = false;
	  protected boolean receivedPropositionOutFromAgentInNeed = false;
	  protected boolean receivedPropositionInFromAgentInNeed = false;
	  // Numbering of the reorganization phases (1 phase = 1 feedback from Env.)
	  protected int currentReorganizationPhase = 0;
	  protected boolean inPrejudice = false;
	  protected boolean goalReached = true;
	
	  // To know if the sending of an SNC results in some answers.
	  // Else : enable an alternative SNC
	  protected int numberOfNeedInHLNCSWithoutAnswer = 0;
	  //###DEBUG : maxNumberOfNeedInHLSNCWithoutAnswer = 10000 to disable it (not needed)
	  protected int maxNumberOfNeedInHLNCSWithoutAnswer = 10000;
	  
	  // To manage the waiting for proposals to be able to choose
	  protected WaitForProposalsTimerManager waitForProposalsTimerManager;
	  protected ReorganizationLimiter reorganizationLimiter;
	  protected BestOfWorseManager bestOfWorseManager;
	  
	  // End of Higher/Lower Feedback declarations

  public ReorganizationManager(Agent ownerAgent) {
    this.ownerAgent = ownerAgent;
    gui = ownerAgent.getGUI();
    potentialInOutAcquaintances = new PotentialInOutAcquaintances(gui, ownerAgent);
    thresholdAcquaintances = new ThresholdAcquaintances(gui, ownerAgent, this);
    bestOfWorseManager = new BestOfWorseManager(ownerAgent, this, reorganizationEngine,
    																					  potentialInOutAcquaintances, gui);
    waitForProposalsTimerManager = new WaitForProposalsTimerManager(ownerAgent, this, 
    																																bestOfWorseManager, gui);
    reorganizationLimiter = new ReorganizationLimiter(gui, waitForProposalsTimerManager);
    ownerAgent.addReorganizationListener(reorganizationLimiter);
    reorganizationEngine = new ReorganizationEngine(this, waitForProposalsTimerManager);
    linkMultiplicityManager = new LinkMultiplicityManager(this, ownerAgent, gui);
    messageProcessor = new ReorganizationMessageProcessor(this, 
																									    		reorganizationEngine,
																									    		ownerAgent, gui,
                                                          potentialInOutAcquaintances,
    																											waitForProposalsTimerManager,
														  														bestOfWorseManager,
																													linkMultiplicityManager);
    requestManager = new RequestManager(this, gui);
    firedNcsTimer = new Timer(firedNcsTimerDelay, this);
    firedNcsTimer.setRepeats(false);
    firedRequestTimer = new Timer(firedRequestTimerDelay, this);
    firedRequestTimer.setRepeats(false);
    uselessnessTimer = new Timer(uselessnessTimerDelay, this);
    uselessnessTimer.setRepeats(false);
  }

  public void reInitialize() {
    solveBlockingSituationCounter = solveBlockingSituationCounterInit;
  }
  
  protected void reInitializePotentialInOutAcquaintances() {
    potentialInOutAcquaintances.reInitializePotentialAcq();
    setReceivedPropositionOutFromAgentInNeed(false);
    setReceivedPropositionInFromAgentInNeed(false);
  }

  public ReorganizationEngine getReorganizationEngine() {
		return reorganizationEngine;
	}

  public ReorganizationLimiter getReorganizationLimiter() {
		return reorganizationLimiter;
	}
  
  public PotentialInOutAcquaintances getPotentialInOutAcquaintances() {
    return potentialInOutAcquaintances;
  }

  public void manageNCSs() {
		reorganizationEngine.manageNCSs();
  }
  
// Get and Sets section
  
  public RequestManager getRequestManager() {
    return requestManager;
  }
  
  public LinkMultiplicityManager getLinkMultiplicityManager() {
  	return linkMultiplicityManager;
  }
  
  public int getMaxNumberOfRejects() {
    return maxNumberOfRejects;
  }
  
  public void setLastInputValue(int pos, int x) {
    lastInputValue[pos-1] = x;
  }

  public int getLastInputValue(int pos) {
    return lastInputValue[pos-1];
  }
  
  public void setLastOutputValue(int x) {
    lastOutputValue = x;
  }

  public int getLastOutputValue() {
    return lastOutputValue;
  }
  
  public void setInputThreshold(int pos, int x) {
    inputThreshold[pos-1] = x;
  }
  
  public int getInputThreshold(int pos) {
    return inputThreshold[pos-1];
  }

  // ###DEBUG GoalReached : currently testing output update via potentialOutput
  // and input update via goalReachedMessage
  public void setPotentialInput(int pos, int x) {
  	if ((possessPotentialInput(pos))
  			&& (getPotentialInput(pos) == x)) {
  		// No update needed
  		gui.println("No update needed : potential IN" + pos + " = " + x);
  	}
  	else {
	  	gui.println("New potential IN" + pos + " = " + x);
	    potentialInput[pos-1] = x;
	    setPossessPotentialInput(pos, true);
	    
	    // check if possess all inputs
		  if (possessPotentialOutput())
		  	setSpecificPotentialOutput();
	    
	    if (isInHigherLowerReorganization()) {
		    //String check = checkPotentialInputsVsInputThresholds();
	    	String check = checkGoalReached();
		    if (check.equals("SUCCEEDED")) {
		    	setGoalReached(true);
			    goalReachedNotification(true, ownerAgent);
		    }
		    else if (check.equals("FAILED")){
		    	setGoalReached(false);
			    goalReachedNotification(false, ownerAgent);
		    }
		    else { // ERROR
		    	SystemPrinter.printSystem("ERROR : unknown check case.");
		    	gui.errorNotification();
		    }
	    }
  	}
    // ###DEBUG test : setPotential => always goalReached (and not only if change)
    // Check if the goalReachedStatusChanged
//    if (isInHigherLowerReorganization()) {
//	    String check = checkPotentialInputsVsInputThresholds();
//	    if ((check.equals("SUCCEEDED")) && (!isGoalReached())) {
//	    	setGoalReached(true);
//		    goalReachedNotification(true);
//	    }
//	    else if ((check.equals("FAILED")) && (isGoalReached())) {
//	    	setGoalReached(false);
//		    //goalReachedNotification(false);
//	    }
//    }
  }

  public void resetPotentialInput(int pos) {
  	gui.println("Reset potential IN" + pos);
    potentialInput[pos-1] = 0;
  }
  
  public void goalReachedNotification(boolean goalReached, Agent origin) {
  	// Notify OUT
//  	if (ownerAgent.getAcquaintancesManager().possessingOUT())
//    	ownerAgent.fireMessage(new GoalReachedMessage(ownerAgent, goalReached), 
//							        			 new AgentInterface("OUT", 1), ownerAgent);
  	// Notify INs
  	for (int i=1; i<= numberOfINs(); i++)
  		if (ownerAgent.getAcquaintancesManager().possessingIN(i))
  			ownerAgent.fireMessage(new GoalReachedMessage(ownerAgent, goalReached, origin), 
									        		 new AgentInterface("IN", i), ownerAgent);
  	
  	gui.println("GoalReached = " + goalReached + 
  							" and notification done.", java.awt.Color.red);
  }
  
  // Not used currently (checkGoalReached used instead)
  protected String checkPotentialInputsVsInputThresholds() {
  	// Check for a higher potentialInput (1 higher + 1 lower is considered a success)
  	for (int i=1; i<=numberOfINs(); i++)
  		if (isBetterForGoal(getPotentialInput(i), getInputThreshold(i)))
  			return "SUCCEEDED";
  	// exit of the loop : no higher = some lower or all the same => FAILED
  	return "FAILED";
  }
  
  public boolean isBetterForGoal(int newValue, int oldValue) {
  	if (((getHigherLowerGoal().equals("HIGHER")) 
					&& (newValue > oldValue))
			 || ((getHigherLowerGoal().equals("LOWER")) 
			 		&& (newValue < oldValue)))
  		return true;
  	else 
  		return false;
  }
  
  protected String checkGoalReached() {
  	// check against the outputThreshold
  	if (isBetterForGoal(getPotentialOutput(), getOutputThreshold()))
  			return "SUCCEEDED";
  	else
  		return "FAILED";
  }
  
  public int getPotentialInput(int pos) {
    return potentialInput[pos-1];
  }

  public boolean possessPotentialInput(int pos) {
  	return possessPotentialInput[pos - 1];
  }

  public void setPossessPotentialInput(int pos, boolean truthValue) {
  	possessPotentialInput[pos -1] = truthValue;
  	// If set(false) => reset potentialInput values
  	if (!truthValue)
  		resetPotentialInput(pos);
  	
  	// if set(false) => change potentialOutput
  	// if set(true) => the change is done by the setpotentialInput
  	boolean possessAllInputs = true;
    for (int i = 1; i <= numberOfINs(); i++)
	    if (!possessPotentialInput(i))
	    	possessAllInputs = false;
	  if (possessAllInputs)
	  	setPossessPotentialOutput(true);
  	else {
  		setPossessPotentialOutput(false);
  		// ###DEBUG : if no potentialInput => setGoalReached(false) ?
  		if (isInHigherLowerReorganization())
  			setGoalReached(false);
  	}
  }
  
  // Called by setSpecificPotentialOutput because each agent has to calculate the output
  public void setPotentialOutput(int x) {
    potentialOutput = x;
		setPossessPotentialOutput(true);
		if (ownerAgent.getAcquaintancesManager().possessingOUT()) {
    	// Possess PotentialOutput again => message to OUT
    	gui.println("possessing PotentialOutput again.", java.awt.Color.red);
    	if (ownerAgent.getAcquaintancesManager().possessingOUT())
    		ownerAgent.fireMessage(new PotentialOutputMessage(x, ownerAgent), 
    										 			 new AgentInterface("OUT", 1),
    													 ownerAgent);
		}

		// ###DEBUG : test : setPotentialOutput => always notify outputs (not only if change)
//    if (!possessPotentialOutput()) {
//    	setPossessPotentialOutput(true);
//    	if ((isInHigherLowerReorganization()
//    			&& (ownerAgent.getAcquaintancesManager().possessingOUT()))) {
//	    	// Possess PotentialOutput again => message to OUT
//	    	gui.println("possessing PotentialOutput again.", java.awt.Color.red);
//	    	if (ownerAgent.getAcquaintancesManager().possessingOUT())
//	    		ownerAgent.fireMessage(new PotentialOutputMessage(x, ownerAgent), 
//	    										 			 new AgentInterface("OUT", 1),
//	    													 ownerAgent);
//    	}
//    }
  }

  public void resetPotentialOutput() {
    potentialOutput = 0;
  }
  
  public int getPotentialOutput() {
    return potentialOutput;
  }

  public void setPossessPotentialOutput(boolean truthValue) {
  	possessPotentialOutput = truthValue;
  	if (!truthValue) {
  		resetPotentialOutput();
  		if (ownerAgent.getAcquaintancesManager().possessingOUT())
		  	ownerAgent.fireMessage(new NCSNoPotentialOutputMessage(ownerAgent),
		      												 new AgentInterface("OUT", 1),
		      												 ownerAgent);
  	}
  }
  
  public boolean possessPotentialOutput() {
  	return possessPotentialOutput;
  }
  
  public void updatePotentialInput() {
  	setPotentialInput(1, getLastInputValue(1));
  	setPotentialInput(2, getLastInputValue(2));
  }
  
  public void updatePotentialOuput() {
  	setPotentialOutput(getLastOutputValue());
  }
  
  public void setOutputThreshold(int x) {
    outputThreshold = x;
  }
  
  public int getOutputThreshold() {
    return outputThreshold;
  }

  public void setHigherLowerGoal(String value) {
    higherLowerGoal = value;    
  }
  
  public String getHigherLowerGoal() {
    return higherLowerGoal;
  }
  
  public void setCurrentImposingPrejudice(int x) {
    currentImposingPrejudice = x;
  }
  
  public int getImposingPrejudice() {
    return currentImposingPrejudice;
  }
  
  public void setInPrejudice(boolean inPrejudice) {
  	this.inPrejudice = inPrejudice;
  }
  
  public boolean isInPrejudice() {
  	return inPrejudice;
  }
  
  public void setReorganizationPhase(boolean truthValue) {
    reorganizationPhase = truthValue;
  }
  
  public boolean inReorganizationPhase() {
    return reorganizationPhase;
  }
  
  public void setInHigherLowerReorganization(boolean truthValue) {
    inHigherLowerReorganization = truthValue;
  }
  
  public boolean isInHigherLowerReorganization() {
    return inHigherLowerReorganization;
  }
  
  public boolean lastInOutAcquaintancesManagementEnabled() {
    return (!isInHigherLowerReorganization());
  }

  public void setReceivedPropositionOutFromAgentInNeed(boolean truthValue) {
    receivedPropositionOutFromAgentInNeed = truthValue;
  }
  
  public boolean receivedPropositionOutFromAgentInNeed() {
    return receivedPropositionOutFromAgentInNeed;
  }
  
  public void setReceivedPropositionInFromAgentInNeed(boolean truthValue) {
    receivedPropositionInFromAgentInNeed = truthValue;
  }
  
  public boolean receivedPropositionInFromAgentInNeed() {
    return receivedPropositionInFromAgentInNeed;
  }
  
  public int getCurrentReorganizationPhase() {
  	return currentReorganizationPhase;
  }

  public void setCurrentReorganizationPhase(int currentReorganizationPhase) {
  	this.currentReorganizationPhase = currentReorganizationPhase;
  }
  
  public int getNumberOfNeedInHLNCSWithoutAnswer() {
	  return numberOfNeedInHLNCSWithoutAnswer;
  }

  public void setNumberOfNeedInHLNCSWithoutAnswer(int numberOfNeedInHLNCSWithoutAnswer) {
	  this.numberOfNeedInHLNCSWithoutAnswer = numberOfNeedInHLNCSWithoutAnswer;
  }

  public int getMaxNumberOfNeedInHLNCSWithoutAnswer() {
	  return maxNumberOfNeedInHLNCSWithoutAnswer;
  }
  
  public void reInitializenumberOfNeedInHLNCSWithoutAnswer() {
  	numberOfNeedInHLNCSWithoutAnswer = 0;
  }
  
  public boolean isGoalReached() {
  	return goalReached;
  }

  public boolean isPersonalGoalReached() {
  	String check = checkGoalReached();
  	return (check.equals("SUCCEEDED"));
  }
  
  public void setGoalReached(boolean goalReached) {
  	gui.println("GOAL REACHED = " + goalReached, java.awt.Color.red);
  	this.goalReached = goalReached;
  	// Tell the BestOfWorseTimers to stop if goal reached
  	// or restart if goalNotReached 
  	if (goalReached)
	  	for (int i=1; i <= numberOfINs(); i++)
	  		waitForProposalsTimerManager.stopWaiting(i);
  	
  	// ###DEBUG goalreached(false) => relaunch timers needed ?
//  	else
//  		for (int i=1; i <= numberOfINs(); i++)
//	  		waitForProposalsTimerManager.waitForProposals(i);
  }
  
	public void setThresholdAcq(Agent agent, AgentInterface agentInterface) {
		thresholdAcquaintances.addThresholdAcquaintance(agent, agentInterface);
	}
  
	public ThresholdAcquaintance getThresholdAcq(AgentInterface agentInterface) {
		return thresholdAcquaintances.getThresholdAcquaintance(agentInterface);
	}
	
	public boolean isAThresholdAcq(Agent agent, AgentInterface agentInterface) {
		return thresholdAcquaintances.isAThresholdAcquaintance(agent, agentInterface); 
	}
	
// End of Get and Set section
  
  // Initialization of Higher/Lower reorganization attributes
  public void initializeHigherLowerReorganizationIfNeeded(String information, int phase) {
    if (phase > getCurrentReorganizationPhase()) {
    	setCurrentReorganizationPhase(phase);
      setReorganizationPhase(true);
      for (int i = 1; i <= ownerAgent.getNumberOfINs(); i++) {
        setInputThreshold(i, getLastInputValue(i));
        setPossessPotentialInput(i, true);
      }
      setOutputThreshold(getLastOutputValue());
      setHigherLowerGoal(information);
      setCurrentImposingPrejudice(0);
      setInHigherLowerReorganization(true);
      setInPrejudice(false);
      setGoalReached(false);
      reInitializePotentialInOutAcquaintances();
      bestOfWorseManager.reinitialize();
      reorganizationLimiter.reinitialize();
      // set the ThresholdAcqs
      thresholdAcquaintances.initializeThresholdAcquaintances(numberOfINs());
      
	    ownerAgent.getAcquaintancesManager().showAllAcquaintances();
	    showAgentStateInfo();
    }
  }
  
  // firedNcsTimer, firedRequestTimer and uselessnessTimer 
  // call this method when they get to zero
  public void actionPerformed(ActionEvent e) {
    if ((e.getSource()).equals(firedNcsTimer)) {
      ownerAgent.setActed(true);
      firedNcsTimerRanOut = true;
      ownerAgent.startAgent();
    }
    else if ((e.getSource()).equals(firedRequestTimer)) {
      ownerAgent.setActed(true);
      firedRequestTimerRanOut = true;
      ownerAgent.startAgent();
    }
    else if ((e.getSource()).equals(uselessnessTimer)) {
      ownerAgent.setActed(true);
      uselessnessTimerRanOut = true;
      ownerAgent.startAgent();
    }
    else {
      SystemPrinter.printSystem("ERROR : wrong action event received from timers");
      ownerAgent.errorHappened();
    }
  }
  
  public void ncsFired() {
    firedNcsTimerRanOut = false;
    firedNcsTimer.start();
  }
  
  public boolean readyToFireNCSs() {
    return firedNcsTimerRanOut;
  }
  
  public void requestFired() {
    firedRequestTimerRanOut = false;
    firedRequestTimer.start();
  }
  
  protected void fireUselessness() {
    // Needed to solve isolation problem.
    // ###LOW_DEBUG : uselessnessSNCs not activated yet : some cases aren't detected
    // Temporary solution : agent IN knows all the agents 
    // so they get contacted eventually if needed
    /* 
    uselessnessTimerRanOut = false;
    uselessnessTimer.start();
    // ###LOW_DEBUG : uselessnessNCS
    SystemPrinter.printSystem(ownerAgent.getAgentName() + " : DEBUG : USelessness fired");
     **/
  }
  
  public void detectInOutNCSs() {
    // Detects the NCSs : NCSneedIN, NCSneedOut
    checkInOut();
    gui.println("Trying to detect NeedIn/OutNCSs");
    if ( needIN() | needOUT() ) {
      ownerAgent.setCooperativeState(false);
      // NeedIn
      for (int i=1; i<=numberOfINs(); i++)
	      if (needIN(i)) {
	        gui.println("Detected NeedIN" + i + " NCS[" 
	                    + dataTypeNeeded("IN", i) +"]", java.awt.Color.magenta);
	        setPossessPotentialInput(i, false);
	        gui.println("NoPotentialInput => NoPotentialOutputNCS detected.", 
	        						java.awt.Color.magenta);
	      }
	    // NeedOut
      if (needOUT())
        gui.println("Detected NeedOutNCS["
        						+ dataTypeNeeded("OUT", 1) +"]", java.awt.Color.magenta);
    }
    else {
    	if (!requestManager.hasRequestedInOut())
    		reInitializePotentialInOutAcquaintances();
      ownerAgent.setCooperativeState(true);
    }
  }

  public void checkCooperativeState() {
    checkInOut();
    if ( needIN() | needOUT() )
      ownerAgent.setCooperativeState(false);
    else
      ownerAgent.setCooperativeState(true);
  }
  
  // Sp?cific for each agent : each agent has to check its INs and OUTs
  public abstract void checkInOut();
  
  // The agent OUT overides this method because it knows it is usefull
  public void detectUselessnessNCSs() {
    connectedToOutputFlag = false;
    java.util.Vector positions = ownerAgent.getAcquaintancesManager()
                                           .getAcquaintances()
                                           .findAcquaintance("OUT");
    epe.agents.acquaintances.Acquaintance acq;
    for (int i=0; i<positions.size(); i++) {
      int rang = ((Integer)positions.elementAt(i)).intValue();
      acq = ownerAgent.getAcquaintancesManager()
                      .getAcquaintances()
                      .getAcquaintance(rang);
      if (acq.getAgent().isUsefull())
        connectedToOutputFlag = true;
    }
  }
  
  public boolean isUsefull() {
    return connectedToOutputFlag;
  }

// Start of Abstract methods section
// Override these methods for each agent

  public abstract int numberOfINs();
  
  public abstract boolean needIN(int position);
  
  public abstract boolean needIN();
  
  public abstract boolean needOUT();
  
  protected abstract void setNeedIN(int position, boolean truthValue);

  protected abstract void setNeedOUT(boolean truthValue);
  
  protected abstract String dataTypeNeededOriginally(String interfaceType, int position);
  
  public abstract String dataTypeNeeded(String interfaceType, int position);
  
  // The agent decides if it has the competences to answer to a request
  public abstract boolean isRelevant(AgentInterface interfaceTypeAsked,
                                     int position);
  
  public abstract boolean isDataRelevant(String interfaceType,
                                         int position,
                                         String dataType);

  public abstract boolean isOriginallyRelevant (AgentInterface interfaceTypeAsked,
                                                int position);
   
  // When the agents gets a new In/Out, if the agent has GENERIC In/Out, 
  // it checks if the new acq changes its dataTypes
  public abstract void checkDataTypes();
  
  public abstract void setSpecificPotentialOutput();
  
  public abstract int calculatePotentialOutput(int IN1, int IN2);
 
// End of abstract methods section
  
  public String synchronizeDataTypes(String dataType1, String dataType2) {
    if (dataType1 == "GENERIC")
      return dataType2;
    else if (dataType2 == "GENERIC")
      return dataType1;
    else if ((dataType1 == "INTEGER") && (dataType2 == "INTEGER"))
      return "INTEGER";
    else if ((dataType1 == "BOOLEAN") && (dataType2 == "BOOLEAN"))
      return "BOOLEAN";
    else {
      gui.println("WARNING : data synchronization failled : "
                  + "[" + dataType1 + "/" + dataType2 + "]", java.awt.Color.magenta);
      return "FAILLED";
    }
  }

  public void addInAcquaintance(Agent agentToAdd,
                                   String interfaceType,
                                   int position,
                                   String dataType) {
    ownerAgent.addAcquaintance(agentToAdd, interfaceType, position, dataType);
    checkDataTypes();
    setNeedIN(position, false);
    potentialInOutAcquaintances
      .addLastInOutAcquaintance(agentToAdd, interfaceType, position, dataType);
  }

  public void addOutAcquaintance(Agent agentToAdd,
                                    String interfaceType,
                                    int position,
                                    String dataType) {
    ownerAgent.addAcquaintance(agentToAdd, interfaceType, position, dataType);
    checkDataTypes();
    setNeedOUT(false);
    potentialInOutAcquaintances
      .addLastInOutAcquaintance(agentToAdd, interfaceType, position, dataType);
  }
  
  public void processMessage(Message receivedMessage) {
    String typeId = receivedMessage.getId();
    
    // NCS
    if      (typeId.equals(Message.NCS_NEED_IN_MESSAGE))
      messageProcessor.processNCSneedInMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_NEED_OUT_MESSAGE))
      messageProcessor.processNCSneedOutMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_INADEQUATE_DATA_MESSAGE))
      messageProcessor.processNCSinadequateDataMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_MISSING_DATA_MESSAGE))
      messageProcessor.processNCSmissingDataMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_INFORMED_INADEQUATE_DATA_MESSAGE))
    	messageProcessor.processNCSInformedInadequateDataMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_INFORMED_NEED_IN_MESSAGE))
    	messageProcessor.processNCSInformedNeedInMessage(receivedMessage);
    else if (typeId.equals(Message.NCS_NO_POTENTIAL_OUTPUT_MESSAGE))
    	messageProcessor.processNCSNoPotentialOutputMessage(receivedMessage);
    
    // Contract
    else if (typeId.equals(Message.PROPOSING_COOPERATION_MESSAGE))
      messageProcessor.processProposingCooperationMessage(receivedMessage);
    else if (typeId.equals(Message.REQUESTING_COOPERATION_MESSAGE))
      messageProcessor.processRequestingCooperationMessage(receivedMessage);
    else if (typeId.equals(Message.ACKNOWLEDGING_COOPERATION_MESSAGE))
      messageProcessor.processAcknowledgingCooperationMessage(receivedMessage);
    else if (typeId.equals(Message.REFUSING_COOPERATION_MESSAGE))
      messageProcessor.processRefusingCooperationMessage(receivedMessage);
    else if (typeId.equals(Message.CANCELLING_COOPERATION_MESSAGE))
      messageProcessor.processCancellingCooperationMessage(receivedMessage);
    else if (typeId.equals(Message.PROPOSING_COOPERATION_OUT_WITH_PREJUDICE_MESSAGE))
      messageProcessor
      	.processProposingCooperationOutWithPrejudiceMessage(receivedMessage);
    else if (typeId.equals(Message.PROPOSING_COOPERATION_IN_WITH_POTENTIAL_VALUE_MESSAGE))
      messageProcessor
      	.processProposingCooperationInWithPotentialValueMessage(receivedMessage);
    
    // Information
    else if (typeId.equals(Message.DATATYPE_CHANGED_MESSAGE))
      messageProcessor.processDataTypeChangedMessage(receivedMessage);
    else if (typeId.equals(Message.POTENTIAL_OUTPUT_MESSAGE))
    	messageProcessor.processPotentialOutputMessage(receivedMessage);
		else if (typeId.equals(Message.GOAL_REACHED_MESSAGE))
    	messageProcessor.processGoalReachedMessage(receivedMessage);
		else if (typeId.equals(Message.LINK_MULTIPLICITY_CHANGED_MESSAGE))
			messageProcessor.processLinkMultiplicityChangedMessage(receivedMessage);
		else if (typeId.equals(Message.CYCLE_PING_MESSAGE))
			messageProcessor.procesCyclePingMessage(receivedMessage);
  }
  
  public void showAllPotentialAcquaintances() {
    gui.println(ownerAgent.getAgentName() +"'s Potential Acquaintances : ", java.awt.Color.cyan); 
    potentialInOutAcquaintances.showAllPotentialAcquaintances();
  }
   
  public void cancelCooperation(AgentInterface interfaceType) {
    // Notify cooperating agent of canceling
  	CancellingCooperationMessage message = new CancellingCooperationMessage(ownerAgent,
  																																					interfaceType);
    ownerAgent.fireMessage(message, 
                           ownerAgent.getAcquaintancesManager()
                                     .getAgentIN(interfaceType.getPosition()), 
                           "NEUTRAL", 
                           ownerAgent);

     // Cancel cooperation
    int position = interfaceType.getPosition();
    gui.println("Cancelling old Cooperation : ["
                + interfaceType.getType() + ":"
                + position + ":"
                + interfaceType.getDataType() + "]");
    Acquaintances acquaintances = ownerAgent.getAcquaintancesManager().getAcquaintances();
    Acquaintance acq = acquaintances.getAcquaintance(interfaceType.getType(), 
                                                     interfaceType.getPosition());
    gui.println("Acquaintance ["
                + acq.getAgent().getAgentName() + ":"
                + acq.getInterfaceType() + ":"
                + acq.getInterfacePosition() + ":"
                + acq.getDataType() + "]"
                + " has been set to NEUTRAL", java.awt.Color.blue);
    ownerAgent.getAcquaintancesManager().cancelCooperation(acq);
  }
  
  public void showAgentStateInfo() {
  	gui.println("Agent" + ownerAgent.getAgentName() +" State Info : ", java.awt.Color.blue);
  	
  	gui.println("	_____IN_____", java.awt.Color.blue);
  	gui.println("	lastInputValue[IN1/IN2] : [" + 
  							getLastInputValue(1) + "/" +
  							getLastInputValue(2) + "]", java.awt.Color.blue);
  	gui.println("	ThresholdAcqs :", java.awt.Color.blue); 
  	thresholdAcquaintances.showAllThresholdAcquaintances(); 
  	gui.println("	inputThreshold[IN1/IN2] : [" + 
  							getInputThreshold(1) + "/" +
  							getInputThreshold(2) + "]", java.awt.Color.blue);
  	gui.println("	possessPotentialInput[IN1/IN2] : [" + 
				  			possessPotentialInput(1) + "/" +
				  			possessPotentialInput(2) + "]", java.awt.Color.blue);
  	gui.println("	potentialInput[IN1/IN2] : [" + 
  							getPotentialInput(1) + "/" +
  							getPotentialInput(2) + "]", java.awt.Color.blue);
  	
  	gui.println("	_____OUT____", java.awt.Color.blue);
  	gui.println("	lastOutputValue : [" + 
  							getLastOutputValue() + "]", java.awt.Color.blue);
  	gui.println("	outputThreshold : [" + 
  							getOutputThreshold() + "]", java.awt.Color.blue);
  	gui.println("	possessingPotentialOutput : [" + 
  							possessPotentialOutput() + "]", java.awt.Color.blue);
  	gui.println("	potentialOutput : [" + 
  							getPotentialOutput() + "]", java.awt.Color.blue);
  	
  	gui.println("	_____MISC___", java.awt.Color.blue);
  	gui.println("	inHigherLowerReorganization : [" + 
  							isInHigherLowerReorganization() + "]", java.awt.Color.blue);
  	gui.println("	currentReorganizationPhase : [" + 
  							getCurrentReorganizationPhase() + "]", java.awt.Color.blue);
  	gui.println("	higherLowerGoal : [" + 
  							getHigherLowerGoal() + "]", java.awt.Color.blue);
  	gui.println("	inPrejudice : [" + 
  							isInPrejudice() + "]", java.awt.Color.blue);
  	gui.println("	imposingPrejudice : [" + 
  							getImposingPrejudice() + "]", java.awt.Color.blue);
  	gui.println("	goalReached : [" + 
  							isGoalReached() + "]", java.awt.Color.blue);
  	bestOfWorseManager.showBestOfWorse();
  }
	
}