package epe.reorganization;

	import java.util.Vector;
	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.ncs.NCSNeedInMessage;
	import epe.communication.messages.ncs.NCSNeedOutMessage;
	import epe.communication.messages.ncs.NCSInformedNeedInMessage;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;
	import epe.reorganization.bestOfWorseManagement.BestOfWorseManager;
	import epe.reorganization.bestOfWorseManagement.WaitForProposalsTimerManager;

	import epe.reorganization.messageProcessors.information.DataTypeChangedMessageProcessor;
	import epe.reorganization.messageProcessors.information.GoalReachedMessageProcessor;
	import epe.reorganization.messageProcessors.information.LinkMultiplicityChangedMessageProcessor;
	import epe.reorganization.messageProcessors.information.PotentialOutputMessageProcessor;
	import epe.reorganization.messageProcessors.information.CyclePingMessageProcessor;
	
	import epe.reorganization.messageProcessors.ncs
						.NCSInformedInadequateDataMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSInadequateDataMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSMissingDataMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSInformedNeedInMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSNeedInMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSNeedOutMessageProcessor;
	import epe.reorganization.messageProcessors.ncs
						.NCSNoPotentialOutputMessageProcessor;
	
	import epe.reorganization.messageProcessors.contract
						.AcknowledgingCooperationMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.CancellingCooperationMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.ProposingCooperationInWithPotentialValueMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.ProposingCooperationMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.ProposingCooperationOutWithPrejudiceMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.RefusingCooperationMessageProcessor;
	import epe.reorganization.messageProcessors.contract
						.RequestingCooperationMessageProcessor;
	

public class ReorganizationMessageProcessor extends java.lang.Object {
  
	  protected Agent ownerAgent;
	  protected GUI gui;
	  
	  // information
	  protected DataTypeChangedMessageProcessor dataTypeChangedMessageProcessor;
	  protected PotentialOutputMessageProcessor potentialOutputMessageProcessor;
	  protected GoalReachedMessageProcessor goalReachedMessageProcessor;
	  protected LinkMultiplicityChangedMessageProcessor linkMultiplicityChangedMessageProcessor;
	  protected CyclePingMessageProcessor cyclePingMessageProcessor;
	  
	  // NCS
	  protected NCSNeedInMessageProcessor ncsNeedInMessageProcessor;
	  protected NCSNeedOutMessageProcessor ncsNeedOutMessageProcessor;
	  protected NCSMissingDataMessageProcessor ncsMissingDataMessageProcessor;
	  protected NCSInadequateDataMessageProcessor ncsInadequateDataMessageProcessor;
	  protected NCSInformedInadequateDataMessageProcessor 
	  						ncsInformedInadequateDataMessageProcessor;
	  protected NCSInformedNeedInMessageProcessor ncsInformedNeedInMessageProcessor;
	  protected NCSNoPotentialOutputMessageProcessor ncsNoPotentialOutputMessageProcessor;
	  
	  // Contract
	  protected ProposingCooperationMessageProcessor proposingCooperationMessageProcessor;
	  protected RequestingCooperationMessageProcessor requestingCooperationMessageProcessor;
	  protected AcknowledgingCooperationMessageProcessor acknowledgingCooperationMessageProcessor;
	  protected RefusingCooperationMessageProcessor refusingCooperationMessageProcessor;
	  protected CancellingCooperationMessageProcessor cancellingCooperationMessageProcessor;
	  
	  protected ProposingCooperationOutWithPrejudiceMessageProcessor 
	              proposingCooperationOutWithPrejudiceMessageProcessor;
	  protected ProposingCooperationInWithPotentialValueMessageProcessor
	              proposingCooperationInWithPotentialValueMessageProcessor;
	  
	  
	  
	  // Tells to how many agents the agent should propagate NCSs in "sendMessageToSomeAcqs()"
	  protected int limitNumberOfAgentsForPropagation = 1;
	  // Used to choose to which agent to propagate the NCSs => cycling through the acqs
	  protected int acquaintancesCyclingCounter = 0;

  public ReorganizationMessageProcessor(
  			 	 ReorganizationManager reorganizationManager,
				 	 ReorganizationEngine reorganizationEngine,
           Agent ownerAgent,
           GUI gui,
           PotentialInOutAcquaintances potentialInOutAcquaintances,
		   		 WaitForProposalsTimerManager waitForProposalsTimerManager,
		   		 BestOfWorseManager bestOfWorseManager,
				 	 LinkMultiplicityManager linkMultiplicityManager) {
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    
    // Create the different MessageProcessors
    
    // NCS
    ncsNeedInMessageProcessor
      = new NCSNeedInMessageProcessor(reorganizationManager, this, ownerAgent, gui);
    ncsNeedOutMessageProcessor
      = new NCSNeedOutMessageProcessor(reorganizationManager, this, ownerAgent, gui);
    ncsMissingDataMessageProcessor
      = new NCSMissingDataMessageProcessor(this, gui);
    ncsInadequateDataMessageProcessor
      = new NCSInadequateDataMessageProcessor(this, ownerAgent, gui);
    ncsInformedInadequateDataMessageProcessor
    	= new NCSInformedInadequateDataMessageProcessor(reorganizationManager, 
    																									this, ownerAgent, gui,
																											waitForProposalsTimerManager);
    ncsInformedNeedInMessageProcessor
    	= new NCSInformedNeedInMessageProcessor(reorganizationManager, this, ownerAgent, gui,
    																					waitForProposalsTimerManager);
    ncsNoPotentialOutputMessageProcessor
    	= new NCSNoPotentialOutputMessageProcessor(reorganizationManager, this, ownerAgent, gui);
    
    proposingCooperationMessageProcessor
     	= new ProposingCooperationMessageProcessor(reorganizationManager, ownerAgent, 
                                                 gui, potentialInOutAcquaintances);
    
    // Contract
    requestingCooperationMessageProcessor
     	= new RequestingCooperationMessageProcessor(reorganizationManager, ownerAgent, 
                                                  gui, potentialInOutAcquaintances,
												  												linkMultiplicityManager);
    acknowledgingCooperationMessageProcessor
      = new AcknowledgingCooperationMessageProcessor(reorganizationManager, ownerAgent, 
                                                     gui, potentialInOutAcquaintances,
													 													 linkMultiplicityManager);
    refusingCooperationMessageProcessor
      = new RefusingCooperationMessageProcessor(reorganizationManager, gui, 
      																					potentialInOutAcquaintances);
    cancellingCooperationMessageProcessor
      = new CancellingCooperationMessageProcessor(ownerAgent,linkMultiplicityManager, gui);
    proposingCooperationOutWithPrejudiceMessageProcessor
      = new ProposingCooperationOutWithPrejudiceMessageProcessor(reorganizationManager,
                                                               	 ownerAgent,
                                                               	 gui, 
                                                               	 potentialInOutAcquaintances);
    proposingCooperationInWithPotentialValueMessageProcessor
      = new ProposingCooperationInWithPotentialValueMessageProcessor(
      				reorganizationManager,
      				reorganizationEngine,
							ownerAgent,
							gui,
							potentialInOutAcquaintances,
							bestOfWorseManager);
    // Information
    dataTypeChangedMessageProcessor
      = new DataTypeChangedMessageProcessor(reorganizationManager, ownerAgent, gui,
      																			linkMultiplicityManager);
    potentialOutputMessageProcessor
    	= new PotentialOutputMessageProcessor(reorganizationManager, ownerAgent, gui);
    goalReachedMessageProcessor
    	= new GoalReachedMessageProcessor(
    					reorganizationManager, ownerAgent, gui, waitForProposalsTimerManager);
    linkMultiplicityChangedMessageProcessor
    	= new LinkMultiplicityChangedMessageProcessor(reorganizationManager, ownerAgent, gui);
    cyclePingMessageProcessor 
			= new CyclePingMessageProcessor(reorganizationManager, reorganizationEngine, ownerAgent, gui);
  }
  
  
  // NCS
  
  public void processNCSneedInMessage(Message receivedMessage) {
    ncsNeedInMessageProcessor.processNCSneedInMessage(receivedMessage);
  }
  
  public void processNCSneedOutMessage(Message receivedMessage) {
    ncsNeedOutMessageProcessor.processNCSneedOutMessage(receivedMessage);
  }
  
  public void processNCSmissingDataMessage(Message receivedMessage) {
    ncsMissingDataMessageProcessor.processNCSmissingDataMessage(receivedMessage);
  }

  public void processNCSinadequateDataMessage(Message receivedMessage) {
    ncsInadequateDataMessageProcessor.processNCSinadequateDataMessage(receivedMessage);
  }

  public void processNCSInformedInadequateDataMessage(Message receivedMessage) {
  	ncsInformedInadequateDataMessageProcessor
  	.processNCSInformedInadequateDataMessage(receivedMessage);
  }
  
  public void processNCSInformedNeedInMessage(Message receivedMessage) {
  	ncsInformedNeedInMessageProcessor.processNCSInformedNeedInMessage(receivedMessage);
  }
  
  public void processNCSNoPotentialOutputMessage(
  		Message receivedMessage) {
  	ncsNoPotentialOutputMessageProcessor.processNCSNoPotentialOutputMessage(receivedMessage);
  }
  
  
  // Contract
  
  public void processProposingCooperationMessage(Message receivedMessage) {
    proposingCooperationMessageProcessor.processProposingCooperationMessage(receivedMessage);
  }
  
  public void processRequestingCooperationMessage(Message receivedMessage) {
    requestingCooperationMessageProcessor.processRequestingCooperationMessage(receivedMessage);
  }
  
  public void processAcknowledgingCooperationMessage(Message receivedMessage) {
    acknowledgingCooperationMessageProcessor
    	.processAcknowledgingCooperationMessage(receivedMessage);
  }
  
  public void processRefusingCooperationMessage(Message receivedMessage) {
    refusingCooperationMessageProcessor.processRefusingCooperationMessage(receivedMessage);
  }
  
  public void processCancellingCooperationMessage(Message receivedMessage) {
    cancellingCooperationMessageProcessor.processCancellingCooperationMessage(receivedMessage);
  }
  
  public void processProposingCooperationOutWithPrejudiceMessage(
  							Message receivedMessage) {
    proposingCooperationOutWithPrejudiceMessageProcessor
      .processProposingCooperationOutWithPrejudiceMessage(receivedMessage);
  }
  
  public void processProposingCooperationInWithPotentialValueMessage(
  							Message receivedMessage) {
    proposingCooperationInWithPotentialValueMessageProcessor
      .processProposingCooperationInWithPotentialValueMessage(receivedMessage);
  }

  
  // Information
  
  public void processDataTypeChangedMessage(Message receivedMessage) {
    dataTypeChangedMessageProcessor.processDataTypeChangedMessage(receivedMessage);
  }
  
  public void processPotentialOutputMessage(Message receivedMessage) {
  	potentialOutputMessageProcessor
  		.processPotentialOutputMessage(receivedMessage);
  }
  
  public void processGoalReachedMessage(Message receivedMessage) {
  	goalReachedMessageProcessor
  		.processGoalReachedMessage(receivedMessage);
  }
  
  public void processLinkMultiplicityChangedMessage(Message receivedMessage) {
  	linkMultiplicityChangedMessageProcessor
  		.processLinkMultiplicityChangedMessage(receivedMessage);
  }
  
  public void procesCyclePingMessage(Message receivedMessage) {
  	cyclePingMessageProcessor.processCyclePingMessage(receivedMessage);
  }
  
  
// Propagation  
  
  public void propagateNCS(Message receivedMessage) {
    gui.println("Propagating NCS ...");
    String typeId = receivedMessage.getId();
    if (typeId.equals(Message.NCS_NEED_IN_MESSAGE)) {
      ((NCSNeedInMessage)receivedMessage).decreaseEffectValue();
      if (((NCSNeedInMessage)receivedMessage).isEffective()) {
        //sendMessageToAllAcquaintancesButSender(receivedMessage);
        sendMessageToSomeAcquaintancesButNotSender(receivedMessage);
      }
      else
      	gui.println("Effective no more.");
    }
    else if (typeId.equals(Message.NCS_NEED_OUT_MESSAGE)) {
      ((NCSNeedOutMessage)receivedMessage).decreaseEffectValue();
      if (((NCSNeedOutMessage)receivedMessage).isEffective()) {
        //sendMessageToAllAcquaintancesButSender(receivedMessage);
        sendMessageToSomeAcquaintancesButNotSender(receivedMessage);
      }
      else
      	gui.println("Effective no more.");
    }
		else if (typeId.equals(Message.NCS_INFORMED_NEED_IN_MESSAGE)) {
		  ((NCSInformedNeedInMessage)receivedMessage).decreaseEffectValue();
		  if (((NCSInformedNeedInMessage)receivedMessage).isEffective()) {
			//sendMessageToAllAcquaintancesButSender(receivedMessage);
			sendMessageToSomeAcquaintancesButNotSender(receivedMessage);
		  }
		  else
		  	gui.println("Effective no more.");
		}
    else {
			gui.println("Message type not for propagation : " 
								  + typeId, java.awt.Color.red);
    }
  }

  public void targetedPropagateNCS(Message receivedMessage, AgentInterface target) {
    gui.println("Propagating NCS ...");
    receivedMessage.setSender(ownerAgent);
    ownerAgent.fireMessage(receivedMessage, target, ownerAgent);
  }
  
  public void sendMessageToAllAcquaintances(Message message) {
    ownerAgent.fireMessageToAllAcquaintances(message);
  }

  protected void sendMessageToAllAcquaintancesButSender(Message message) {
    ownerAgent.fireMessageToAllAcquaintancesButSender(message);
  }

  // Send the message to a restricted number of agents
  // Restriction strategy ? : 1) cycling through all acquaintances
  // Restriction strategy ? : 2) random choice of acquaintances
  // Needed for the propagation of NCSs
  // Prevent message flooding
  protected void sendMessageToSomeAcquaintancesButNotSender(Message message) {
  	Vector acqs = ownerAgent.getAcquaintancesManager().getAcquaintances().getAllAcquaintances();
  	Agent sender = message.getSender();
  	
  	// Strategy 1 : Cycling Strategy
  	//cyclingStrategy(acqs, sender, message);
  	
  	// Strategy 2 : Random Strategy  	
  	randomStrategy(acqs, sender, message);
  }
  
  protected void cyclingStrategy(Vector acqs, Agent sender, Message message) {
    if (acquaintancesCyclingCounter >= acqs.size())
        acquaintancesCyclingCounter = 0;
    int i = 0;
    boolean fullCycleCompleted = false;
    int initialCycleCounter = acquaintancesCyclingCounter;
    Acquaintance acq;
    while ((i < limitNumberOfAgentsForPropagation) && (!fullCycleCompleted)) {
      acq = (Acquaintance)ownerAgent.getAcquaintancesManager()
                                    .getAcquaintances()
                                    .elementAt(acquaintancesCyclingCounter);
      if ((!acq.getAgent().equals(sender))
       	 && (!acq.getAgent().getAgentName().equals("Env"))) {
        ownerAgent.fireMessage(message, acq.getAgent(), acq.getInterfaceType(), ownerAgent);
        i++;
      }
      acquaintancesCyclingCounter++;
      if (acquaintancesCyclingCounter >= acqs.size())
        acquaintancesCyclingCounter = 0;
      if (acquaintancesCyclingCounter == initialCycleCounter)
        fullCycleCompleted = true;
    }
  }
  
  protected void randomStrategy(Vector acqs, Agent sender, Message message) {
    int numberOfAcqs = acqs.size();
    Acquaintance acq;
    
    // check if there are some possible acqs 
    int numberOfRelevantAcqs = 0;
    for (int i = 0; i < numberOfAcqs; i++) {
    	acq = (Acquaintance)ownerAgent.getAcquaintancesManager()
		                                .getAcquaintances().elementAt(i);
    	if (!(acq.getAgent().equals(sender))
  			 && (!acq.getAgent().getAgentName().equals("Env")))
    		numberOfRelevantAcqs++;
    }
    
//    // ###DEBUG randomStrategy
//    gui.println("Random Strategy :  numberOfAcqs = " + numberOfAcqs +
//    						" / relevant = " + numberOfRelevantAcqs, java.awt.Color.red);
    
    if (numberOfRelevantAcqs > 0) {
    	int numberOfPropagationsCounter = 0;
    	int counterLimit;
    	
    	if (numberOfRelevantAcqs < limitNumberOfAgentsForPropagation)
    		counterLimit = numberOfRelevantAcqs;
    	else counterLimit = limitNumberOfAgentsForPropagation;
    	while (numberOfPropagationsCounter < counterLimit) {
		    int position = (int)(java.lang.Math.random() * numberOfAcqs);
		    
//		    // ###DEBUG randomStrategy
//		    gui.println("Random Strategy chosen position = " + position, java.awt.Color.red);
		    
		  	acq = (Acquaintance)ownerAgent.getAcquaintancesManager()
			                                .getAcquaintances().elementAt(position);
		  	while ((acq.getAgent().equals(sender))
		  				|| (acq.getAgent().getAgentName().equals("Env"))) {
		  		position = (int)(java.lang.Math.random() * numberOfAcqs);
			  	acq = (Acquaintance)ownerAgent.getAcquaintancesManager()
				                                .getAcquaintances().elementAt(position);
		  	}
		  	ownerAgent.fireMessage(message, acq.getAgent(), acq.getInterfaceType(), ownerAgent);
		  	numberOfPropagationsCounter++;
    	}
    }
    // else : no acqs to propagate
  }
    
}
