package epe.reorganization.bestOfWorseManagement;

	import epe.agents.Agent;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.ReorganizationEngine;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.messages.contract.RequestingCooperationMessage;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintance;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;
	import epe.gui.GUI;

public class BestOfWorseManager {
	
		protected BestOfWorse[] bestOfWorseForPosition;
		protected Agent ownerAgent;
		protected ReorganizationManager reorganizationManager;
		protected ReorganizationEngine reorganizationEngine;
		protected PotentialInOutAcquaintances potentialInOutAcquaintances;
		protected GUI gui;
		
	public BestOfWorseManager(Agent ownerAgent,
							  						ReorganizationManager reorganizationManager,
							  						ReorganizationEngine reorganizationEngine,
							  						PotentialInOutAcquaintances potentialInOutAcquaintances, 
							  						GUI gui) {
		this.ownerAgent = ownerAgent;
		this.reorganizationManager = reorganizationManager;
		this.reorganizationEngine = reorganizationEngine;
		this.potentialInOutAcquaintances = potentialInOutAcquaintances;
		this.gui = gui;
		bestOfWorseForPosition = new BestOfWorse[2];
	}

	public void reinitialize() {
		reinitialize(1);
		reinitialize(2);
	}
	
	public void reinitialize(int pos) {
		bestOfWorseForPosition[pos-1] = null;
	}
	
	public void manageBestOfWorse(Agent agent, String dataType, int position, 
								  							AgentInterface interfaceType, int potentialValue) {
		//if (!(ownerAgent.getAcquaintancesManager().isAnInOutAcquaintance(agent, "OUT"))) {
		if (!(reorganizationManager.isAThresholdAcq(agent, new AgentInterface("IN")))) {
			boolean allreadyBestOfWorse = false;
			for (int i=0; i< reorganizationManager.numberOfINs(); i++)
				if (bestOfWorseForPosition[i] != null)
					if (agent.equals(bestOfWorseForPosition[i].getAgent()))
						allreadyBestOfWorse = true;
				
			if (!allreadyBestOfWorse) {
				if (isBetter(position, potentialValue)) {
					gui.println("Proposal is better than bestOfWorse or no bestOfWorse yet : " +
											"proposal = " + potentialValue);
					bestOfWorseForPosition[position-1] = new BestOfWorse(agent, dataType, position, 
																															 interfaceType, potentialValue);
				}
				else
					gui.println("Proposal is not better than bestOfWorse : " +
											"proposal = " + potentialValue + " | oldProposal = " + 
											(bestOfWorseForPosition[position-1]).getPotentialValue() +
											"from : " + (bestOfWorseForPosition[position-1]).getAgent().getAgentName());
			}
			else
				gui.println("Proposal is allready a bestOfWorse.");
		}
		else
			gui.println("Proposal is allready an InOutAcquaintance.");
	}
	
	protected boolean isEmpty(int pos) {
		return (bestOfWorseForPosition[pos-1] == null);
	}
	
	protected boolean isBetter(int pos, int potentialValue) {
		if (isEmpty(pos))
			return true;
		else
			if (reorganizationManager.getHigherLowerGoal().equals("HIGHER"))
				return (potentialValue >= ((bestOfWorseForPosition[pos-1]).getPotentialValue()));
			else
				return (potentialValue <= ((bestOfWorseForPosition[pos-1]).getPotentialValue()));
	}
	
	public void linkWithBestOfWorse(int pos) {
		gui.println("WaitingForProposalsTimer ran out => link with bestOfWorse.", 
								java.awt.Color.red);
		if (!isEmpty(pos)){
			Agent proposer = bestOfWorseForPosition[pos-1].getAgent();
			if (!(ownerAgent.getAcquaintancesManager().isAnInOutAcquaintance(proposer, "IN"))) {
				AgentInterface interfaceType = bestOfWorseForPosition[pos-1].getInterfaceType();
				String dataType = bestOfWorseForPosition[pos-1].getDataType();
				
				PotentialInOutAcquaintance acq;
				acq = new PotentialInOutAcquaintance(proposer, interfaceType, false);
				//potentialInOutAcquaintances.reInitializePotentialAcq();
				potentialInOutAcquaintances.addPotentialInOutAcquaintance(acq);
				if (ownerAgent.getAcquaintancesManager()
							.possessingIN(interfaceType.getPosition())) {
					reorganizationManager.cancelCooperation(interfaceType);
				}
				potentialInOutAcquaintances.setRequested(proposer, interfaceType, true);
				potentialInOutAcquaintances.setNewDataType(proposer, 
																									 interfaceType, dataType);
				reorganizationManager.getRequestManager().setRequestedIN(pos);
				RequestingCooperationMessage message 
					= new RequestingCooperationMessage(ownerAgent, interfaceType, pos, 
																						 reorganizationManager.getPotentialOutput());
				ownerAgent.fireMessage(message,proposer, "NEUTRAL", ownerAgent);
				reorganizationManager.requestFired();
		
				// ###DEBUG is the goal considered as reached when link with bestOfWorse ? No for now
				// Goal reached
	//			reorganizationManager.setGoalReached(true);
	//			if (ownerAgent.getAcquaintancesManager().possessingOUT())
	//				ownerAgent.fireMessage(new GoalReachedMessage(ownerAgent), 
	//						new AgentInterface("OUT", 1),
	//						ownerAgent);
	//			gui.println("Goal Reached(bestOfWorse) and notification done.", java.awt.Color.red);
			}
			else
				gui.println("Link with bestOfWorse failed => bestOfWorse allready an InOutAcquaintance : " +
										proposer.getAgentName(),
										java.awt.Color.red);
		}
		else {
			gui.println("Link with bestOfWorse failed => no bestOfWorse.", 
									java.awt.Color.red);
		}
	}
	
	public void showBestOfWorse() {
		gui.print("	BestOfWorse (IN1/IN2) : ", java.awt.Color.blue);
		BestOfWorse bow;
		for (int i=0; i< reorganizationManager.numberOfINs(); i++) {
			bow = bestOfWorseForPosition[i];
			if (bow != null)
				gui.print("[IN" + (i+1) + " = " + bow.getAgent().getAgentName() + ":" +
									bow.getPotentialValue() + "] / ", java.awt.Color.blue);
		}
	}

}
