package epe.reorganization.messageProcessors.contract;

	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.AcknowledgingCooperationMessage;
	import epe.communication.messages.contract.CancellingCooperationMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;
	import epe.reorganization.LinkMultiplicityManager;

public class AcknowledgingCooperationMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected Agent ownerAgent;
	  protected GUI gui;
	  protected PotentialInOutAcquaintances potentialInOutAcquaintances;
	  protected LinkMultiplicityManager linkMultiplicityManager;

  public AcknowledgingCooperationMessageProcessor(
           ReorganizationManager reorganizationManager,
           Agent ownerAgent, 
           GUI gui, 
           PotentialInOutAcquaintances potentialInOutAcquaintances,
		   		 LinkMultiplicityManager linkMultiplicityManager) {
    this.reorganizationManager = reorganizationManager;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.potentialInOutAcquaintances = potentialInOutAcquaintances;
    this.linkMultiplicityManager = linkMultiplicityManager;
  }

  public void processAcknowledgingCooperationMessage(Message receivedMessage) {
    Agent sender = receivedMessage.getSender();
    AcknowledgingCooperationMessage castMessage 
    	= (AcknowledgingCooperationMessage)receivedMessage;
    AgentInterface interfaceType = castMessage.getInterfaceType();
    String type = interfaceType.getType();
    int position = interfaceType.getPosition();
    String dataTypeProposed = interfaceType.getDataType();
    int potentialValue = castMessage.getPotentialValue();
    gui.println("Processing message AcknowledgingCooperation : ["
                + type + ":"
                + position + ":"
                + dataTypeProposed + " : potentialValue = " + potentialValue + "] from ["
                + castMessage.getSender().getAgentName() + "]");

    String dataTypeNeeded = reorganizationManager.dataTypeNeeded(type, position);
    String dataType = reorganizationManager.synchronizeDataTypes(dataTypeProposed,
    																														 dataTypeNeeded);
    if (dataType.equals("FAILLED")) {
    	dataTypeSynchronizationFailled(sender, interfaceType, dataTypeProposed);
    }
    else {
      interfaceType.setDataType(dataType);
      if (interfaceType.equals(new AgentInterface("IN"))) {
        manageINAcknowledgement(sender, interfaceType, position, castMessage, potentialValue);
      }
      else if (interfaceType.equals(new AgentInterface("OUT"))) {
        manageOUTAcknowledgement(sender, interfaceType);
      }
      else {
        SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
        ownerAgent.errorHappened();
        gui.println(ownerAgent.getAgentName()
                    + " : ERROR(while processing acknowledgment) : Agent not added"
                    , GUI.blue);
      }
    }
  }
  
  protected void dataTypeSynchronizationFailled(Agent sender, 
																								AgentInterface interfaceType,
  																							String dataTypeProposed) {
  	CancellingCooperationMessage message 
	  	= new CancellingCooperationMessage(ownerAgent,
	  																		 interfaceType);
  	ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
  	if (potentialInOutAcquaintances.isAPotentialAcquaintance(sender, 
  			new AgentInterface(interfaceType.getType(),
								  				 interfaceType.getPosition(),
								  				 dataTypeProposed))) {
  		potentialInOutAcquaintances
  		.removePotentialInOutAcquaintance(sender,
  				new AgentInterface(interfaceType.getType(),
  						interfaceType.getPosition(),
  						dataTypeProposed));
  	}
  	gui.println("Cancelling cooperation because of incompatible dataTypes",
  							GUI.red);
  	linkMultiplicityManager.manageLinkMultiplicityUpdate(interfaceType);
  }
 
  protected void manageINAcknowledgement(Agent sender, 
										 										 AgentInterface interfaceType,
  																			 int position, 
										 										 AcknowledgingCooperationMessage castMessage,
																			 	 int potentialValue) {
  	reorganizationManager.getRequestManager().requestedInAgentAnswered(position);
  	// Cancel old Acq if it exists (Higher/Lower feedback case)
  	if (ownerAgent.getAcquaintancesManager().possessingIN(interfaceType.getPosition())) {
  		reorganizationManager.cancelCooperation(interfaceType);
  	}
  	else
  		gui.println("DEBUG  : no IN[" 
				  				+ interfaceType.getPosition()
				  				+ "] acq."
				  				, GUI.orange);
  	
  	// Add the new acquaintance
  	gui.println(ownerAgent.getAgentName() + " : trying to add : " +
				  			sender.getAgentName() + "[IN" + position + "]" +
				  			" (while processing Acknowledging coop) : " +
				  			interfaceType.getType() + ":" +
				  			interfaceType.getPosition() + ":" +
				  			interfaceType.getDataType() + " : potentialValue = " + potentialValue, 
  							GUI.blue);
  	reorganizationManager.addInAcquaintance(sender,
																		  			interfaceType.getType(),
																		  			interfaceType.getPosition(),
																						interfaceType.getDataType());
  	if (potentialInOutAcquaintances.isAPotentialAcquaintance(sender, interfaceType))
  		potentialInOutAcquaintances.removePotentialInOutAcquaintance(sender, interfaceType);
  	// For the Higher/Lower Feedback
  	reorganizationManager.setPotentialInput(interfaceType.getPosition(), potentialValue);
  }
  
  protected void manageOUTAcknowledgement(Agent sender, AgentInterface interfaceType) {
  	int senderPrejudice = potentialInOutAcquaintances
  													.getRequestedPotentialInOutAcquaintance(sender, interfaceType)
  													.getPrejudice();
  	reorganizationManager.setCurrentImposingPrejudice(senderPrejudice);
  	reorganizationManager.addOutAcquaintance(sender,
																		  			 interfaceType.getType(),
																		  			 interfaceType.getPosition(),
																						 interfaceType.getDataType());
  	reorganizationManager.getRequestManager().requestedOutAgentAnswered();
  	
  	if (potentialInOutAcquaintances.isAPotentialAcquaintance(sender, interfaceType))
  		potentialInOutAcquaintances.removePotentialInOutAcquaintance(sender, interfaceType);
  	
  	// For the UNIQUE/MULTIPLE Link management
  	linkMultiplicityManager.manageLinkMultiplicityUpdate(interfaceType);
  }
  
}
