package epe.reorganization.messageProcessors.contract;

	import java.util.Vector;
	
	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.CancellingCooperationMessage;
	import epe.reorganization.LinkMultiplicityManager;
	

public class CancellingCooperationMessageProcessor {
  
	  protected Agent ownerAgent;
	  protected LinkMultiplicityManager linkMultiplicityManager;
	  protected GUI gui;
  
  public CancellingCooperationMessageProcessor(Agent ownerAgent,
											   											 LinkMultiplicityManager linkMultiplicityManager,
																	           	 GUI gui) {
    this.ownerAgent = ownerAgent;
    this.linkMultiplicityManager = linkMultiplicityManager;
    this.gui = gui;
  }

  public void processCancellingCooperationMessage(Message receivedMessage) {
    Agent sender = receivedMessage.getSender();
    CancellingCooperationMessage castMessage 
    	= (CancellingCooperationMessage)receivedMessage;
    AgentInterface interfaceType = castMessage.getInterfaceType();
    int position = interfaceType.getPosition();
    gui.println("Processing message CancellingCooperation : ["
                + interfaceType.getType() + ":"
                + position + ":"
                + interfaceType.getDataType() + "] from ["
                + castMessage.getSender().getAgentName() + "]");
    AgentInterface myType;
    Acquaintance acq;
    Vector<Object> positions;
    Acquaintances acquaintances = ownerAgent.getAcquaintancesManager().getAcquaintances();
    if (interfaceType.equals(new AgentInterface("IN"))) {
      myType = new AgentInterface("OUT");
    }
    else if (interfaceType.equals(new AgentInterface("OUT"))) {
      myType = new AgentInterface("IN");
    }
    else {
      SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
      ownerAgent.errorHappened();
      gui.println(ownerAgent.getAgentName()
                  + " : ERROR(while processing Cancelling) : unknown case",
                  java.awt.Color.red);
      myType = null;
    }
    if (myType != null) {
      positions = acquaintances .findAcquaintance(sender, myType.getType());
      if (positions.size()!=1) {
        SystemPrinter.printSystem("WARNING : " + ownerAgent.getAgentName());
        gui.println("WARNING : processing CancellingMessage : "
                    + "don't know which agent to cancel (2 cancels may have crossed each other)"
                    , java.awt.Color.red);
      }
      else {
        int rang = ((Integer)positions.elementAt(0)).intValue();
        acq = acquaintances.getAcquaintance(rang);
        gui.println("Acquaintance ["
                    + acq.getAgent().getAgentName() + ":"
                    + acq.getInterfaceType() + ":"
                    + acq.getInterfacePosition() + ":"
                    + acq.getDataType() + "]"
                    + " has cancelled cooperation : "
                    + " setting to NEUTRAL", java.awt.Color.blue);
        ownerAgent.getAcquaintancesManager().cancelCooperation(acq);

        // For the UNIQUE/MULTIPLE Link management
        linkMultiplicityManager.manageLinkMultiplicityUpdate(myType);
      }
    }
  }
  
}
