package epe.reorganization.messageProcessors.contract;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract
						.ProposingCooperationInWithPotentialValueMessage;
	import epe.communication.messages.contract.RequestingCooperationMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.ReorganizationEngine;
	import epe.reorganization.bestOfWorseManagement.BestOfWorseManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintance;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;

public class ProposingCooperationInWithPotentialValueMessageProcessor {

	  protected Agent ownerAgent;
	  protected ReorganizationManager reorganizationManager;
	  protected ReorganizationEngine reorganizationEngine;
	  protected GUI gui;
	  protected PotentialInOutAcquaintances potentialInOutAcquaintances;
	  protected BestOfWorseManager bestOfWorseManager;

  public ProposingCooperationInWithPotentialValueMessageProcessor(
           ReorganizationManager reorganizationManager,
		   		  ReorganizationEngine reorganizationEngine,
           Agent ownerAgent, 
           GUI gui,
           PotentialInOutAcquaintances potentialInOutAcquaintances,
		   		 BestOfWorseManager bestOfWorseManager) {
    this.reorganizationManager = reorganizationManager;
    this.reorganizationEngine = reorganizationEngine;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.potentialInOutAcquaintances = potentialInOutAcquaintances;
    this.bestOfWorseManager = bestOfWorseManager;
  }

  public void processProposingCooperationInWithPotentialValueMessage(Message receivedMessage) {
  	reorganizationManager.reInitializenumberOfNeedInHLNCSWithoutAnswer();
    ProposingCooperationInWithPotentialValueMessage castMessage 
      = (ProposingCooperationInWithPotentialValueMessage)receivedMessage;
    
    Agent sender = receivedMessage.getSender();
    AgentInterface type = castMessage.getInterfaceType();
    String dataTypeProposed = castMessage.getDataType();
    boolean inNeedInOutNcs = castMessage.isInNeedInOutNcs();

		gui.println("HIGHER/LOWER Reorganization : proposingIN with potentialValue message : "
		            + "Sender Potential = " + castMessage.getPotentialValue()
								+ " / InputThresholds(IN1/IN2) = " + reorganizationManager.getInputThreshold(1)
								+ "/" + reorganizationManager.getInputThreshold(2)
								, java.awt.Color.orange);
    
    for (int i=1; i<=reorganizationManager.numberOfINs(); i++) {
    	String dataType;
    	dataType = reorganizationManager
	    	.synchronizeDataTypes(dataTypeProposed,
	    												reorganizationManager.dataTypeNeeded("IN",i));
    	AgentInterface interfaceType = new AgentInterface("IN", i, dataType);
    	int potentialValue = castMessage.getPotentialValue(); 
    	
      if ((reorganizationManager.isDataRelevant("IN", i, type.getDataType()))
				&& (reorganizationManager.isBetterForGoal(potentialValue, 
				    reorganizationManager.getInputThreshold(i)))					
      	&&(!reorganizationManager.isGoalReached())
     		// ###DEBUG Prevent InputOuput crossing (but no good for AgentFirstTime) 
      	&&(!((ownerAgent.getAcquaintancesManager()
      	     .isAnInOutAcquaintance(sender, "IN"))
					  || (ownerAgent.getAcquaintancesManager()
					 					 		  .isAnInOutAcquaintance(sender, "OUT"))))) {
      	// Adding to potentialAcquaintances
      	PotentialInOutAcquaintance acq;
        acq = new PotentialInOutAcquaintance(sender, interfaceType, inNeedInOutNcs);
        potentialInOutAcquaintances.addPotentialInOutAcquaintance(acq);
        // if not already waiting for an answer => immediate link
      	if ((!reorganizationManager.getRequestManager().hasRequestedIN(i))
      			&& (!reorganizationManager.getPotentialInOutAcquaintances()
      															  .isAlreadyRequestedAsIN(acq.getAgent()))) {
      		if (! reorganizationManager.getReorganizationEngine().checkCycle("IN", sender)) {      		
		        // ###DEBUG received proposal : check if good technique to add acq and send goalReached
		        gui.println("Better proposal and goal not reached yet => immediate link.", 
		        						java.awt.Color.red);
		        if (ownerAgent.getAcquaintancesManager()
		        							.possessingIN(interfaceType.getPosition())) {
		        	reorganizationManager.cancelCooperation(interfaceType);
		        }
		        
		        potentialInOutAcquaintances.setRequested(sender, interfaceType, true);
		        potentialInOutAcquaintances.setNewDataType(sender, interfaceType, dataType);
		        reorganizationManager.getRequestManager().setRequestedIN(i);
		        RequestingCooperationMessage message 
			        = new RequestingCooperationMessage(ownerAgent, interfaceType, i, 
			        																	 reorganizationManager.getPotentialOutput());
		        ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
		        reorganizationManager.requestFired();
      		}
     			else // cycle detected
     				gui.println("Cycle detected : No immediate link possible.", GUI.red);
      	}
        
//        // Goal reached (because requestIn always accepted)
//        reorganizationManager.setGoalReached(true);
//        if (ownerAgent.getAcquaintancesManager().possessingOUT())
//        	ownerAgent.fireMessage(new GoalReachedMessage(ownerAgent, true), 
//									        			 new AgentInterface("OUT", 1), ownerAgent);
//        gui.println("Goal Reached and notification done.", java.awt.Color.red);
     // => useless : done by the setPotentialInput during acknowledgement
      }
      else {
      	bestOfWorseManager.manageBestOfWorse(sender, dataType, i, 
					   			             interfaceType, potentialValue);
      	// Worse proposal => if betterThanWorse then keep in case no better at the end
      	
      }
    }
  }
}
