package epe.reorganization.messageProcessors.contract;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.ProposingCooperationMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintance;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;

public class ProposingCooperationMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected Agent ownerAgent;
	  protected GUI gui;
	  protected PotentialInOutAcquaintances potentialInOutAcquaintances;
	  
	  boolean needNoMore = true;
    boolean relevantNoMore = true;
    boolean isLastAcquaintance = true;
    PotentialInOutAcquaintance rejectedAcq = null;

  public ProposingCooperationMessageProcessor(
           ReorganizationManager reorganizationManager,
           Agent ownerAgent, 
           GUI gui,
           PotentialInOutAcquaintances potentialInOutAcquaintances) {
    this.reorganizationManager = reorganizationManager;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.potentialInOutAcquaintances = potentialInOutAcquaintances;
  }

  public void processProposingCooperationMessage(Message receivedMessage) {
		reorganizationManager.reInitializenumberOfNeedInHLNCSWithoutAnswer();
    // If it is still in need, the agent adds the proposition to its list

    ProposingCooperationMessage castMessage = (ProposingCooperationMessage)receivedMessage;
    Agent sender = castMessage.getSender();
    AgentInterface interfaceTypeProposed = castMessage.getInterfaceType();
    String dataTypeProposed = castMessage.getDataType();
    boolean inNeedInOutNcs = castMessage.isInNeedInOutNcs();
    int proposedINnumber = castMessage.getProposedINnumber();
    
    gui.println("Processing message ProposingCooperation : ["
                + interfaceTypeProposed.getType() + ":"
                + interfaceTypeProposed.getPosition() + ":"
                + dataTypeProposed + "] from ["
                + sender.getAgentName() + " : proposedInNumber = "
    						+ proposedINnumber + "]");

    // IN proposition
    if (interfaceTypeProposed.equals(new AgentInterface("IN")))
      processProposingIN(sender, dataTypeProposed, inNeedInOutNcs, 
      									 castMessage, interfaceTypeProposed);
    
    // OUT proposition
    if (interfaceTypeProposed.equals(new AgentInterface("OUT")))
    	processProposingOUT(sender, dataTypeProposed, inNeedInOutNcs, proposedINnumber,
    											castMessage, interfaceTypeProposed);
  }
 
  protected void processProposingIN(Agent sender,
  																	String dataTypeProposed,
																		boolean inNeedInOutNcs,
																		ProposingCooperationMessage castMessage,
																		AgentInterface interfaceTypeProposed) {
  	boolean addedToPotentialAcqs = false;
  	for (int i=1; i<=reorganizationManager.numberOfINs(); i++) {
	    if (reorganizationManager.isDataRelevant("IN", i, dataTypeProposed)) {
	      relevantNoMore = false;
	      if (reorganizationManager.needIN(i)) {
	        needNoMore = false;
	        String dataType = reorganizationManager
	                            .synchronizeDataTypes(dataTypeProposed,
	                                                  reorganizationManager.dataTypeNeeded("IN",i));
	        AgentInterface interfaceType = new AgentInterface("IN", i, dataType);
	        PotentialInOutAcquaintance acq;
	        acq = new PotentialInOutAcquaintance(sender, interfaceType, inNeedInOutNcs);
	        // => try to change the agent's partner only when not in Higher/Lower reorg.
	        if ((!reorganizationManager
	              .getPotentialInOutAcquaintances()
	              .isLastInOutAcquaintance(acq))
	        	 | (!reorganizationManager.lastInOutAcquaintancesManagementEnabled())){
	          potentialInOutAcquaintances.addPotentialInOutAcquaintance(acq);
	          addedToPotentialAcqs = true;
	          isLastAcquaintance = false;
	        } else {
	          rejectedAcq = acq;
	          if (reorganizationManager.getPotentialInOutAcquaintances().getNumberOfRejects(acq) 
	            > reorganizationManager.getMaxNumberOfRejects()) {
	            reorganizationManager
	              .getPotentialInOutAcquaintances()
	              .removeLastInOutAcquaintance(rejectedAcq);
	          }
	        }
	      }
	    }
	  }
  	if (addedToPotentialAcqs)
  		reorganizationManager.getReorganizationEngine().checkCycle("IN", sender);
  	
  	// Treat problem cases which may have appeared
    treatProblems(castMessage, interfaceTypeProposed, dataTypeProposed);
  }
  
  protected void processProposingOUT(Agent sender,
  																	 String dataTypeProposed,
																		 boolean inNeedInOutNcs,
																		 int proposedINnumber,
																		 ProposingCooperationMessage castMessage,
																		 AgentInterface interfaceTypeProposed) {
  	boolean addedToPotentialAcqs = false;
  	if (reorganizationManager.isDataRelevant("OUT", 1, dataTypeProposed)) {
	    relevantNoMore = false;
	    if (reorganizationManager.needOUT()) {
	      needNoMore = false;
	      String dataType = reorganizationManager
	                          .synchronizeDataTypes(dataTypeProposed,
	                                                reorganizationManager.dataTypeNeeded("OUT",1));
	      AgentInterface interfaceType = new AgentInterface("OUT", 1, dataType);
	      PotentialInOutAcquaintance acq;
	      acq = new PotentialInOutAcquaintance(sender, proposedINnumber, 
	      																		 interfaceType, inNeedInOutNcs);
	      //potentialInOutAcquaintances.addPotentialInOutAcquaintance(acq);
	      
	//          // Removed because not needed for Higer/lower reorganization
	//          // Originally needed to ensure trying to compute a different value
	//          // => try to change the agent's partner
	      if ((!reorganizationManager
	      		  .getPotentialInOutAcquaintances()
	      			.isLastInOutAcquaintance(acq))
	      	| (!reorganizationManager.lastInOutAcquaintancesManagementEnabled())) {
	        potentialInOutAcquaintances.addPotentialInOutAcquaintance(acq);
	        reorganizationManager.setReceivedPropositionOutFromAgentInNeed(true);
	        gui.println("Adding potentialAcq", java.awt.Color.orange);
	        addedToPotentialAcqs = true;
	        isLastAcquaintance = false;
	      } else {
	        rejectedAcq = acq;
	        if (reorganizationManager.getPotentialInOutAcquaintances().getNumberOfRejects(acq) 
	            > reorganizationManager.getMaxNumberOfRejects()) {
	          reorganizationManager
	            .getPotentialInOutAcquaintances()
	            .removeLastInOutAcquaintance(rejectedAcq);
	        }
	      }
	    }
  	}
  	
  	if (addedToPotentialAcqs)
  		reorganizationManager.getReorganizationEngine().checkCycle("OUT", sender);
  	
  	// Treat problem cases which may have appeared
    treatProblems(castMessage, interfaceTypeProposed, dataTypeProposed);
  }
  
  protected void treatProblems(ProposingCooperationMessage castMessage,
															 AgentInterface interfaceTypeProposed,
															 String dataTypeProposed) {
	  if (relevantNoMore)
      gui.println("Agent [" + ownerAgent.getAgentName()
                  + "] refusing ProposingCooperation from agent ["
                  + castMessage.getSender().getAgentName()
                  + "] : relevant no more : ["
                  + interfaceTypeProposed.getType() + ":"
                  + dataTypeProposed + "]", java.awt.Color.magenta);
    else if (needNoMore)
      gui.println("Agent [" + ownerAgent.getAgentName()
                  + "] refusing ProposingCooperation from agent ["
                  + castMessage.getSender().getAgentName()
                  + "] : need no more : ["
                  + interfaceTypeProposed.getType() + ":"
                  + dataTypeProposed + "]", java.awt.Color.magenta);
    
    else if (isLastAcquaintance) {
    	// only possible if not in higher/LowerReorganization
    	// => try to change the agent's partner
      reorganizationManager.getPotentialInOutAcquaintances().increaseNumberOfRejects(rejectedAcq);
      gui.println("Agent [" + ownerAgent.getAgentName()
                  + "] refusing ProposingCooperation from agent ["
                  + castMessage.getSender().getAgentName()
                  + "] : isLastAcquaintance : ["
                  + interfaceTypeProposed.getType() + ":"
                  + dataTypeProposed + "] _ NumberOfRejects : "
                  + reorganizationManager.getPotentialInOutAcquaintances()
                                         .getNumberOfRejects(rejectedAcq)
                  , java.awt.Color.magenta);
      gui.printDate();
      gui.print("LastInOutAcquaintances : ", java.awt.Color.cyan);
      potentialInOutAcquaintances.showAllLastAcquaintances();
    }
  }
  
}