package epe.reorganization.messageProcessors.contract;

	import java.awt.Color;
	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.AcknowledgingCooperationMessage;
	import epe.communication.messages.contract.RefusingCooperationMessage;
	import epe.communication.messages.contract.RequestingCooperationMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;
	import epe.reorganization.LinkMultiplicityManager;


public class RequestingCooperationMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected Agent ownerAgent;
	  protected GUI gui;
	  protected PotentialInOutAcquaintances potentialInOutAcquaintances;
	  protected LinkMultiplicityManager linkMultiplicityManager;

  public RequestingCooperationMessageProcessor(
          ReorganizationManager reorganizationManager,
          Agent ownerAgent, 
          GUI gui, 
          PotentialInOutAcquaintances potentialInOutAcquaintances, 
		  		LinkMultiplicityManager linkMultiplicityManager) {
    this.reorganizationManager = reorganizationManager;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.potentialInOutAcquaintances = potentialInOutAcquaintances;
    this.linkMultiplicityManager = linkMultiplicityManager;
  }

  public void processRequestingCooperationMessage(Message receivedMessage) {
    Agent sender = receivedMessage.getSender();
    RequestingCooperationMessage castMessage = (RequestingCooperationMessage)receivedMessage;
    AgentInterface interfaceTypeAsked = castMessage.getInterfaceType();
    String dataTypeAsked = interfaceTypeAsked.getDataType();
    int proposedINnumber = castMessage.getProposedINnumber();
    int potentialValue = castMessage.getPotentialValue();
    gui.println("Processing message RequestingCooperation : ["
                + interfaceTypeAsked.getType() + ":"
                + interfaceTypeAsked.getPosition() + ":"
                + dataTypeAsked + " : potentialValue = " + potentialValue + "] from ["
                + castMessage.getSender().getAgentName() + "]");
    // the agent already decided that it had the competences : cf. processNeedInNCS but
    // it may have changed because of GENERIC becoming INTEGER/BOOLEAN
    if (interfaceTypeAsked.equals(new AgentInterface("IN"))) {
      processRequestIN(sender, interfaceTypeAsked, dataTypeAsked);
    }
    else if (interfaceTypeAsked.equals(new AgentInterface("OUT"))) {
      processRequestOUT(sender, interfaceTypeAsked, dataTypeAsked, 
    										proposedINnumber, potentialValue);
    }
  }

  protected void processRequestIN(Agent sender, 
                                  AgentInterface interfaceTypeAsked,
                                  String dataTypeAsked) {
    // The agent can always provide a value if it is competent
    if (reorganizationManager.isRelevant(interfaceTypeAsked, 1)) {
      String type = "NEUTRAL";
      String dataType = reorganizationManager
                        .synchronizeDataTypes(dataTypeAsked,
                                              reorganizationManager
                                                .dataTypeNeeded("OUT",1));
      AgentInterface interfaceType = new AgentInterface("OUT", 1, dataType);
      boolean accepting = false;
      // Check for cycle
      if (! reorganizationManager.getReorganizationEngine().checkCycle("OUT", sender)) {
	      if (potentialInOutAcquaintances
	      			.requestingCooperationFrom(sender, interfaceType, interfaceTypeAsked.getPosition())) {
	        // The agent already sent a request to this agent
	        reorganizationManager.getRequestManager().requestedOutAgentAnswered();
	        gui.println("Allready requesting this agent " + 
	        						"=> accepting but waiting for acknowledgement.", GUI.blue);
	        accepting = true;
	      }
	      else {
	      	if (potentialInOutAcquaintances.requestingCooperationFrom(sender))
	      		// the requester is allready requested by this agent for another position
	      		accepting = false;
	      	else {
	      		accepting = true;
		        type = "OUT";
		        reorganizationManager
		          .addOutAcquaintance(sender,
		                              interfaceType.getType(),
		                              interfaceType.getPosition(),
		                              interfaceType.getDataType());
		        if (potentialInOutAcquaintances.isAPotentialAcquaintance(sender, interfaceType))
		          potentialInOutAcquaintances.removePotentialInOutAcquaintance(sender, interfaceType);
	      	}
	      }
      }
      else // cycle detected
	  		gui.println("Cycle detected : refusing request.", GUI.red);
      
      if (accepting) {
	      interfaceTypeAsked.setDataType(dataType);
	      AcknowledgingCooperationMessage message 
	      	= new AcknowledgingCooperationMessage(ownerAgent,
	                                              interfaceTypeAsked,
	                                              reorganizationManager.getPotentialOutput());
	      ownerAgent.fireMessage(message, sender, type, ownerAgent);
	      // For the UNIQUE/MULTIPLE Link management
	      linkMultiplicityManager.manageLinkMultiplicityUpdate(interfaceType);
      }
      else {
      	gui.println("Refusing request : ["
                  + interfaceTypeAsked.getType() + ":"
                  + interfaceTypeAsked.getPosition() + ":"
                  + interfaceTypeAsked.getDataType() + "] from "
                  + sender.getAgentName()
                  + " : allready requested for another position or cycle detected.", 
									GUI.magenta);
      	refusingCooperation(interfaceTypeAsked, sender);
      }
    }
    else { // The agent is not relevant
      gui.println(ownerAgent.getAgentName() + " isn't relevant any more");
      refusingCooperation(interfaceTypeAsked, sender);
    }
  }

  protected void processRequestOUT(Agent sender, 
                                   AgentInterface interfaceTypeAsked, 
                                   String dataTypeAsked,
                                   int proposedINnumber,
                                   int potentialValue) {
    boolean accepting = false;
    boolean alreadyRequested = false;
    int acceptedPosition = 0;
	  // Either the agents is also in need and it accepts
	  // or it is in a higher/Lower reorganization so it has to accept 
	  // to find some acceptable organization
	  if ((reorganizationManager.isRelevant(interfaceTypeAsked, proposedINnumber))
	      &&((reorganizationManager.needIN(proposedINnumber)) 
	          || reorganizationManager.isInHigherLowerReorganization())) {
	  	// Check for cycle
	  	if (! reorganizationManager.getReorganizationEngine().checkCycle("IN", sender)) {
		    // Cancel the old acq if it exists
		    if ((reorganizationManager.isInHigherLowerReorganization()) 
		     && (ownerAgent.getAcquaintancesManager().possessingIN(proposedINnumber))) {
		      reorganizationManager.cancelCooperation(new AgentInterface("IN", proposedINnumber));
		      accepting = true;
		    }
		    if (reorganizationManager.getRequestManager().hasRequestedIN(proposedINnumber)) {
		      // The agent is already waiting for a confirm from someone
		      if (potentialInOutAcquaintances
		      			.requestingCooperationFrom(sender, new AgentInterface("IN"), proposedINnumber)) {
		        // The agent is waiting for the confirm from the agent who sends the request
		        // (the 2 messages passed each other) => accept but wait for confirm
		        accepting = true;
		        acceptedPosition = proposedINnumber;
		        alreadyRequested = true;
		      } // else :  The agent is waiting for the confirm from someone else => refuse
		    }
		    else {  // The agent is not waiting for a confirm => accept spontaneously
		      accepting = true;
		      acceptedPosition = proposedINnumber;
		    }
		  // else : The agent refuses because it isn't avaible anymore
		  }
	  	else // cycle detected
	  		gui.println("Cycle detected : refusing request.", GUI.red);
	  }
    
    if (accepting) { // The agent accepts the request
      acceptingRequestOut(dataTypeAsked, 
                          acceptedPosition, 
                          alreadyRequested, 
                          sender, 
                          interfaceTypeAsked,
                          potentialValue);
      checkPrejudiceState(acceptedPosition, potentialValue);
    }
    else { // The agent refuses the request
      gui.println("Refusing request : ["
                  + interfaceTypeAsked.getType() + ":"
                  + interfaceTypeAsked.getPosition() + ":"
                  + interfaceTypeAsked.getDataType() + "] from "
                  + sender.getAgentName()
                  + " : need no more, relevant no more, or cycle detected.", 
									GUI.magenta);
      refusingCooperation(interfaceTypeAsked, sender);
    }
  }
  
  
  protected void acceptingRequestOut(String dataTypeAsked,
			                               int acceptedPosition, 
			                               boolean alreadyRequested,
			                               Agent sender,
			                               AgentInterface interfaceTypeAsked,
			                               int potentialValue) {
    String type = "NEUTRAL";
    String dataType = reorganizationManager
                      .synchronizeDataTypes(dataTypeAsked,
                                            reorganizationManager
                                              .dataTypeNeeded("IN",
                                                              acceptedPosition));
    if (!alreadyRequested) {
      type = "IN";
      AgentInterface interfaceType = new AgentInterface("IN", acceptedPosition, dataType);
      gui.println(ownerAgent.getAgentName() + " : trying to add : " +
                  sender.getAgentName() + "[IN]" +
                  " (while processing requesting coop) : ["
                  + interfaceType.getType() + ":"
                  + interfaceType.getPosition() + ":"
                  + interfaceType.getDataType() + " : potentialValue = " + potentialValue + "]", 
      						java.awt.Color.blue);

      reorganizationManager.addInAcquaintance(sender,
                                              interfaceType.getType(),
                                              interfaceType.getPosition(),
                                              interfaceType.getDataType());
      if (potentialInOutAcquaintances.isAPotentialAcquaintance(sender, interfaceType))
        potentialInOutAcquaintances.removePotentialInOutAcquaintance(sender, interfaceType);
      // For the Higher/Lower Feedback
      reorganizationManager.setPotentialInput(interfaceType.getPosition(), 
                                              potentialValue);
    }
    interfaceTypeAsked.setDataType(dataType);
    AcknowledgingCooperationMessage message 
    	= new AcknowledgingCooperationMessage(ownerAgent, interfaceTypeAsked, 0);
    ownerAgent.fireMessage(message, sender, type, ownerAgent);
  }
  
  
  protected void refusingCooperation(AgentInterface interfaceTypeAsked, 
                                     Agent sender) {
  	RefusingCooperationMessage message 
  		= new RefusingCooperationMessage(ownerAgent, interfaceTypeAsked);
    ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
  }
  
  protected void checkPrejudiceState(int position, int potentialValue) {
  	int oldInput = reorganizationManager.getInputThreshold(position);
  	int newInput = potentialValue;
  	gui.println("HIGHER/LOWER Reorganization : requestingCooperation[OUT] message : "
  			+ "InputThreshold = " + oldInput
  			+ " / PotentialValue = " + newInput, java.awt.Color.orange);
  	if (!reorganizationManager.isBetterForGoal(newInput, oldInput)) {
  		// In a prejudice state
  		gui.println("Agent now in prejudice.", Color.red);
  		reorganizationManager.setInPrejudice(true);
  	}
  	else if (reorganizationManager.isInPrejudice()) {
  		// Not in a prejudice state anymore
  		gui.println("Agent no longer in prejudice.", Color.red);
  		reorganizationManager.setInPrejudice(false);
  	}
  }
  
}
