package epe.reorganization.messageProcessors.information;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.information.CyclePingMessage;
	import epe.reorganization.ReorganizationEngine;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintances;

public class CyclePingMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected ReorganizationEngine reorganizationEngine;
	  protected Agent ownerAgent;
	  protected GUI gui;

  public CyclePingMessageProcessor(ReorganizationManager reorganizationManager,
  																 ReorganizationEngine reorganizationEngine,
	                                 Agent ownerAgent,
	                                 GUI gui) {
    this.reorganizationManager = reorganizationManager;
    this.reorganizationEngine = reorganizationEngine;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
  }
  
  public void processCyclePingMessage(Message receivedMessage) {
  	Agent sender = receivedMessage.getSender();
  	CyclePingMessage castMsg = (CyclePingMessage)receivedMessage;
  	String direction = castMsg.getDirection();
  	Agent testedAgent = castMsg.getTestedAgent();
  	Agent originAgent = castMsg.getOriginAgent();
  	
  	gui.println("Processing message CyclePing [direction = " + 
  							direction + " / testedAgent = " + 
  							testedAgent.getAgentName() + " / origin = " +
								originAgent.getAgentName() + "] from [" + 
								sender.getAgentName() + "]");
    if (originAgent.equals(ownerAgent)) {
    	// received own message => cycle detected => removePotentialAcq
	    gui.println("Cycle detected.", GUI.red);
	    PotentialInOutAcquaintances potentialAcqs 
				= reorganizationManager.getPotentialInOutAcquaintances();
	    AgentInterface interfaceType = new AgentInterface(direction);
	    if (potentialAcqs.isAPotentialAcquaintance(testedAgent, interfaceType)) {
	    	gui.println("Trying to remove potentialAcq [" + testedAgent.getAgentName() + 
	    							" / " + direction + "]");
	      potentialAcqs.removePotentialInOutAcquaintance(testedAgent, interfaceType);
	    }
	    else
	    	gui.println("But not a potentialAcq : [" + originAgent.getAgentName() + "/" +
	    							interfaceType.getType() + interfaceType.getPosition() + "]", GUI.red);
	    reorganizationEngine.setCycleDetected(true);
    }
    else {
    	// propagate to INs or OUT depending of "direction" of message
    	gui.printDate();
    	gui.print("Propagating CyclePing...", GUI.red);
    	boolean sent = false;
    	if (direction.equals("IN"))
	    	for (int i=1; i<= reorganizationManager.numberOfINs(); i++)
		  		if (ownerAgent.getAcquaintancesManager().possessingIN(i)) {
		  			gui.println("");
		  			ownerAgent.fireMessage(new CyclePingMessage(direction, testedAgent, 
		  																									originAgent, ownerAgent), 
											        		 new AgentInterface(direction, i), ownerAgent);
		  			sent = true;
		  		}
    	else if (direction.equals("OUT"))
	  		if (ownerAgent.getAcquaintancesManager().possessingOUT()) {
	  			gui.println("");
	  			ownerAgent.fireMessage(new CyclePingMessage(direction, testedAgent, 
	  																									originAgent, ownerAgent), 
										        		 new AgentInterface(direction, i), ownerAgent);
	  			sent = true;
	  		}
    		
	  	if (!sent)
	  		gui.print("no propagation possible [" + direction + "]", GUI.red); gui.println("");
    }
  }
  
}
