package epe.reorganization.messageProcessors.information;

	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.CancellingCooperationMessage;
import epe.communication.messages.information.DataTypeChangedMessage;
	import epe.reorganization.ReorganizationManager;
	import java.util.Vector;
	import epe.reorganization.LinkMultiplicityManager;

public class DataTypeChangedMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected Agent ownerAgent;
	  protected GUI gui;
	  protected LinkMultiplicityManager linkMultiplicityManager;

  public DataTypeChangedMessageProcessor(ReorganizationManager reorganizationManager,
                                         Agent ownerAgent, 
                                         GUI gui, 
										 										 LinkMultiplicityManager linkMultiplicityManager) {
    this.reorganizationManager = reorganizationManager;                                           
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.linkMultiplicityManager = linkMultiplicityManager;
  }

  public void processDataTypeChangedMessage(Message receivedMessage) {
    Agent sender = receivedMessage.getSender();
    DataTypeChangedMessage castMessage 
    	= (DataTypeChangedMessage)receivedMessage;
    AgentInterface interfaceType = castMessage.getInterfaceType();
    int position = interfaceType.getPosition();
    gui.println("Processing message DataTypeChanged : ["
                + interfaceType.getType() + ":"
                + position + ":"
                + interfaceType.getDataType() + "] from ["
                + castMessage.getSender().getAgentName() + "]");
    AgentInterface myType;
    Acquaintance acq;
    Vector<Object> positions;
    Acquaintances acquaintances = ownerAgent.getAcquaintancesManager().getAcquaintances();
    if (interfaceType.equals(new AgentInterface("IN"))) {
      myType = new AgentInterface("OUT");

    }
    else if (interfaceType.equals(new AgentInterface("OUT"))) {
      myType = new AgentInterface("IN");
    }
    else {
      SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
      ownerAgent.errorHappened();
      gui.println(ownerAgent.getAgentName()
      + " : ERROR(while processing DataTypeChanged) : unknown case",
      java.awt.Color.red);
      myType = null;
    }
    if (myType != null) {
      positions = acquaintances .findAcquaintance(sender, myType.getType());
      if (positions.size()!=1) {
        SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
        ownerAgent.errorHappened();
        gui.println("ERROR : processing DataTypeChanged : "
                    + "don't know which agent to change Data", java.awt.Color.red);
      }
      else {
        int rang = ((Integer)positions.elementAt(0)).intValue();
        acq = acquaintances.getAcquaintance(rang);
        if (acq.getDataType() == "GENERIC") {
          acq.setDataType(interfaceType.getDataType());
          gui.println("Acquaintance ["
                      + acq.getAgent().getAgentName() + ":"
                      + acq.getInterfaceType() + ":"
                      + acq.getInterfacePosition() + ":"
                      + acq.getDataType() + "]"
                      + " has changed its DataType : "
                      + " changed accepted and modification were done", java.awt.Color.blue);
          reorganizationManager.checkDataTypes();
        }
        else if (acq.getDataType() != interfaceType.getDataType()) {
          gui.println("Acquaintance ["
                      + acq.getAgent().getAgentName() + ":"
                      + acq.getInterfaceType() + ":"
                      + acq.getInterfacePosition() + ":"
                      + acq.getDataType() + "]"
                      + " has changed its DataType : "
                      + " incompatible change : cancellingCooperation", java.awt.Color.blue);
          Agent adressee = acq.getAgent();
          AgentInterface type = new AgentInterface(acq.getInterfaceType());
          CancellingCooperationMessage message 
          	= new CancellingCooperationMessage(ownerAgent, type);
          ownerAgent.fireMessage(message, adressee, acq.getInterfaceType(), ownerAgent);
          ownerAgent.getAcquaintancesManager().cancelCooperation(acq);
          linkMultiplicityManager.manageLinkMultiplicityUpdate(acq.getAgentInterface());
        }
        // else : already the same DataType : do nothing
      }
    }
  }
  
}
