package epe.reorganization.messageProcessors.ncs;

	import epe.gui.GUI;
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.reorganization.ReorganizationMessageProcessor;

public class NCSInadequateDataMessageProcessor {
  
	  protected ReorganizationMessageProcessor owner;
	  protected Agent ownerAgent;
	  protected GUI gui;
	
	  // Enables a total destructive feedback treatment stategy
	  protected boolean destroyAllLinksStrategy = true;
	  
  public NCSInadequateDataMessageProcessor(ReorganizationMessageProcessor owner, 
                                           Agent ownerAgent, 
                                           GUI gui) {
    this.owner = owner;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
  }

  public void processNCSinadequateDataMessage(Message receivedMessage) {
    // The value the agent sended may be inadequate
    // => make a probalistic choice depending of the TrustValues so as to choose
    // which link to decrease
    // unless the agent is already reorganizing
    // Note : if OUT is chosen => weakening link (the agent think it is sending to the wrong agent)
    //        if INi is chosen => propagating (the agent think it is getting a wrong value)
    // Update : so as not to defavorize agents near AgentOUT, propagating all the time
    gui.println("Processing message inadequateData from ["
                + receivedMessage.getSender().getAgentName() + "]"
                , java.awt.Color.orange);
    if (destroyAllLinksStrategy) {
      for (int i = 1; i <= ownerAgent.getNumberOfINs(); i++)
         owner.targetedPropagateNCS(receivedMessage, new AgentInterface("IN", i));
      ownerAgent.getAcquaintancesManager()
								.decreaseTrustValue(new AgentInterface("OUT"), "inadequateData");
    }
    else
      if (ownerAgent.inCooperativeState()) {
        int outTrustValue = getOutTrustValue(receivedMessage);
        int in1TrustValue = getIn1TrustValue();
        int in2TrustValue = getIn2TrustValue();
        AgentInterface decreaseChoice = probabilisticChoice(in1TrustValue , 
        																										in2TrustValue, 
																														outTrustValue);
        gui.println("DecreaseChoice = [" + decreaseChoice.getType() 
                    + decreaseChoice.getPosition() 
                    + "].", java.awt.Color.orange);
        if (decreaseChoice.getType() == "OUT") {
          gui.println("inadequateData : decreasing [" + 
                      decreaseChoice.getType() + decreaseChoice.getPosition() +
                      "] trustValue.", 
                      java.awt.Color.orange);
          ownerAgent.getAcquaintancesManager().decreaseTrustValue(decreaseChoice, "inadequateData");
        } else if (decreaseChoice.getType() == "IN") {
          gui.println("inadequateData : propagating to [" +
                      decreaseChoice.getType() + decreaseChoice.getPosition() + "].", 
                      java.awt.Color.orange);
          owner.targetedPropagateNCS(receivedMessage, decreaseChoice);
        }
      }
  }  
  
  protected int getIn1TrustValue() {
    Acquaintances acqs = ownerAgent.getAcquaintancesManager().getAcquaintances();
    Acquaintance acq;
    acq = acqs.getAcquaintance("IN", 1);
    if (acq == null)
      return 0;
    else
      return acq.getTrustValue().getValue();
  }
  
  public int getIn2TrustValue() {
    Acquaintances acqs = ownerAgent.getAcquaintancesManager().getAcquaintances();
    Acquaintance acq;
    acq = acqs.getAcquaintance("IN", 2);
    if (acq == null)
      return 0;
    else
      return acq.getTrustValue().getValue();
  }
  
  public int getOutTrustValue(Message receivedMessage) {
    Acquaintances acqs = ownerAgent.getAcquaintancesManager().getAcquaintances();
    Acquaintance acq;
    Agent sender = receivedMessage.getSender();
    acq = acqs.getAcquaintance(sender, "OUT");
    if (acq == null)
      return 0;
    else
      return acq.getTrustValue().getValue();
  }
  
 protected AgentInterface probabilisticChoice(int in1TrustValue, int in2TrustValue, int outTrustValue) {
    // a trustValue of 0 meaning that there is no IN/OUT or that it cannot be chosen
    int minimalProbabilityModifier = 1; // ensure even a trust of 100 can be chosen
    int in1ChooseValue = 0;
    int in2ChooseValue = 0;
    int outChooseValue = 0;
    if (in1TrustValue != 0) in1ChooseValue = (100 + minimalProbabilityModifier) - in1TrustValue ;
    if (in2TrustValue != 0) in2ChooseValue = (100 + minimalProbabilityModifier) - in2TrustValue ;
    if (outTrustValue != 0) outChooseValue = (100 + minimalProbabilityModifier) - outTrustValue ;
    int sum = in1ChooseValue + in2ChooseValue + outChooseValue;
    double in1Ratio = in1ChooseValue*1.0 / sum*1.0;
    double in2Ratio = in2ChooseValue*1.0 / sum*1.0;
    double outRatio = outChooseValue*1.0 / sum*1.0;
    double in1Limit = in1Ratio;
    double in2Limit = in1Ratio + in2Ratio;
       
    double randomDouble = java.lang.Math.random();
    // ###KEEP_DEBUG : some printings for the obsolete probabilisticChoice
    gui.println("in1TrustValue = " + in1TrustValue,java.awt.Color.orange);
    gui.println("in2TrustValue = " + in2TrustValue,java.awt.Color.orange);
    gui.println("outTrustValue = " + outTrustValue,java.awt.Color.orange);
    gui.println("sum = " + sum,java.awt.Color.orange);
    gui.println("in1Ratio = " + in1Ratio,java.awt.Color.orange);
    gui.println("in2Ratio = " + in2Ratio,java.awt.Color.orange);
    gui.println("outRatio = " + outRatio,java.awt.Color.orange);
    gui.println("in1Limit = " + in1Limit,java.awt.Color.orange);
    gui.println("in2Limit = " + in2Limit,java.awt.Color.orange);
    gui.println("randomDouble = " + randomDouble,java.awt.Color.orange);
    //end DEBUG
    if ((0 <= randomDouble) && (randomDouble < in1Limit))
      return new AgentInterface("IN", 1);
    else if ((in1Limit <= randomDouble) && (randomDouble < in2Limit))
      return new AgentInterface("IN", 2);
    else if ((in2Limit <= randomDouble) && (randomDouble < 1))
      return new AgentInterface("OUT");
    else {
      SystemPrinter.printSystem("ERROR : problem while generating random for probabilistic choice.");
      return null;
    }
  }
 
}