package epe.reorganization.messageProcessors.ncs;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract
						.ProposingCooperationInWithPotentialValueMessage;
	import epe.communication.messages.ncs.NCSInformedNeedInMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.ReorganizationMessageProcessor;
	import epe.reorganization.bestOfWorseManagement.WaitForProposalsTimerManager;

public class NCSInformedNeedInMessageProcessor {
  
	  protected Agent ownerAgent;
	  protected ReorganizationManager reorganizationManager;
	  protected ReorganizationMessageProcessor owner;
	  protected GUI gui;
	  protected WaitForProposalsTimerManager waitForProposalsTimerManager;
  
  public NCSInformedNeedInMessageProcessor(
  				 ReorganizationManager reorganizationManager, 
					 ReorganizationMessageProcessor owner, 
					 Agent ownerAgent, 
           GUI gui,
					 WaitForProposalsTimerManager waitForProposalsTimerManager) {
    this.reorganizationManager = reorganizationManager;
    this.ownerAgent = ownerAgent;
    this.owner = owner;
    this.gui = gui;
    this.waitForProposalsTimerManager = waitForProposalsTimerManager;
  }

  public void processNCSInformedNeedInMessage(Message receivedMessage) {
    NCSInformedNeedInMessage castMessage
        = (NCSInformedNeedInMessage)receivedMessage;
    String information = castMessage.getInformation();
    
    reorganizationManager
    	.initializeHigherLowerReorganizationIfNeeded(information,
    																							 castMessage.getPhase());
    
    if (ownerAgent.getAcquaintancesManager()
                  .isAnInOutAcquaintance(castMessage.getSender(), "OUT")) { 
      // sender is an OUT acquaintance
      // => sending its own NCSInformedNeedIn to try to satisfy its OutAcq
      // ###LOW_DEBUG  Need to be updated : higher/lower depending on the type of agent
    	// For now : only agent types that don't change Higher/lower goals
    	manageNeedInFromOutAcquaintance(information);
    }
    else { //sender is a NEUTRAL acquaintance
      manageNeedInFromNeutralAcquaintance(castMessage);      
    }
    owner.propagateNCS(receivedMessage);
  }
  
  protected void manageNeedInFromOutAcquaintance(String information) {
  	reorganizationManager.setHigherLowerGoal(information);
  	gui.println("Sender is an OUT acquaintance : if possessing INs => sending own needIN", 
  							java.awt.Color.orange);
  	if (reorganizationManager.readyToFireNCSs()) {
  		for (int i = 1; i <= ownerAgent.getNumberOfINs(); i++) {
  			if (ownerAgent.getAcquaintancesManager().possessingIN(i)) {
  				if (reorganizationManager.getNumberOfNeedInHLNCSWithoutAnswer() 
  						>= reorganizationManager.getMaxNumberOfNeedInHLNCSWithoutAnswer()) {
  					reorganizationManager.reInitializenumberOfNeedInHLNCSWithoutAnswer();
  					// Self doubt
  					ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
  				}
  				else {
  					gui.println("Sending its own NCSInformedNeedIn[" + information + 
				  							"][IN" + i + " : " +
				  							reorganizationManager.getInputThreshold(i) + 
				  							"] to all acquaintances.", java.awt.Color.orange);
  					NCSInformedNeedInMessage message 
	  					= new NCSInformedNeedInMessage(ownerAgent, information,
														  							 reorganizationManager.dataTypeNeeded("IN", i), 
																					 	 i, reorganizationManager.getInputThreshold(i),
																						 reorganizationManager
																						 	.getCurrentReorganizationPhase());
  					owner.sendMessageToAllAcquaintances(message);
  					
  					// Launch waitingForProposalsTimer
  					waitForProposalsTimerManager.waitForProposals(i);
  					
  					reorganizationManager.ncsFired();
  					ownerAgent.notifyNcsListeners();
  				}
  			}
  		}
  	}
  	else
  		gui.println("Not ready to fire NCSs", java.awt.Color.orange);
  }
  
  protected void manageNeedInFromNeutralAcquaintance(NCSInformedNeedInMessage castMessage) {
  	AgentInterface interfaceTypeAsked = castMessage.getInterfaceType();
  	gui.println("Processing message NeedIN : ["
				  			+ interfaceTypeAsked.getType() + ":"
				  			+ interfaceTypeAsked.getPosition() + ":"
				  			+ interfaceTypeAsked.getDataType() + "][Threshold = " 
				  			+ castMessage.getRequiredThreshold() + "] from ["
				  			+ castMessage.getSender().getAgentName() +"]");
  	if (reorganizationManager.isOriginallyRelevant(interfaceTypeAsked, 1)) {
  		// old test : only responding if higher/lower  
  		// new test : if possessPotentialoutput => proposal. The receiver will choose.
//  			&&(reorganizationManager.getPotentialOutput() > castMessage.getRequiredThreshold())
// 				&&(reorganizationManager.getHigherLowerGoal().equals("HIGHER"))
  		if (reorganizationManager.possessPotentialOutput()) {
	  		gui.println(ownerAgent.getAgentName()
					  				+ " is relevant concerning an agent needing : ["
					  				+ interfaceTypeAsked.getType() + ":"
					  				+ interfaceTypeAsked.getDataType()
					  				+ "][Threshold = " 
					  				+ castMessage.getRequiredThreshold() 
					  				+ "] | [PotentialOutput = "
					  				+ reorganizationManager.getPotentialOutput()
					  				+ "].", java.awt.Color.orange);
	  		Agent sender = castMessage.getSender();
	  		AgentInterface type 
	  			= new AgentInterface("IN", 
											  			 interfaceTypeAsked.getPosition(),
											  			 reorganizationManager
																.dataTypeNeeded("OUT", 
																								interfaceTypeAsked.getPosition()));
	  		ProposingCooperationInWithPotentialValueMessage message 
	  			= new ProposingCooperationInWithPotentialValueMessage(
	  							ownerAgent, 
	  							type, 
									reorganizationManager.getLastOutputValue(), 
									reorganizationManager.getHigherLowerGoal(),
									reorganizationManager.needOUT());
	  		ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
  		}
  		else {
  			gui.println(ownerAgent.getAgentName()
  					+ " has no PotentialOutput : cannot send proposal to needIN", 
  					java.awt.Color.red);
  		}
  	}
  	else {
  		gui.println(ownerAgent.getAgentName()
				  				+ " is not relevant concerning an agent needing : ["
				  				+ interfaceTypeAsked.getType() + ":"
				  				+ interfaceTypeAsked.getDataType()
				  				+ "][Threshold = " 
				  				+ castMessage.getRequiredThreshold() 
				  				+ "] | [PotentialOutput = "
				  				+ reorganizationManager.getPotentialOutput()
				  				+ "].", java.awt.Color.orange);
  	}
  }
  
}
