package epe.reorganization.messageProcessors.ncs;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.ProposingCooperationMessage;
	import epe.communication.messages.ncs.NCSNeedInMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.ReorganizationMessageProcessor;

public class NCSNeedInMessageProcessor {
  
  protected ReorganizationManager reorganizationManager;
  protected ReorganizationMessageProcessor owner;
  protected Agent ownerAgent;
  protected GUI gui;
  

  public NCSNeedInMessageProcessor(ReorganizationManager reorganizationManager,
                                   ReorganizationMessageProcessor owner,
                                   Agent ownerAgent, 
                                   GUI gui) {
    this.reorganizationManager = reorganizationManager;
    this.owner = owner;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
  }

  public void processNCSneedInMessage(Message receivedMessage) {
    // If the agent has the competences, it says that it is interested
    NCSNeedInMessage castMessage 
    	= (NCSNeedInMessage)receivedMessage;
    AgentInterface interfaceTypeAsked = castMessage.getInterfaceType();
    gui.println("Processing message NeedIN : ["
                + interfaceTypeAsked.getType() + ":"
                + interfaceTypeAsked.getPosition() + ":"
                + interfaceTypeAsked.getDataType() + "] from ["
                + castMessage.getSender().getAgentName() +"]");
    if (reorganizationManager.isRelevant(interfaceTypeAsked, 1)) {
      Agent sender = castMessage.getSender();
      AgentInterface type = new AgentInterface("IN", reorganizationManager
      																								.dataTypeNeeded("OUT", 1));
      ProposingCooperationMessage message 
      	= new ProposingCooperationMessage(ownerAgent, type, reorganizationManager.needOUT());
      ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
    }
    else {
      gui.println(ownerAgent.getAgentName()
                  + " is not relevant concerning an agent needing : ["
                  + interfaceTypeAsked.getType() + ":"
                  + interfaceTypeAsked.getDataType() + "]");
    if (reorganizationManager.isOriginallyRelevant(interfaceTypeAsked, 1)) {
          // The agent originally could have respondend => weakening of all the links
          // This is done because the agent could be wrong to insist for too long to
          // keep a certain type of data for its generic IN/OUT
          ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
        }
    }
    owner.propagateNCS(receivedMessage);
  }
  
}
