package epe.reorganization.messageProcessors.ncs;

	import epe.gui.GUI;
	import epe.agents.Agent;
	import epe.agents.acquaintances.AcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
	import epe.communication.Message;
	import epe.communication.messages.contract.ProposingCooperationMessage;
	import epe.communication.messages.contract
						.ProposingCooperationOutWithPrejudiceMessage;
	import epe.communication.messages.ncs.NCSNeedOutMessage;
	import epe.reorganization.ReorganizationManager;
	import epe.reorganization.ReorganizationMessageProcessor;


public class NCSNeedOutMessageProcessor {
  
	  protected ReorganizationManager reorganizationManager;
	  protected AcquaintancesManager acquaintancesManager;
	  protected ReorganizationMessageProcessor owner;
	  protected Agent ownerAgent;
	  protected GUI gui;
  

  public NCSNeedOutMessageProcessor(ReorganizationManager reorganizationManager,
                                    ReorganizationMessageProcessor owner, 
                                    Agent ownerAgent, 
                                    GUI gui) {
    this.reorganizationManager = reorganizationManager;
    this.owner = owner;
    this.ownerAgent = ownerAgent;
    this.gui = gui;
    this.acquaintancesManager = ownerAgent.getAcquaintancesManager();
  }

  public void processNCSneedOutMessage(Message receivedMessage) {
    // If the agents has the competences AND it is in need, it says that it is interrested
    // else if it is in a higher/lower reorganization phase, proposing with prejudice
  	//      else, it weakens its links.
    boolean relevant = false;
    NCSNeedOutMessage castMessage = (NCSNeedOutMessage)receivedMessage;
    Agent sender = castMessage.getSender();
    AgentInterface interfaceTypeAsked = castMessage.getInterfaceType();
    gui.println("Processing message NeedOUT : ["
                + interfaceTypeAsked.getType() + ":"
                + interfaceTypeAsked.getPosition() + ":"
                + interfaceTypeAsked.getDataType() + "] from ["
                + sender.getAgentName() + "]");
    
    // The agent scans its INs to see if it is relevant
    for (int i=1; i<=reorganizationManager.numberOfINs(); i++) {
      if (reorganizationManager.isRelevant(interfaceTypeAsked, i)) {
        relevant = true;
        AgentInterface type = new AgentInterface("OUT", 
        																				 reorganizationManager.dataTypeNeeded("IN", i));
        if (reorganizationManager.needIN(i)) { 
        	// Also in Need => The agents says that it is interrested
        	ProposingCooperationMessage message 
        		= new ProposingCooperationMessage(ownerAgent, type, i, true);
          gui.println("Proposing because in need too [IN" + i + "]", java.awt.Color.orange);
          ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
        }
        else {
        	// Agent already has its INs
          // Checks if it is in a Higher/Lower reorganization or a classic acqs search
          if (!reorganizationManager.isInHigherLowerReorganization()) {
	      		// Classic reorganization because Higher/Lower Feedback strategy is exclusive
	      		// The agent is not free => weakening of the links
	      		decreaseTrustValues(i);
	      	}
          else {
          	// higher/Lower reorganization phase
          	manageHigherLowerReorganization(i, castMessage, type, sender);
          	
          	// OLD version
//            // higher/Lower reorganization phase
//          	if (!reorganizationManager.isInPrejudice()) {
//          		manageHigherLowerReorganization(i, castMessage, type, sender);
//          	}
//          	else {
//          		// the agent is already in a situation where it solves a NCS
//          		// => do nothing (not to destroy that solving)
//          		gui.println("Already in Prejudice => do nothing.", java.awt.Color.red);
//          	}
          }
          
        }
      }
    }
    
    manageOriginalRelevance(relevant, interfaceTypeAsked);
    manageMultipleOUT(interfaceTypeAsked);
    owner.propagateNCS(receivedMessage);
  }
  
  protected void manageHigherLowerReorganization(int position, 
												 												 NCSNeedOutMessage castMessage, 
												 												 AgentInterface type, 
																								 Agent sender) {
  	int oldInput = reorganizationManager.getInputThreshold(position);
  	int newInput = castMessage.getPotentialValue();
  	String multiplicity = acquaintancesManager
												  	.getAcquaintances()
												  	.getAcquaintance("IN", position).getLinkMultiplicity();
  	gui.println("HIGHER/LOWER Reorganization : needOUT message[PotentialValue = " +
  							newInput + "] vs " +
  							"[InputThreshold = " + oldInput + 
  							" / Multiplicity = " + multiplicity + "]",
  							java.awt.Color.orange);
  	if (reorganizationManager.isBetterForGoal(newInput, oldInput)) {
  		// The cooperation the sender is looking for is also beneficial for itself
  		// => proposition With Prejudice = 0
  		// Also sending link multiplicity and if the sender is an inputThresholdAcq
  		int prejudice = 0;
  		ProposingCooperationOutWithPrejudiceMessage message 
	  		= new ProposingCooperationOutWithPrejudiceMessage(
	  						ownerAgent, type, position, prejudice, multiplicity,
								reorganizationManager.isPersonalGoalReached());
  		ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
  		gui.println("Proposing with 0-Prejudice [OUT" + position + "]", java.awt.Color.orange);
  	} 
  	else {
  		// The cooperation the sender is looking for is penalizing for itself
  		// => propositionWithPrejudice
  		// Also sending link multiplicity
  		int prejudice = calculatePrejudice(position, newInput);
  		ProposingCooperationOutWithPrejudiceMessage message 
	  		= new ProposingCooperationOutWithPrejudiceMessage(
	  				ownerAgent, type, position, prejudice, multiplicity,
						reorganizationManager.isPersonalGoalReached());
  		ownerAgent.fireMessage(message, sender, "NEUTRAL", ownerAgent);
  		gui.println("Proposing with Prejudice = " + prejudice 
  				+ "[OUT" + position + "]", java.awt.Color.orange);

  		// ###DEBUG  : test to see if it solves the unending reorg process
  		//decreasetrusValues(i);
  	}
  }
  
  protected int calculatePrejudice(int INPosition, int newInput) {
  	//	a) Prejudice calculated only from inputs
  	//  	int prejudice = java.lang.Math.abs(oldInput - newInput);
  	
  	// 	b) Prejudice calculated from the induced change to the output
  	int oldOutput = reorganizationManager.getOutputThreshold();
  	int IN1;
  	int IN2;
  	if (INPosition==1) {
  		IN1 = newInput;  
  		IN2 = reorganizationManager.getInputThreshold(2);
  	}
  	else { 
  		IN1 = reorganizationManager.getInputThreshold(1);  
  		IN2 = newInput;
  	}
  	int newOutput = reorganizationManager.calculatePotentialOutput(IN1, IN2);   
  	int prejudice = java.lang.Math.abs(oldOutput - newOutput);
  	
  	// ###DEBUG Prejudice calculus check
  	gui.println("IN1 = " + IN1, java.awt.Color.orange);
  	gui.println("IN2 = " + IN2, java.awt.Color.orange);
  	gui.println("newOutput = " + newOutput, java.awt.Color.orange);
  	// end DEBUG
  	
  	return prejudice;
  }
  
  protected void manageOriginalRelevance(boolean relevant,
											 									 AgentInterface interfaceTypeAsked) {
  	if (!relevant) {
  		gui.println(ownerAgent.getAgentName()
  				+ " is not relevant concerning an agent needing : ["
  				+ interfaceTypeAsked.getType() + ":"
  				+ interfaceTypeAsked.getDataType() + "]");
  		for (int i=1; i<=reorganizationManager.numberOfINs(); i++) {
  			if (reorganizationManager.isOriginallyRelevant(interfaceTypeAsked, i))
  				// The agent originally could have respondend => weakening of all the links
  				// This is done because the agent could be wrong to insist for too long to
  				// keep a certain type of data for its generic IN/OUT
  				ownerAgent.getAcquaintancesManager().decreaseAllTrustValues("needInOut");
  		}
  	}
  }
  
  protected void manageMultipleOUT(AgentInterface interfaceTypeAsked) {
  	//	The agent scans its OUTs : if it has more than one OUT, it may be blocking
  	//	some other agent's IN for nothing => link weakening
  	// (only done in normal reorganization)
  	if ((ownerAgent.getAcquaintancesManager().possessingMultipleOUT())
  			&& (reorganizationManager.isDataRelevant("OUT", 1, interfaceTypeAsked.getDataType()))
  			&& (!reorganizationManager.isInHigherLowerReorganization())) {
  		gui.println(ownerAgent.getAgentName()
  				+ " has multipleOUTs : decreasing OUT values", java.awt.Color.pink);
//  		AgentInterface typeToDecrease = new AgentInterface("OUT", 1);
//  		ownerAgent.getAcquaintancesManager().decreaseTrustValue(typeToDecrease, "multipleOut");
  		// ###DEBUG test : multipleOut => canceling of one outAcq.	
  		ownerAgent.getAcquaintancesManager().cancelOneOutAcquaintance();
  	}
  }
  
  protected void decreaseTrustValues(int position) {
	  // => decrease trustValue
	  AgentInterface typeToDecrease = new AgentInterface("IN", position);
	  ownerAgent.getAcquaintancesManager().decreaseTrustValue(typeToDecrease, "needInOut");
  }
  
}
