package epe.reorganization.potentialInOutAcquaintances;

	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.agentInterface.AgentInterface;

public class PotentialInOutAcquaintance extends Object {
  
	  protected Agent agent;
	  protected AgentInterface interfaceType;
	  protected boolean requested = false;
	  // proposedINnumber : for a proposingOUT, tells which IN is to be requested. 
	  protected int proposedINnumber=-1;
	  
	  // prejudice : The potentially suffered prejudice for an agent accepting a coop 
	  // he hasn't requested.
	  // -1 => not a proposal with prejudice but also in need
	  protected int prejudice = -1;
	  protected boolean inNeedInOutNcs;
	  // Whether the proposing agent is allready connected to a MULTIPLE link
	  // => breaking a MULTIPLE link will not cause a needIN/OUT NCS
	  protected String multiplicity = Acquaintance.UNKNOWN;
	  // Whether the proposing agent has managed to reach it Higher/lower goal
	  // => used when choosing a potentialOUT with prejudice strategy
	  protected boolean inGoalReachedState = false;
	  // Whether the proposing agent received its threshold from the agent
	  protected boolean isAnInputThresholdReceiverFromMe = false;

	public PotentialInOutAcquaintance(Agent agent, 
									  								AgentInterface interfaceType,
									  								boolean inNeedInOutNcs) {
	  this.agent = agent;
	  this.interfaceType = interfaceType;
	  this.inNeedInOutNcs = inNeedInOutNcs;
	}
	
	public PotentialInOutAcquaintance(Agent agent,
																		int proposedINnumber, 
																		AgentInterface interfaceType,
																		boolean inNeedInOutNcs) {
	  this.agent = agent;
	  this.proposedINnumber = proposedINnumber;
	  this.interfaceType = interfaceType;
	  this.inNeedInOutNcs = inNeedInOutNcs;
	}
	
	public PotentialInOutAcquaintance(Agent agent, 
	                                  AgentInterface interfaceType,
	                                  int prejudice,
									  								String multiplicity) {
	  this.agent = agent;
	  this.interfaceType = interfaceType;
	  this.prejudice = prejudice;
	  this.multiplicity = multiplicity;
	}
	
	public PotentialInOutAcquaintance(Agent agent,
																		int proposedINnumber, 
										  							AgentInterface interfaceType,
										  							int prejudice,
																		String multiplicity,
																		boolean inGoalReachedState,
																		boolean isAnInputThresholdReceiverFromMe) {
	  this.agent = agent;
	  this.proposedINnumber = proposedINnumber;
	  this.interfaceType = interfaceType;
	  this.prejudice = prejudice;
	  this.multiplicity = multiplicity;
	  this.inGoalReachedState = inGoalReachedState;
	  this.isAnInputThresholdReceiverFromMe = isAnInputThresholdReceiverFromMe;
	}
	
	public Agent getAgent() {
	  return agent; 
	}
	
	public AgentInterface getInterface() {
	  return interfaceType;
	}
	
	public String getInterfaceType() {
	  return interfaceType.getType();
	}
	
	public String getDataType() {
	  return interfaceType.getDataType();
	}
	
	public int getProposedINnumber() {
		return proposedINnumber;
	}
	
	public void setPrejudice(int prejudice) {
	  this.prejudice = prejudice;
	}
	
	public int getPrejudice() {
	  return prejudice;
	}

	public void setMultiplicity(String multiplicity) {
		this.multiplicity = multiplicity;
	}
	
	public String getMultiplicity() {
		return multiplicity;
	}
	
	public String getRequestedState() {
	  if (isRequested())
	    return "requested";
	  else
	    return "";
	}
	    
	public boolean isRequested() {
	  return requested;
	}
    
	public boolean isInNeedInOutNcs() {
		return inNeedInOutNcs;
	}
	
	public boolean isInGoalReachedState() {
		return inGoalReachedState;
	}

	public boolean isAnInputThresholdReceiverFromMe() {
		return isAnInputThresholdReceiverFromMe;
	}
	
	public void setRequested(boolean truthValue) {
	  requested = truthValue;
	}
    
	public boolean setDataType(String dataType) {
	  return (interfaceType.setDataType(dataType));
	}
 
	public boolean equals(PotentialInOutAcquaintance acq) {
	  return ((acq.getAgent().equals(agent))
	         &&(acq.getInterface().equals(interfaceType))
	  			 &&(acq.getProposedINnumber() == proposedINnumber));
	}

}