package epe.reorganization.potentialInOutAcquaintances.lastInOutAcquaintances;

	import epe.gui.SystemPrinter;
	import java.util.Vector;
	import epe.gui.GUI;
	import epe.reorganization.potentialInOutAcquaintances.PotentialInOutAcquaintance;
	import epe.agents.Agent;
	import epe.agents.agentInterface.AgentInterface;

public class LastInOutAcquaintances {
	  protected GUI gui;
	  protected Agent owner;
	  protected Vector<Object> acqsStore;
  
  public LastInOutAcquaintances(GUI gui, Agent owner) {
    this.gui = gui;
    this.owner = owner;
    acqsStore = new Vector<Object>();
  }

  public void reInitialize() {
    acqsStore.clear();
  }
  
  public void addLastInOutAcquaintance(LastInOutAcquaintance acq) {
    if (!(acq.getAgent().equals(owner)))
      if (!isALastInOutAcquaintance(acq))
        acqsStore.addElement(acq);
      else {
        removeLastInOutAcquaintance(acq.getInterface());
        acqsStore.addElement(acq);
      }
  }

  public void addLastInOutAcquaintance(Agent agentToAdd,
                                       String interfaceType,
                                       int position,
                                       String dataType) {
    addLastInOutAcquaintance(new LastInOutAcquaintance(agentToAdd, 
                                                       new AgentInterface(interfaceType,
                                                                          position,
                                                                          dataType)));
  }
  
  public LastInOutAcquaintance getLastInOutAcquaintance(int pos) {
    return ((LastInOutAcquaintance)acqsStore.elementAt(pos));
  }

  public LastInOutAcquaintance getLastInOutAcquaintance(Agent agent, 
																												AgentInterface interfaceType) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getInterface().equals(interfaceType))
         &(acq.getAgent().equals(agent)))
        return acq;
    }
    // Debug : si LastAcq non pr?sente
    SystemPrinter.printSystem("ERROR : " + owner.getAgentName());
    owner.errorHappened();
    SystemPrinter.printSystem(owner.getAgentName() 
    													+ " : ERROR : LastInOutAcquaintances :" 
    													+ " requested LastAcquaintance not found : ["
                       			  + agent.getAgentName() + " : "
                       				+ interfaceType.getType() + "]");
    owner.errorHappened();
    gui.println(owner.getAgentName() 
    						+ " : ERROR : LastInOutAcquaintances :" 
    						+ " requested LastAcquaintance not found : ["
                + agent.getAgentName() + " : "
                + interfaceType.getType() + "]", java.awt.Color.red);
    gui.print("LastAcquaintances list : ", java.awt.Color.red);
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      gui.print("[" + acq.getAgent().getAgentName() + " : "
                       + acq.getInterfaceType() + "] ", java.awt.Color.red);
    }
    gui.println("");
    gui.println("Examining LastAcq(end) : Size: " + acqsStore.size(), java.awt.Color.red);
    return null;
  }

 /*  Testing if the agent is already a lastInOutAcq
  public boolean isALastInOutAcquaintance(LastInOutAcquaintance acq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq2 = getLastInOutAcquaintance(i);
      if (acq.equals(acq2))
        return(true);
    }
    return (false);
  }*/
  
  // Testing if there is already a lastInOutAcq on the same interface
  protected boolean isALastInOutAcquaintance(LastInOutAcquaintance acq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq2 = getLastInOutAcquaintance(i);
      if (acq.getInterface().equals(acq2.getInterface()))
        return(true);
    }
    return (false);
  }

  public void removeLastInOutAcquaintance(PotentialInOutAcquaintance potentialAcq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(potentialAcq.getAgent()))
         &(acq.getInterface().equals(potentialAcq.getInterface())))
        acqsStore.removeElement(acq);
    }
  }
  
  public void removeLastInOutAcquaintance(AgentInterface interfaceType) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if (acq.getInterface().equals(interfaceType))
        acqsStore.removeElement(acq);
    }
  }

  public void removeLastInOutAcquaintance(Agent agent, AgentInterface interfaceType) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(agent))&(acq.getInterface().equals(interfaceType)))
        acqsStore.removeElement(acq);
    }
    gui.println(owner.getAgentName() + " : removed LastAcq : " + 
                "[" + agent.getAgentName() + "]", java.awt.Color.blue);
  }

  public void removeLastInOutAcquaintance(LastInOutAcquaintance acq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance lastAcq = getLastInOutAcquaintance(i);
      if (lastAcq.equals(acq))
        acqsStore.removeElement(lastAcq);
    }
    gui.println(owner.getAgentName() + " : removed LastAcq : " + 
                "[" + acq.getAgent().getAgentName() + "]", java.awt.Color.blue);
  }
  
  public void showAllLastAcquaintances() {
    LastInOutAcquaintance acq;
    if (acqsStore.size() == 0)
      gui.print("[  ]", java.awt.Color.cyan);
    for (int i=0; i< acqsStore.size(); i++) {
      acq = getLastInOutAcquaintance(i);
      gui.print("[" + acq.getAgent().getAgentName() 
                + ":" + acq.getInterfaceType() 
                + ":" + acq.getInterface().getPosition()
                + ":" + acq.getDataType()
                + "] ", java.awt.Color.cyan);
    }
    gui.print(".");
    gui.gotoNextLine();
  }
  
  public boolean isLastInOutAcquaintance(PotentialInOutAcquaintance potentialAcq) {
    boolean isLast = false;
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(potentialAcq.getAgent()))
         &(acq.getInterface().equals(potentialAcq.getInterface())))
        isLast = true;
    }
    return isLast;
  }
  
  public int getNumberOfRejects(PotentialInOutAcquaintance potentialAcq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(potentialAcq.getAgent()))
         &(acq.getInterface().equals(potentialAcq.getInterface())))
        return acq.getNumberOfRejects();
    }
    return -1;
  }
  
  public void increaseNumberOfRejects(PotentialInOutAcquaintance potentialAcq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(potentialAcq.getAgent()))
         &(acq.getInterface().equals(potentialAcq.getInterface())))
        acq.increaseNumberOfRejects();
    }
  }
  
  public void resetNumberOfRejects(PotentialInOutAcquaintance potentialAcq) {
    for (int i=0; i< acqsStore.size(); i++) {
      LastInOutAcquaintance acq = getLastInOutAcquaintance(i);
      if ((acq.getAgent().equals(potentialAcq.getAgent()))
         &(acq.getInterface().equals(potentialAcq.getInterface())))
        acq.resetNumberOfRejects();
    }
  }
  
}