package epe.reorganization.specificManagers;

	import java.util.Vector;
	
	import epe.gui.SystemPrinter;
	import epe.agents.Agent;
	import epe.agents.acquaintances.Acquaintance;
	import epe.agents.acquaintances.Acquaintances;
	import epe.agents.acquaintances.specificManagers.AgentFirstTimeAcquaintancesManager;
	import epe.agents.agentInterface.AgentInterface;
import epe.communication.messages.information.DataTypeChangedMessage;

public class AgentFirstTimeReorganizationManager 
       extends epe.reorganization.ReorganizationManager {

	  protected boolean needIN1 = false;
	  protected boolean needIN2 = false;
	  protected boolean needOUT = false;

  public AgentFirstTimeReorganizationManager(Agent ownerAgent) {
    super(ownerAgent);
  }

  public void checkInOut() {
    AgentFirstTimeAcquaintancesManager acquaintancesManager;
    acquaintancesManager = (AgentFirstTimeAcquaintancesManager)(ownerAgent.getAcquaintancesManager());

    if (!acquaintancesManager.possessingIN1())
      needIN1 = true;
    else
      needIN1 = false;
    if (!acquaintancesManager.possessingIN2())
      needIN2 = true;
    else
      needIN2 = false;
    if (!acquaintancesManager.possessingOUT())
      needOUT = true;
    else
      needOUT = false;
  }
  
  public int numberOfINs() {
    return 2 ;
  }
  
  public boolean needIN(int position) {
    if (position == 1)
      return needIN1;
    else if (position == 2)
      return needIN2;
    else {
      SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
      ownerAgent.errorHappened();
      gui.println("ERROR : needIN called with wrong argument position : "
                  + position, java.awt.Color.red); 
      return false;
    }
  }
  
  public boolean needIN() {
    return (needIN1 | needIN2);
  }

  public boolean needOUT() {
    return (needOUT);
  }

  protected void setNeedIN(int position, boolean truthValue) {
   if (position == 1)
     needIN1 = truthValue;
   else if (position == 2)
     needIN2 = truthValue;
   else {
    SystemPrinter.printSystem("ERROR : " + ownerAgent.getAgentName());
    ownerAgent.errorHappened();
    gui.println("ERROR : setNeedIN called with wrong argument position : "
                + position, java.awt.Color.red);  
   }
  }

  protected void setNeedOUT(boolean truthValue) {
    needOUT = truthValue;
  }
  
  public String dataTypeNeeded(String interfaceType, int position) {
    Vector<Object> positions;
    String dataType = "GENERIC";
    Acquaintances acqs = ownerAgent.getAcquaintancesManager().getAcquaintances();
    positions = acqs.findAcquaintance("IN");
    if (positions.size()!=0) {
      int pos = ((Integer)positions.elementAt(0)).intValue();
      dataType = acqs.getAcquaintance(pos).getDataType();
    }
    positions = ownerAgent.getAcquaintancesManager().getAcquaintances().findAcquaintance("OUT");
    if (positions.size()!=0) {
      int pos = ((Integer)positions.elementAt(0)).intValue();
      dataType = acqs.getAcquaintance(pos).getDataType();
    }
    return dataType;
  }

    
  protected String dataTypeNeededOriginally(String interfaceType, int position) {
    return "GENERIC";
  }
   
  public boolean isDataRelevant(String interfaceType, int position, String dataType) {
    return ((dataType.equals("GENERIC"))
           |(dataTypeNeeded(interfaceType, position).equals("GENERIC"))
           |(dataType.equals(dataTypeNeeded(interfaceType, position))));
  }
  
  protected String dataTypeNeededVSAsked(AgentInterface interfaceTypeAsked,
                                         int position) {
    String type = interfaceTypeAsked.getType();
    if (type.equals("IN"))
      return dataTypeNeeded("OUT", 1);
    else //(type.equals("OUT"))
      return dataTypeNeeded("IN", position);
  }
  
  protected String dataTypeNeededOriginallyVSAsked(AgentInterface interfaceTypeAsked,
                                                   int position) {
    String type = interfaceTypeAsked.getType();
    if (type.equals("IN"))
      return dataTypeNeededOriginally("OUT", 1);
    else //(type.equals("OUT"))
      return dataTypeNeededOriginally("IN", position);
  }
  
  public boolean isRelevant(AgentInterface interfaceTypeAsked, int position) {
    // The agent tests if it has the capacities to respond to the request
    String dataTypeAsked = interfaceTypeAsked.getDataType();
    String dataTypeNeeded = dataTypeNeededVSAsked(interfaceTypeAsked, position);
    if (dataTypeAsked.equals("GENERIC"))
      return true;
    if (dataTypeNeeded.equals("GENERIC"))
      return true;
    return dataTypeAsked.equals(dataTypeNeeded);
  }  

  public boolean isOriginallyRelevant(AgentInterface interfaceTypeAsked, int position) {
    // The agent tests if it originally had the capacities to respond to the request
    String dataTypeAsked = interfaceTypeAsked.getDataType();
    String dataTypeNeeded = dataTypeNeededOriginallyVSAsked(interfaceTypeAsked, position);
    if (dataTypeAsked.equals("GENERIC"))
      return true;
    if (dataTypeNeeded.equals("GENERIC"))
      return true;
    return dataTypeAsked.equals(dataTypeNeeded);
  }
  
  public void checkDataTypes() {
    String dataType = "GENERIC";
    Acquaintances acqs = ownerAgent.getAcquaintancesManager().getAcquaintances();
    Vector<Object> positions = acqs.findAcquaintance("IN");
    for (int i=0; i<positions.size(); i++) {
      int pos = ((Integer)positions.elementAt(i)).intValue();
      Acquaintance acq = acqs.getAcquaintance(pos);
      if (acq.getDataType() != "GENERIC")
        dataType = acq.getDataType();
    }
    
    positions = acqs.findAcquaintance("OUT");
    for (int i=0; i<positions.size(); i++) {
      int pos = ((Integer)positions.elementAt(i)).intValue();
      Acquaintance acq = acqs.getAcquaintance(pos);
      if (acq.getDataType() != "GENERIC")
        dataType = acq.getDataType();
    }
    
    if (dataType != "GENERIC") {
      positions = acqs.findAcquaintance("IN");
      for (int i=0; i<positions.size(); i++) {
        int pos = ((Integer)positions.elementAt(i)).intValue();
        Acquaintance acq = acqs.getAcquaintance(pos);
        if (acq.getDataType()=="GENERIC") {
          acq.setDataType(dataType);
          DataTypeChangedMessage message 
          	= new DataTypeChangedMessage(ownerAgent, 
                                         acq.getAgentInterface());
          ownerAgent.fireMessage(message, acq.getAgent(), acq.getInterfaceType(), ownerAgent);
          gui.println("Changed DataType to :[" + dataType + "]", java.awt.Color.red);                                                                
        }
      }
      positions = acqs.findAcquaintance("OUT");
      for (int i=0; i<positions.size(); i++) {
        int pos = ((Integer)positions.elementAt(i)).intValue();
        Acquaintance acq = acqs.getAcquaintance(pos);
        if (acq.getDataType()=="GENERIC") {
          acq.setDataType(dataType);
          DataTypeChangedMessage message 
          	= new DataTypeChangedMessage(ownerAgent, 
                                         acq.getAgentInterface());
          ownerAgent.fireMessage(message, acq.getAgent(), acq.getInterfaceType(), ownerAgent);
          gui.println("Changed DataType to :[" + dataType + "]", java.awt.Color.red);
        }
      }
    }
  }
  
  public void setSpecificPotentialOutput() {
    if (isDataRelevant("IN", 1, "INTEGER"))
      setPotentialOutput(getPotentialInput(1));
  }
  
  public int calculatePotentialOutput(int IN1, int IN2) {
  	// ###DEBUG AgentFirstTime reorg strategy => not defined yet for Higher/lower 
  	return (IN1);
  }

}