package epe.synchronization;

import java.util.Vector;

import epe.gui.GUI;
import epe.agents.Agent;
import epe.agents.acquaintances.Acquaintance;
import epe.agents.acquaintances.AcquaintancesManager;
import epe.agents.acquaintances.AcquaintancesUpdatedListener;
import epe.agents.agentInterface.AgentInterface;
import epe.communication.Message;
import epe.synchronization.waiters.Waiter;
import epe.synchronization.waiters.Waiters;


public class SynchronizationManager extends Object 
                                    implements AcquaintancesUpdatedListener {

  protected boolean outReady = false;
  protected AcquaintancesManager acquaintancesManager;
  protected Waiters waiters;
  protected Agent owner;
  protected GUI gui;
  
  /** Creates new SynchronizationManager */
  public SynchronizationManager(AcquaintancesManager acquaintancesManager, Agent owner) {
    this.acquaintancesManager = acquaintancesManager;
    this.owner = owner;
    gui = owner.getGUI();
    acquaintancesManager.addAcquaintancesUpdatedListener(this);
    initWaiters();
  }

  protected void initWaiters() {
    Vector<Object> positions;
    Waiter waiter;
    
    waiters  = new Waiters();
    positions = acquaintancesManager
                  .getAcquaintances()
                  .findAcquaintance(new AgentInterface("OUT",1));
    for (int i=0; i < positions.size(); i++) {
      waiter = new Waiter(acquaintancesManager
                            .getAcquaintances()
                            .getAcquaintance(((Integer)positions.elementAt(i)).intValue())
                            .getAgent());
      waiters.addWaiter(waiter);
    }
    gui.println("SynchronizationManager[ " + owner.getAgentName()
                       +" ] : number of waiters : " + waiters.size());
  }
  
  public void reInitialize() {
    waiters.setAllNotReady();
  }
  
  public void processReadyMessage(Message message) {
    Agent sender = message.getSender();
    waiters.setReady(sender);
    if (waitersAreReady())
      owner.startAgent();
  }
  
  public void sendToOutDone() {
    waiters.setAllNotReady();
  }
  
  public void acquaintancesUpdated(Acquaintance acq) {
//    gui.println(owner.getAgentName()
//                + " : synchronizationManager received update event :"
//                + acq.getAgent().getAgentName(), java.awt.Color.blue);
    if (!isAWaiter(acq.getAgent()) 
        && (acquaintancesManager.isAnInOutAcquaintance(acq.getAgent(), "OUT"))) {
      waiters.addWaiter(new Waiter(acq.getAgent()));
    }
    if (isAWaiter(acq.getAgent()) 
        && !(acquaintancesManager.isAnInOutAcquaintance(acq.getAgent(), "OUT"))) {
      waiters.removeWaiter(acq.getAgent());
    }
    if (!(acquaintancesManager.isAnInOutAcquaintance(acq.getAgent(), "IN"))) {
      owner.cooperationWithInAcquaintanceMayHaveBeenCancelled(acq);
    }
  }
  
  public boolean isAWaiter(Agent agent) {
    for (int i=0; i < waiters.size(); i++)
      if ((((Waiter)waiters.elementAt(i)).getAgent()).equals(agent))
        return true;
    return false;
  }
  
  public boolean waitersAreReady() {
    if (waiters.size() == 0)
      return false;
    for (int i=0; i < waiters.size(); i++)
      if (! ((Waiter)waiters.elementAt(i)).isReady())
        return false;
    return true;
  }
  
  public boolean aWaiterIsReady() {
    if (waiters.size() == 0)
      return false;
    for (int i=0; i < waiters.size(); i++)
      if (((Waiter)waiters.elementAt(i)).isReady())
        return true;
    return false;
  }
  
  public Waiters getReadyWaiters() {
    Waiters readyWaiters = new Waiters();
    
    for (int i=0; i < waiters.size(); i++)
      if (((Waiter)waiters.elementAt(i)).isReady())
        readyWaiters.addWaiter((Waiter)waiters.elementAt(i)); 
    return readyWaiters;
  }
  
  public void sendToReadyWaitersDone(Waiters readyWaiters) {
    for (int i=0; i < readyWaiters.size(); i++) {
      Waiter readyWaiter = (Waiter)readyWaiters.elementAt(i);
      waiters.setNotReady(readyWaiter);
    }
  }
  
  public void tellWhoIsNotReady() {
    boolean everybodyIsReady = true;
    if (waiters.size() == 0) {
      gui.println(owner.getAgentName()
                         + " : Hey, Nobody's ready to receive cause nobody's waiting.");
      everybodyIsReady = false;
    }
    else 
      for (int i=0; i < waiters.size(); i++)
        if (! ((Waiter)waiters.elementAt(i)).isReady()) {
          gui.println(owner.getAgentName() + " : "
                             + ((Waiter)waiters.elementAt(i)).getAgent().getAgentName()
                             + " : is not ready to receive.");
          everybodyIsReady = false;
        }
    if (everybodyIsReady)
      gui.println("Agent " + owner.getAgentName()
                         + " : Well, Everybody's ready to receive.");
  }
  
}